@@ -1,7 +1,7 @@
 /*
  *   cda - Command-line CD Audio Player
  *
- *   Copyright (C) 1993-1997  Ti Kan
+ *   Copyright (C) 1993-1998  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -20,11 +20,9 @@
  *
  */
 #ifndef LINT
-static char *_cda_c_ident_ = "@(#)cda.c	6.97 97/08/08";
+static char *_cda_c_ident_ = "@(#)cda.c	6.109 98/05/18";
 #endif
 
-#define _CDA_
-
 #include "common_d/appenv.h"
 #include "common_d/util.h"
 #include "common_d/patchlevel.h"
@@ -33,6 +31,19 @@ static char *_cda_c_ident_ = "@(#)cda.c	6.97 97/08/08";
 #include "cda_d/cda.h"
 #include "cda_d/visual.h"
 
+#if !defined(USE_TERMIOS) && !defined(USE_TERMIO) && !defined(USE_SGTTY)
+#define USE_TERMIOS	/* Make USE_TERMIOS the default if not specified */
+#endif
+
+#ifdef USE_TERMIOS
+#include <termios.h>
+#endif
+#ifdef USE_TERMIO
+#include <termio.h>
+#endif
+#ifdef USE_SGTTY
+#include <sgtty.h>
+#endif
 
 #define PGM_SEPCHAR	','			/* Program seq separator */
 #define MAX_PKT_RETRIES	200			/* Max retries reading pkt */
@@ -58,7 +69,8 @@ STATIC dev_t		cd_rdev;		/* CD device number */
 STATIC bool_t		visual,			/* Visual (curses) mode */
 			isdaemon,		/* Am I the daemon process */
 			stat_cont,		/* Continuous display status */
-			cddb;			/* CDDB entry exists */
+			cddb,			/* CDDB entry exists */
+			batch;			/* Non-interactive */
 STATIC FILE		*ttyfp;			/* /dev/tty */
 STATIC cddb_client_t	cddb_cldata;		/* Client info for libcddb */
 STATIC di_client_t	di_cldata;		/* Client info for libdi */
@@ -127,6 +139,62 @@ onintr(int signo)
 }
 
 
+/*
+ * cda_echo
+ *	Turn off/on echo mode.  It is assumed that the program starts
+ *	with echo mode enabled.
+ *
+ * Args:
+ *	doecho - Whether echo should be enabled
+ *
+ * Return:
+ *	Nothing.
+ */
+STATIC void
+cda_echo(FILE *fp, bool_t doecho)
+{
+	int			fd = fileno(fp);
+#ifdef USE_TERMIOS
+	struct termios		tio;
+
+	(void) tcgetattr(fd, &tio);
+
+	if (doecho)
+		tio.c_lflag |= ECHO;
+	else
+		tio.c_lflag &= ~ECHO;
+
+	(void) tcsetattr(fd, TCSADRAIN, &tio);
+#endif	/* USE_TERMIOS */
+
+#ifdef USE_TERMIO
+	struct termio		tio;
+
+	(void) ioctl(fd, TCGETA, &tio);
+
+	if (doecho)
+		tio.c_lflag |= ECHO;
+	else
+		tio.c_lflag &= ~ECHO;
+
+	(void) ioctl(fd, TCSETAW, &tio);
+#endif	/* USE_TERMIO */
+
+#ifdef USE_SGTTY
+	struct sgttyb		tio;
+
+	(void) ioctl(fd, TIOCGETP, &tio);
+
+	if (doecho)
+		tio.sg_flags |= ECHO;
+	else
+		tio.sg_flags &= ~ECHO;
+
+	(void) ioctl(fd, TIOCSETP, &tio);
+#endif	/* USE_SGTTY */
+}
+
+
 /*
  * cda_pgm_parse
  *	Parse the shuffle/program mode play sequence text string, and
@@ -143,7 +211,8 @@ STATIC bool_t
 cda_pgm_parse(curstat_t *s)
 {
 	int		i,
-			progtot;
+			progtot,
+			retcode;
 	char		*p,
 			*q,
 			*tmpbuf;
@@ -202,7 +271,7 @@ cda_pgm_parse(curstat_t *s)
 
 	if (progtot > 0) {
 		arg[0] = -progtot;
-		ret = cda_sendcmd(CDA_PROGRAM, arg);
+		ret = cda_sendcmd(CDA_PROGRAM, arg, &retcode);
 	}
 
 	MEM_FREE(arg);
@@ -228,14 +297,15 @@ cda_init(curstat_t *s)
 	char	*bdevname,
 		str[FILE_PATH_SZ * 2];
 
-	bdevname = basename(app_data.device);
+	bdevname = util_basename(app_data.device);
 
 	/* Get system-wide device-specific configuration parameters */
 	(void) sprintf(str, SYS_DSCFG_PATH, app_data.libdir, bdevname);
 	di_devspec_parmload(str, TRUE);
 
 	/* Get user device-specific configuration parameters */
-	(void) sprintf(str, USR_DSCFG_PATH, homedir(get_ouid()), bdevname);
+	(void) sprintf(str, USR_DSCFG_PATH, util_homedir(util_get_ouid()),
+		       bdevname);
 	di_devspec_parmload(str, FALSE);
 
 	/* Initialize the CD hardware */
@@ -254,7 +324,7 @@ cda_init(curstat_t *s)
 
 /*
  * cda_start
- *	Start the CD interface subsystem.
+ *	Secondary startup functions.
  *
  * Args:
  *	s - Pointer to the curstat_t structure.
@@ -265,6 +335,9 @@ cda_init(curstat_t *s)
 STATIC void
 cda_start(curstat_t *s)
 {
+	/* Start up libutil */
+	util_start();
+
 	/* Start up I/O interface */
 	di_start(s);
 
@@ -328,7 +401,7 @@ cda_daemonlock(void)
 				if (pid <= 0 ||
 				    (kill(pid, 0) < 0 && errno == ESRCH)) {
 					/* Pid died, steal its lockfile */
-					(void) unlink(dlock);
+					(void) UNLINK(dlock);
 				}
 				else {
 					/* Pid still running: clash */
@@ -548,7 +621,8 @@ cda_docmd(curstat_t *s, cdapkt_t *p)
 			j,
 			min,
 			sec;
-	word32_t	blkno;
+	word32_t	blkno,
+			discid;
 	bool_t		stopfirst,
 			offsets;
 	static time_t	cur_time,
@@ -709,8 +783,13 @@ cda_docmd(curstat_t *s, cdapkt_t *p)
 				break;
 			}
 
-			msftoblk((byte_t) p->arg[1], (byte_t) p->arg[2],
-				 0, &blkno, 0);
+			util_msftoblk(
+				(byte_t) p->arg[1],
+				(byte_t) p->arg[2],
+				0,
+				&blkno,
+				0
+			);
 
 			if (blkno >=
 			    (s->trkinfo[i+1].addr - s->trkinfo[i].addr)) {
@@ -724,9 +803,13 @@ cda_docmd(curstat_t *s, cdapkt_t *p)
 			s->cur_trk_frame = 0;
 
 			s->cur_tot_addr = s->trkinfo[i].addr + s->cur_trk_addr;
-			blktomsf(s->cur_tot_addr, &s->cur_tot_min,
-				 &s->cur_tot_sec, &s->cur_tot_frame,
-				 MSF_OFFSET(s));
+			util_blktomsf(
+				s->cur_tot_addr,
+				&s->cur_tot_min,
+				&s->cur_tot_sec,
+				&s->cur_tot_frame,
+				MSF_OFFSET
+			);
 		}
 
 		/* Start playback */
@@ -955,11 +1038,17 @@ cda_docmd(curstat_t *s, cdapkt_t *p)
 			    ((int) (s->level_right - s->level_left) / 2) + 50;
 		p->arg[5] = (word32_t) app_data.ch_route;
 		if (s->mode == MOD_NODISC || s->mode == MOD_BUSY)
-			p->arg[6] = 0;
+			discid = 0;
 		else
-			p->arg[6] = cddb_discid(s);
+			discid = cddb_discid(s);
 
+		if (discid != cur_db.discid)
+			cur_db.queryid = 0;
+
+		cur_db.discid = discid;
 		p->arg[7] = s->cur_disc;
+		p->arg[6] = discid;
+		p->arg[8] = cur_db.queryid;
 		break;
 
 	case CDA_TOC:
@@ -1031,6 +1120,10 @@ cda_docmd(curstat_t *s, cdapkt_t *p)
 
 		break;
 
+	case CDA_SETQUERYID:
+		cur_db.queryid = p->arg[0];
+		break;
+
 	case CDA_EXTINFO:
 		if (s->mode == MOD_NODISC || s->mode == MOD_BUSY) {
 			p->retcode = CDA_INVALID;
@@ -1429,7 +1522,8 @@ prn_debug(word32_t arg[])
 STATIC void
 usage(char *progname)
 {
-	(void) fprintf(errfp, "Usage: %s [-dev device] [-debug] command\n",
+	(void) fprintf(errfp,
+		"Usage: %s [-dev device] [-batch] [-debug] command\n",
 		progname);
 	(void) fprintf(errfp, "%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
 		"Valid commands are:\n",
@@ -1538,6 +1632,9 @@ cda_parse_args(int argc, char **argv, word32_t *cmd, word32_t arg[])
 		else if (strcmp(argv[i], "-debug") == 0) {
 			app_data.debug = TRUE;
 		}
+		else if (strcmp(argv[i], "-batch") == 0) {
+			batch = TRUE;
+		}
 		else if (strcmp(argv[i], "on") == 0) {
 			*cmd = CDA_ON;
 		}
@@ -1848,6 +1945,142 @@ cda_parse_args(int argc, char **argv, word32_t *cmd, word32_t arg[])
 }
 
 
+/*
+ * cda_inexact_select
+ *	Ask the user to select from a list of inexact CDDB matches.
+ *
+ * Args:
+ *	matchlist - List of inexact matches
+ *
+ * Return:
+ *	User selection number, or 0 for no selection.
+ */
+STATIC int
+cda_inexact_select(cddb_match_t *matchlist)
+{
+	int		i,
+			n;
+	cddb_match_t	*p;
+	char		input[64];
+
+	(void) fprintf(ttyfp, "\n%s\n%s\n\n",
+		"There were no exact matches in the CD database.",
+		"Close matches found:");
+
+	/* Display list */
+	for (p = matchlist, i = 1; p != NULL; p = p->next, i++)
+		(void) fprintf(ttyfp, "  %2d. %-73s\n", i, p->dtitle);
+
+	(void) fprintf(ttyfp, "  %2d. None of the above\n\n", i);
+
+	for (;;) {
+		(void) fprintf(ttyfp, "Choose one (1-%d): ", i);
+		if (fgets(input, 63, stdin) == NULL) {
+			(void) fprintf(ttyfp, "\n");
+			(void) fflush(ttyfp);
+			return 0;
+		}
+
+		input[strlen(input)-1] = '\0';
+
+		n = atoi(input);
+		if (n > 0 && n <= i)
+			break;
+	}
+
+	(void) fprintf(ttyfp, "\n");
+	(void) fflush(ttyfp);
+	return ((n == i) ? 0 : n);
+}
+
+/*
+ * cda_auth
+ *	Ask the user to enter user and password for proxy authorization.
+ *
+ * Args:
+ *	retrycnt - How many times has the user retried this
+ *
+ * Return:
+ *	0 for failure
+ *	1 for success
+ */
+STATIC int
+cda_auth(int retrycnt)
+{
+	char	*name,
+		*pass,
+		input[64];
+
+	if (retrycnt == 0)
+		(void) fprintf(ttyfp, "Proxy Authorization is required.\n");
+	else {
+		(void) fprintf(ttyfp, "Proxy Authorization failed.");
+
+		if (retrycnt < 3)
+			(void) fprintf(ttyfp, "  Try again.\n");
+		else {
+			(void) fprintf(ttyfp, "  Too many tries.\n\n");
+			(void) fflush(ttyfp);
+			return 0;
+		}
+	}
+
+	(void) fprintf(ttyfp, "%s\n\n",
+		"Please enter your proxy user name and password.");
+
+	/* Get user name */
+	(void) fprintf(ttyfp, "Username: ");
+	if (fgets(input, 63, stdin) == NULL) {
+		(void) fprintf(ttyfp, "\n");
+		(void) fflush(ttyfp);
+		return 0;
+	}
+	input[strlen(input)-1] = '\0';
+	if (input[0] == '\0')
+		return 0;
+
+	name = (char *) MEM_ALLOC(strlen(input) + 1);
+	if (name == NULL) {
+		CDA_FATAL(app_data.str_nomemory);
+		return 0;
+	}
+	(void) strcpy(name, input);
+	(void) memset(input, 0, 64);
+
+	/* Get password */
+	(void) fprintf(ttyfp, "Password: ");
+	cda_echo(ttyfp, FALSE);
+	if (fgets(input, 63, stdin) == NULL) {
+		cda_echo(ttyfp, TRUE);
+		(void) fprintf(ttyfp, "\n");
+		(void) fflush(ttyfp);
+		return 0;
+	}
+	cda_echo(ttyfp, TRUE);
+	input[strlen(input)-1] = '\0';
+	if (input[0] == '\0')
+		pass = NULL;
+	else {
+		pass = (char *) MEM_ALLOC(strlen(input) + 1);
+		if (pass == NULL) {
+			CDA_FATAL(app_data.str_nomemory);
+			return 0;
+		}
+		(void) strcpy(pass, input);
+		(void) memset(input, 0, 64);
+	}
+
+	cddb_set_auth(name, pass == NULL ? "" : pass);
+
+	MEM_FREE(name);
+	MEM_FREE(pass);
+
+	(void) fprintf(ttyfp, "\n");
+	(void) fflush(ttyfp);
+	return 1;
+}
+
+
 /***********************
  *   public routines   *
  ***********************/
@@ -1895,13 +2128,13 @@ cda_quit(curstat_t *s)
 
 		/* Remove FIFOs */
 		if (spipe[0] != '\0')
-			(void) unlink(spipe);
+			(void) UNLINK(spipe);
 		if (rpipe[0] != '\0')
-			(void) unlink(rpipe);
+			(void) UNLINK(rpipe);
 
 		/* Remove lock */
 		if (dlock[0] != '\0')
-			(void) unlink(dlock);
+			(void) UNLINK(dlock);
 	}
 #ifndef NOVISUAL
 	else {
@@ -1980,29 +2213,42 @@ cda_dbclear(curstat_t *s, bool_t reload)
  *	currently loaded disc, if available.
  *
  * Args:
- *	discid - The disc identifier
+ *	id - The disc identifier to be used for querying
+ *	ifunc - Function pointer to a inexact CDDB match handling function.
+ *	pfunc - Function pointer to a proxy authorization handling function.
+ *	depth - recursion depth
  *
  * Return:
  *	Nothing.
  */
 bool_t
-cda_dbload(word32_t discid)
+cda_dbload(
+	word32_t	id,
+	int		(*ifunc)(cddb_match_t *),
+	int		(*pfunc)(int),
+	int		depth
+)
 {
 	int		i,
+			n,
+			retcode,
 			ret;
+	word32_t	queryid;
 	curstat_t	*s;
+	cddb_match_t	*p,
+			*q;
 	word32_t	arg[CDA_NARGS];
 
 	s = &status;
 	(void) memset(arg, 0, CDA_NARGS * sizeof(word32_t));
 
-	if (!cda_sendcmd(CDA_TOC2, arg)) {
+	if (!cda_sendcmd(CDA_TOC2, arg, &retcode)) {
 		(void) printf("%s\n", errmsg);
 		return FALSE;
 	}
 
-	cur_db.discid = cur_db.queryid = discid;
-	s->tot_trks = discid & 0xff;
+	cur_db.queryid = id;
+	s->tot_trks = id & 0xff;
 
 	/* Update curstat with essential info */
 	for (i = 0; i < (int) s->tot_trks; i++) {
@@ -2012,12 +2258,12 @@ cda_dbload(word32_t discid)
 			s->trkinfo[i].frame
 		);
 
-		msftoblk(
+		util_msftoblk(
 			s->trkinfo[i].min,
 			s->trkinfo[i].sec,
 			s->trkinfo[i].frame,
 			&s->trkinfo[i].addr,
-			MSF_OFFSET(s)
+			MSF_OFFSET
 		);
 	}
 
@@ -2027,12 +2273,12 @@ cda_dbload(word32_t discid)
 		s->trkinfo[i].sec,
 		s->trkinfo[i].frame
 	);
-	msftoblk(
+	util_msftoblk(
 		s->trkinfo[i].min,
 		s->trkinfo[i].sec,
 		s->trkinfo[i].frame,
 		&s->trkinfo[i].addr,
-		MSF_OFFSET(s)
+		MSF_OFFSET
 	);
 
 	/* Load CD database entry */
@@ -2040,7 +2286,59 @@ cda_dbload(word32_t discid)
 		/* Failed to load database entry */
 		DBGPRN(errfp, "cddb_load: status=%d arg=%d\n",
 			CDDB_GET_STAT(ret), CDDB_GET_ARG(ret));
-		return FALSE;
+
+		switch (CDDB_GET_STAT(ret)) {
+		case MATCH_ERR:
+			/* Got a list of "inexact matches": Prompt user
+			 * for a selection.
+			 */
+			if (ifunc == NULL ||
+			    (n = (*ifunc)(cur_db.matchlist)) == 0) {
+				/* Clear match list */
+				for (p = q = cur_db.matchlist; p != NULL;
+				     p = q) {
+					q = p->next;
+					MEM_FREE(p->dtitle);
+					MEM_FREE(p);
+				}
+				cur_db.matchlist = NULL;
+				return FALSE;
+			}
+
+			/* Go find the match */
+			for (p = cur_db.matchlist, i=1; i < n; p = p->next, i++)
+				;
+			queryid = p->discid;
+
+			/* Clear match list */
+		        for (p = q = cur_db.matchlist; p != NULL; p = q) {
+				q = p->next;
+				MEM_FREE(p->dtitle);
+				MEM_FREE(p);
+			}
+			cur_db.matchlist = NULL;
+
+			/* Make daemon remember this ID */
+			(void) memset(arg, 0, CDA_NARGS * sizeof(word32_t));
+			arg[0] = queryid;
+			if (!cda_sendcmd(CDA_SETQUERYID, arg, &retcode)) {
+				(void) printf("%s\n", errmsg);
+				return FALSE;
+			}
+
+			/* Recursion */
+			return (cda_dbload(queryid, NULL, pfunc, depth + 1));
+
+		case AUTH_ERR:
+			if (pfunc == NULL || (n = (*pfunc)(depth)) == 0)
+				return FALSE;
+
+			/* Recursion */
+			return (cda_dbload(id, ifunc, pfunc, depth + 1));
+
+		default:
+			return FALSE;
+		}
 	}
 
 	/* Parse play order string and set the play order */
@@ -2058,13 +2356,14 @@ cda_dbload(word32_t discid)
  * Args:
  *	cmd - The command code
  *	arg - Command arguments
+ *	retcode - Return code
  *
  * Return:
  *	TRUE - success
  *	FALSE - failure
  */
 bool_t
-cda_sendcmd(word32_t cmd, word32_t arg[])
+cda_sendcmd(word32_t cmd, word32_t arg[], int *retcode)
 {
 	int		i;
 	cdapkt_t	p,
@@ -2079,6 +2378,7 @@ cda_sendcmd(word32_t cmd, word32_t arg[])
 	/* Send command packet */
 	if (!cda_sendpkt("CD audio", cda_sfd[1], &p)) {
 		errmsg = "\nCannot send packet to CD audio daemon.";
+		*retcode = -1;
 		return FALSE;
 	}
 
@@ -2086,6 +2386,7 @@ cda_sendcmd(word32_t cmd, word32_t arg[])
 		/* Get response packet */
 		if (!cda_getpkt("CD audio", cda_rfd[1], &r)) {
 			errmsg = "Cannot get packet from CD audio daemon.";
+			*retcode = -1;
 			return FALSE;
 		}
 
@@ -2098,6 +2399,7 @@ cda_sendcmd(word32_t cmd, word32_t arg[])
 		if (i >= MAX_PKT_RETRIES) {
 			errmsg =
 			"Retry overflow reading packet from CD audio daemon.";
+			*retcode = -1;
 			return FALSE;
 		}
 
@@ -2110,6 +2412,7 @@ cda_sendcmd(word32_t cmd, word32_t arg[])
 		if (r.rejcnt < MAX_PKT_RETRIES &&
 		    !cda_sendpkt("CD audio", cda_rfd[0], &r)) {
 			errmsg = "Cannot send packet to CD audio daemon.";
+			*retcode = -1;
 			return FALSE;
 		}
 
@@ -2117,12 +2420,14 @@ cda_sendcmd(word32_t cmd, word32_t arg[])
 		 * the order of packets in the pipe
 		 */
 		if ((rand() & 0x80) != 0)
-			delayms(100);
+			util_delayms(100);
 	}
 
 	/* Return args */
 	(void) memcpy(arg, r.arg, CDA_NARGS * sizeof(word32_t));
 
+	*retcode = r.retcode;
+
 	/* Check return code */
 	switch (r.retcode) {
 	case CDA_OK:
@@ -2216,19 +2521,7 @@ cda_daemon(curstat_t *s)
 	/* Make temporary directory, if needed */
 	(void) sprintf(errmsg, app_data.str_tmpdirerr, TEMP_DIR);
 	if (LSTAT(TEMP_DIR, &stbuf) < 0) {
-		if (errno == ENOENT) {
-			int	omask;
-
-			/* The permissions should be writable by all */
-			omask = umask(0);
-			if (mkdir(TEMP_DIR, 0777) < 0) {
-				(void) umask((mode_t) omask);
-				(void) fprintf(errfp, "%s\n", errmsg);
-				return 1;
-			}
-			(void) umask((mode_t) omask);
-		}
-		else {
+		if (!util_mkdir(TEMP_DIR, 0777)) {
 			(void) fprintf(errfp, "%s\n", errmsg);
 			return 1;
 		}
@@ -2293,7 +2586,7 @@ cda_daemon(curstat_t *s)
 		}
 
 		/* Try removing and re-creating the FIFO */
-		if (unlink(spipe) < 0) {
+		if (UNLINK(spipe) < 0) {
 			perror("CD audio: Cannot unlink old send pipe");
 			cda_quit(s);
 			return 1;
@@ -2312,7 +2605,7 @@ cda_daemon(curstat_t *s)
 		}
 
 		/* Try removing and re-creating the FIFO */
-		if (unlink(rpipe) < 0) {
+		if (UNLINK(rpipe) < 0) {
 			perror("CD audio: Cannot unlink old recv pipe");
 			cda_quit(s);
 			return 1;
@@ -2373,9 +2666,11 @@ cda_daemon(curstat_t *s)
 int
 main(int argc, char **argv)
 {
-	int		ret;
+	int		i,
+			retcode,
+			ret;
 	word32_t	cmd,
-			discid,
+			queryid,
 			*arg1,
 			*arg2;
 	struct stat	stbuf;
@@ -2385,6 +2680,17 @@ main(int argc, char **argv)
 			str[FILE_PATH_SZ * 2];
 	bool_t		loaddb = TRUE;
 
+
+	/* Hack: Aside from stdin, stdout, stderr, there shouldn't
+	 * be any other files open, so force-close them.  This is
+	 * necessary in case xmcd was inheriting any open file
+	 * descriptors from a parent process which is for the
+	 * CD-ROM device, and violating exclusive-open requirements
+	 * on some platforms.
+	 */
+	for (i = 3; i < 10; i++)
+		(void) close(i);
+
 	/* Initialize */
 	if ((ttypath = ttyname(2)) == NULL)
 		ttypath = "/dev/tty";
@@ -2458,12 +2764,13 @@ main(int argc, char **argv)
 
 	if (app_data.device != NULL) {
 		if ((int) strlen(app_data.device) >= FILE_PATH_SZ ||
-		    (int) strlen(basename(app_data.device)) >= FILE_BASE_SZ) {
+		    (int) strlen(util_basename(app_data.device)) >=
+		    FILE_BASE_SZ) {
 			CDA_FATAL(app_data.str_longpatherr);
 		}
 	}
 
-	hd = homedir(get_ouid());
+	hd = util_homedir(util_get_ouid());
 	if ((int) strlen(hd) >= FILE_PATH_SZ)
 		CDA_FATAL(app_data.str_longpatherr);
 
@@ -2493,7 +2800,7 @@ main(int argc, char **argv)
 
 	/* Initialize CD database services */
 	(void) strcpy(cddb_cldata.prog, PROGNAME);
-	(void) strcpy(cddb_cldata.user, loginname());
+	(void) strcpy(cddb_cldata.user, util_loginname());
 	cddb_cldata.isdemo = di_isdemo;
 	cddb_cldata.curstat_addr = curstat_addr;
 	cddb_cldata.fatal_msg = cda_fatal_msg;
@@ -2553,7 +2860,7 @@ main(int argc, char **argv)
 		switch (cmd) {
 		case CDA_ON:
 			/* Send command to cda daemon */
-			if (!cda_sendcmd(cmd, arg1)) {
+			if (!cda_sendcmd(cmd, arg1, &retcode)) {
 				(void) printf("%s\n", errmsg);
 				exit(2);
 			}
@@ -2567,7 +2874,7 @@ main(int argc, char **argv)
 
 		case CDA_OFF:
 			/* Send command to cda daemon */
-			if (!cda_sendcmd(cmd, arg1)) {
+			if (!cda_sendcmd(cmd, arg1, &retcode)) {
 				(void) printf("%s\n", errmsg);
 				exit(2);
 			}
@@ -2581,7 +2888,7 @@ main(int argc, char **argv)
 
 		case CDA_STATUS:
 			/* Send command to cda daemon */
-			if (!cda_sendcmd(cmd, arg1)) {
+			if (!cda_sendcmd(cmd, arg1, &retcode)) {
 				(void) printf("%s\n", errmsg);
 				exit(2);
 			}
@@ -2589,14 +2896,15 @@ main(int argc, char **argv)
 			if (RD_ARG_MODE(arg1[0]) == MOD_NODISC)
 				loaddb = TRUE;
 
-			discid = arg1[6];
+			cur_db.discid = arg1[6];
 
 			/* Load CD database entry */
 			if (loaddb &&
 			    RD_ARG_MODE(arg1[0]) != MOD_NODISC &&
-			    discid != 0) {
+			    cur_db.discid != 0) {
 				loaddb = FALSE;
-				cddb = cda_dbload(discid);
+				queryid = (arg1[8] != 0) ? arg1[8] : arg1[6];
+				cddb = cda_dbload(queryid, NULL, NULL, 0);
 			}
 
 			/* Print status */
@@ -2605,7 +2913,7 @@ main(int argc, char **argv)
 
 		default:	/* All other commands */
 			/* Check current status */
-			if (!cda_sendcmd(CDA_STATUS, arg2)) {
+			if (!cda_sendcmd(CDA_STATUS, arg2, &retcode)) {
 				(void) printf("%s\n", errmsg);
 				exit(2);
 			}
@@ -2613,17 +2921,27 @@ main(int argc, char **argv)
 			if (RD_ARG_MODE(arg2[0]) == MOD_NODISC)
 				loaddb = TRUE;
 
-			discid = arg2[6];
+			cur_db.discid = arg2[6];
 
 			/* Load CD database entry */
 			if (loaddb && RD_ARG_MODE(arg2[0]) != MOD_NODISC &&
-			    discid != 0) {
+			    cur_db.discid != 0) {
 				loaddb = FALSE;
-				cddb = cda_dbload(discid);
+				queryid = (arg2[8] != 0) ? arg2[8] : arg2[6];
+				if (cmd == CDA_TOC && !batch)
+					cddb = cda_dbload(
+						queryid,
+						cda_inexact_select,
+						cda_auth,
+						0
+					);
+				else
+					cddb = cda_dbload(queryid, NULL,
+							  NULL, 0);
 			}
 
 			/* Send command to cda daemon */
-			if (!cda_sendcmd(cmd, arg1)) {
+			if (!cda_sendcmd(cmd, arg1, &retcode)) {
 				(void) printf("%s\n", errmsg);
 				exit(2);
 			}
