@@ -1,7 +1,7 @@
 /*
  *   cda - Command-line CD Audio Player
  *
- *   Copyright (C) 1993-1997  Ti Kan
+ *   Copyright (C) 1993-1998  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -28,13 +28,11 @@
  */
 
 #ifndef LINT
-static char *_visual_c_ident_ = "@(#)visual.c	6.49 97/10/01";
+static char *_visual_c_ident_ = "@(#)visual.c	6.56 98/05/18";
 #endif
 
 #ifndef NOVISUAL
 
-#define _CDA_
-
 #include "common_d/appenv.h"
 #include "common_d/util.h"
 #include "common_d/patchlevel.h"
@@ -213,7 +211,6 @@ cda_wgetstr(WINDOW *win, char *inbuf, int max)
 			if (n > 0) {
 				p--;
 				n--;
-
 				/* Echo the effect of backspace */
 				getyx(win, y, x);
 				wmove(win, y, x-1);
@@ -222,7 +219,36 @@ cda_wgetstr(WINDOW *win, char *inbuf, int max)
 			}
 			break;
 
+		case KEY_UP:
 		case KEY_DOWN:
+		case '\036':	/* ctrl-^ */
+		case '\026':	/* ctrl-v */
+			if (n > 0) {
+				n--;
+				getyx(win, y, x);
+				wmove(win, y, x-1);
+				waddch(win, ' ');
+				wmove(win, y, x-1);
+			}
+
+			if (ch == KEY_UP || ch == '\036')
+				scroll_line--;
+			else
+				scroll_line++;
+
+			if (scroll_line < 0) {
+				beep();
+				scroll_line = 0;
+			}
+			if (scroll_line > (scroll_length - 1)) {
+				beep();
+				scroll_line = scroll_length - 1;
+			}
+
+			prefresh(info_win, scroll_line, 0, 0, 0,
+				 LINES-8, COLS-1);
+			break;
+
 		case '\n':
 		case '\r':
 			/* End-of-string */
@@ -248,38 +274,20 @@ cda_wgetstr(WINDOW *win, char *inbuf, int max)
 
 
 /*
- * cda_echo
+ * cda_v_echo
  *	Own versions of curses echo function.
  *
  * Args:
- *	None.
+ *	doecho - Whether to enable echo
  *
  * Return:
  *	Nothing.
  *
  */
 STATIC void
-cda_echo(void)
+cda_v_echo(bool_t doecho)
 {
-	echflag = TRUE;
-}
-
-
-/*
- * cda_noecho
- *	Own versions of curses noecho function.
- *
- * Args:
- *	None.
- *
- * Return:
- *	Nothing.
- *
- */
-STATIC void
-cda_noecho(void)
-{
-	echflag = FALSE;
+	echflag = doecho;
 }
 
 
@@ -306,37 +314,213 @@ cda_clrstatus(void)
 
 
 /*
- * cda_daemon_death
+ * cda_v_errhandler
  *	Handler function used when the daemon has exited.
  *
  * Args:
- *	None.
+ *	code - status code from cda_sendcmd
  *
  * Return:
  *	Nothing.
  */
 STATIC void
-cda_daemon_death(void)
+cda_v_errhandler(int code)
 {
+	if (code == CDA_OK || code == CDA_PARMERR)
+		/* Ignore these codes */
+		return;
+
 	/* Daemon died */
 	cda_clrstatus();
-	wprintw(status_win, "CD audio daemon pid=%d dev=%s exited.",
-		daemon_pid, app_data.device);
+
+	if (code < 0) {
+		wprintw(status_win, "CD audio daemon pid=%d dev=%s exited.",
+			daemon_pid, app_data.device);
+
+		ostat_on = stat_on;
+		stat_on = FALSE;
+
+		if (cda_sfd[1] >= 0) {
+			(void) close(cda_sfd[1]);
+			(void) close(cda_rfd[1]);
+			(void) close(cda_rfd[0]);
+			cda_sfd[1] = cda_rfd[1] = cda_rfd[0] = -1;
+		}
+
+		loaddb = TRUE;	/* Force reload of CDDB */
+		track = -2;	/* Force redisplay of version info */
+		scroll_line = 0;
+	}
+	else
+		wprintw(status_win, "%s", errmsg);
+
 	wrefresh(status_win);
+}
 
-	ostat_on = stat_on;
-	stat_on = FALSE;
 
-	if (cda_sfd[1] >= 0) {
-		(void) close(cda_sfd[1]);
-		(void) close(cda_rfd[1]);
-		(void) close(cda_rfd[0]);
-		cda_sfd[1] = cda_rfd[1] = cda_rfd[0] = -1;
+/*
+ * cda_v_inexact_select
+ *	Ask the user to select from a list of inexact CDDB matches.
+ *
+ * Args:
+ *	matchlist - List of inexact matches
+ *
+ * Return:
+ *	User selection number, or 0 for no selection.
+ */
+STATIC int
+cda_v_inexact_select(cddb_match_t *matchlist)
+{
+	int		i,
+			n,
+			x;
+	cddb_match_t	*p;
+	char		input[64];
+
+	wclear(info_win);
+	wmove(info_win, 0, 0);
+	wprintw(info_win, "\n%s\n%s\n\n",
+		"There were no exact matches in the CD database.",
+		"Close matches found:");
+
+	for (p = matchlist, i = 1; p != NULL; p = p->next, i++)
+		wprintw(info_win, "  %2d. %-73s\n", i, p->dtitle);
+
+	wprintw(info_win, "  %2d. None of the above\n\n", i);
+
+	scroll_line = 0;
+	prefresh(info_win, scroll_line, 0, 0, 0, LINES-8, COLS-1);
+	getyx(info_win, scroll_length, x);
+#ifdef LINT
+	x = x;	/* Get rid of "set but not used" lint warning */
+#endif
+	--scroll_length;
+
+	for (;;) {
+		cda_clrstatus();
+		wprintw(status_win, "Choose one (1-%d): ", i);
+		cda_v_echo(TRUE);
+		putp(cursor_normal);
+		wrefresh(status_win);
+		cda_wgetstr(status_win, input, 60);
+
+		n = atoi(input);
+		if (n > 0 && n <= i)
+			break;
+
+		beep();
 	}
 
-	loaddb = TRUE;	/* Force reload of CDDB */
-	track = -2;	/* Force redisplay of version info */
+	cda_v_echo(FALSE);
+	putp(cursor_invisible);
+
+	if (n == i)
+		return 0;
+
+	cda_clrstatus();
+	wprintw(status_win, "Querying CD database...");
+	wrefresh(status_win);
+	return (n);
+}
+
+
+/*
+ * cda_v_auth
+ *	Ask the user to enter name and password for proxy authorization.
+ *
+ * Args:
+ *	retrycnt - Number of times the user tried
+ *
+ * Return:
+ *	0 - failure
+ *	1 - success
+ */
+STATIC int
+cda_v_auth(int retrycnt)
+{
+	int	x;
+	char	*name,
+		*pass,
+		input[64];
+
+	wclear(info_win);
+	wmove(info_win, 0, 0);
+
+	if (retrycnt == 0)
+		wprintw(info_win, "Proxy Authorization is required.\n");
+	else {
+		wprintw(info_win, "Proxy Authorization failed.");
+
+		if (retrycnt < 3)
+			wprintw(info_win, "  Try again.\n");
+		else {
+			wprintw(info_win, "  Too many tries.\n");
+			scroll_line = 0;
+			prefresh(info_win, scroll_line, 0, 0, 0,
+				 LINES-8, COLS-1);
+			getyx(info_win, scroll_length, x);
+#ifdef LINT
+			x = x;	/* Get rid of "set but not used" lint warning */
+#endif
+			--scroll_length;
+			(void) sleep(2);
+			return 0;
+		}
+	}
+
+	wprintw(info_win, "%s\n\n",
+		"Please enter your proxy user name and password.");
+
 	scroll_line = 0;
+	prefresh(info_win, scroll_line, 0, 0, 0, LINES-8, COLS-1);
+	getyx(info_win, scroll_length, x);
+#ifdef LINT
+	x = x;	/* Get rid of "set but not used" lint warning */
+#endif
+	--scroll_length;
+
+	/* Get user name */
+	cda_clrstatus();
+	wprintw(status_win, "Username: ");
+	cda_v_echo(TRUE);
+	putp(cursor_normal);
+	wrefresh(status_win);
+	cda_wgetstr(status_win, input, 60);
+	if (input[0] == '\0')
+		return 0;
+
+	name = (char *) MEM_ALLOC(strlen(input) + 1);
+	if (name == NULL) {
+		CDA_FATAL(app_data.str_nomemory);
+		return 0;
+	}
+	strcpy(name, input);
+	(void) memset(input, 0, 64);
+
+	/* Get password */
+	cda_clrstatus();
+	wprintw(status_win, "Password: ");
+	cda_v_echo(FALSE);
+	wrefresh(status_win);
+	cda_wgetstr(status_win, input, 60);
+	if (input[0] == '\0')
+		pass = NULL;
+	else {
+		pass = (char *) MEM_ALLOC(strlen(input) + 1);
+		if (pass == NULL) {
+			CDA_FATAL(app_data.str_nomemory);
+			return 0;
+		}
+		strcpy(pass, input);
+		(void) memset(input, 0, 64);
+	}
+
+	cddb_set_auth(name, pass == NULL ? "" : pass);
+
+	MEM_FREE(name);
+	MEM_FREE(pass);
+
+	return 1;
 }
 
 
@@ -356,13 +540,15 @@ STATIC void
 cda_screen(int signo)
 {
 	word32_t	arg[CDA_NARGS],
-			discid = 0,
+			discid,
+			queryid,
 			discnum = 0,
 			astat0 = 0,
 			astat1 = 0;
 	int		x,
 			y,
 			i,
+			retcode,
 			trkno,
 			min,
 			sec,
@@ -464,8 +650,8 @@ cda_screen(int signo)
 			break;
 
 		arg[0] = 0;
-		if (!cda_sendcmd(CDA_ON, arg))
-			cda_daemon_death();
+		if (!cda_sendcmd(CDA_ON, arg, &retcode))
+			cda_v_errhandler(retcode);
 		else {
 			daemon_pid = (pid_t) arg[0];
 			cda_clrstatus();
@@ -480,8 +666,8 @@ cda_screen(int signo)
 	case TRUE:
 		/* Daemon running - get status and update display */
 		(void) memset(arg, 0, CDA_NARGS * sizeof(word32_t));
-		if (!cda_sendcmd(CDA_STATUS, arg)) {
-			cda_daemon_death();
+		if (!cda_sendcmd(CDA_STATUS, arg, &retcode)) {
+			cda_v_errhandler(retcode);
 
 			oastat0 = astat0;
 			oastat1 = astat1;
@@ -520,7 +706,14 @@ cda_screen(int signo)
 			wrefresh(status_win);
 
 			cda_dbclear(s, TRUE);
-			cddb = cda_dbload(discid);
+			cur_db.discid = discid;
+			queryid = (arg[8] != 0) ? arg[8] : arg[6];
+			cddb = cda_dbload(
+				queryid,
+				cda_v_inexact_select,
+				cda_v_auth,
+				0
+			);
 			cda_clrstatus();
 		}
 
@@ -649,8 +842,8 @@ cda_screen(int signo)
 			    RD_ARG_MODE(astat0) != MOD_NODISC &&
 			    !RD_ARG_SHUF(astat0)) {
 				arg[0] = 1;
-				if (!cda_sendcmd(CDA_PROGRAM, arg)) {
-					cda_daemon_death();
+				if (!cda_sendcmd(CDA_PROGRAM, arg, &retcode)) {
+					cda_v_errhandler(retcode);
 
 					oastat0 = astat0;
 					oastat1 = astat1;
@@ -731,6 +924,9 @@ cda_screen(int signo)
 				wprintw(status_win, "Count %u", rptcnt);
 
 			getyx(status_win, y, x);
+#ifdef LINT
+			y = y;	/* Get rid of "set but not used" lint warning */
+#endif
 			for (i = x; i < COLS-1; i++)
 				waddch(status_win, ' ');
 
@@ -745,8 +941,8 @@ cda_screen(int signo)
 			    RD_ARG_MODE(astat0) != MOD_NODISC &&
 			    !RD_ARG_SHUF(astat0)) {
 				arg[0] = 1;
-				if (!cda_sendcmd(CDA_PROGRAM, arg)) {
-					cda_daemon_death();
+				if (!cda_sendcmd(CDA_PROGRAM, arg, &retcode)) {
+					cda_v_errhandler(retcode);
 
 					oastat0 = astat0;
 					oastat1 = astat1;
@@ -846,13 +1042,13 @@ cda_screen(int signo)
 
 			switch ((int) stat_on) {
 			case TRUE:
-				if (cda_sendcmd(CDA_VERSION, arg)) {
+				if (cda_sendcmd(CDA_VERSION, arg, &retcode)) {
 					wprintw(info_win,
 						"CD audio daemon v%s\n",
 						(char *) arg);
 					break;
 				}
-				cda_daemon_death();
+				cda_v_errhandler(retcode);
 				/*FALLTHROUGH*/
 
 			case FALSE:
@@ -866,8 +1062,8 @@ cda_screen(int signo)
 
 			wprintw(info_win, "\nDevice: %s\n", app_data.device);
 			if (stat_on) {
-				if (!cda_sendcmd(CDA_DEVICE, arg)) {
-					cda_daemon_death();
+				if (!cda_sendcmd(CDA_DEVICE, arg, &retcode)) {
+					cda_v_errhandler(retcode);
 
 					oastat0 = astat0;
 					oastat1 = astat1;
@@ -902,8 +1098,8 @@ cda_screen(int signo)
 
 		arg[0] = 0;
 		arg[1] = track;
-		if (!cda_sendcmd(CDA_EXTINFO, arg)) {
-			cda_daemon_death();
+		if (!cda_sendcmd(CDA_EXTINFO, arg, &retcode)) {
+			cda_v_errhandler(retcode);
 
 			oastat0 = astat0;
 			oastat1 = astat1;
@@ -919,8 +1115,8 @@ cda_screen(int signo)
 
 			/* No database entry */
 			arg[0] = 0;
-			if (!cda_sendcmd(CDA_TOC, arg)) {
-				cda_daemon_death();
+			if (!cda_sendcmd(CDA_TOC, arg, &retcode)) {
+				cda_v_errhandler(retcode);
 
 				oastat0 = astat0;
 				oastat1 = astat1;
@@ -931,6 +1127,7 @@ cda_screen(int signo)
 				return;
 			}
 
+			cur_db.discid = arg[0];
 			ntrks = (int) (arg[0] & 0xff);
 
 			wprintw(info_win, "Disc %d  ID: %s %08x%s",
@@ -1054,6 +1251,7 @@ cda_screen(int signo)
 STATIC void
 cda_v_on_off(int inp, word32_t arg[])
 {
+	int		retcode;
 	curstat_t	*s = curstat_addr();
 
 	cda_clrstatus();
@@ -1080,7 +1278,7 @@ cda_v_on_off(int inp, word32_t arg[])
 		}
 	}
 	else {
-		if (cda_sendcmd(CDA_OFF, arg)) {
+		if (cda_sendcmd(CDA_OFF, arg, &retcode)) {
 			wprintw(status_win, "Stopping CD audio daemon...");
 			wrefresh(status_win);
 
@@ -1090,7 +1288,7 @@ cda_v_on_off(int inp, word32_t arg[])
 			} while (cda_daemon_alive());
 		}
 
-		cda_daemon_death();
+		cda_v_errhandler(retcode);
 	}
 }
 
@@ -1110,6 +1308,8 @@ cda_v_on_off(int inp, word32_t arg[])
 STATIC void
 cda_v_load_eject(int inp, word32_t arg[])
 {
+	int	retcode;
+
 	if (!stat_on || RD_ARG_MODE(arg[0]) == MOD_BUSY) {
 		beep();
 		return;
@@ -1117,8 +1317,8 @@ cda_v_load_eject(int inp, word32_t arg[])
 
 	arg[0] = (RD_ARG_MODE(arg[0]) == MOD_NODISC) ? 0 : 1;
 
-	if (!cda_sendcmd(CDA_DISC, arg))
-		cda_daemon_death();
+	if (!cda_sendcmd(CDA_DISC, arg, &retcode))
+		cda_v_errhandler(retcode);
 
 	loaddb = TRUE;	/* Force reload of CDDB */
 	refresh_sts = TRUE;
@@ -1141,6 +1341,7 @@ STATIC void
 cda_v_play_pause(int inp, word32_t arg[])
 {
 	word32_t	cmd;
+	int		retcode;
 
 	if (!stat_on ||
 	    RD_ARG_MODE(arg[0]) == MOD_BUSY ||
@@ -1155,8 +1356,8 @@ cda_v_play_pause(int inp, word32_t arg[])
 		cmd = CDA_PLAY;
 
 	arg[0] = 0;
-	if (!cda_sendcmd(cmd, arg))
-		cda_daemon_death();
+	if (!cda_sendcmd(cmd, arg, &retcode))
+		cda_v_errhandler(retcode);
 }
 
 
@@ -1175,6 +1376,8 @@ cda_v_play_pause(int inp, word32_t arg[])
 STATIC void
 cda_v_stop(int inp, word32_t arg[])
 {
+	int	retcode;
+
 	if (!stat_on ||
 	    RD_ARG_MODE(arg[0]) == MOD_BUSY ||
 	    RD_ARG_MODE(arg[0]) == MOD_NODISC) {
@@ -1188,8 +1391,8 @@ cda_v_stop(int inp, word32_t arg[])
 		return;
 	}
 
-	if (!cda_sendcmd(CDA_STOP, arg))
-		cda_daemon_death();
+	if (!cda_sendcmd(CDA_STOP, arg, &retcode))
+		cda_v_errhandler(retcode);
 
 	refresh_sts = TRUE;
 }
@@ -1210,6 +1413,8 @@ cda_v_stop(int inp, word32_t arg[])
 STATIC void
 cda_v_lock(int inp, word32_t arg[])
 {
+	int	retcode;
+
 	if (!stat_on ||
 	    RD_ARG_MODE(arg[0]) == MOD_BUSY ||
 	    RD_ARG_MODE(arg[0]) == MOD_NODISC) {
@@ -1219,8 +1424,8 @@ cda_v_lock(int inp, word32_t arg[])
 
 	arg[0] = RD_ARG_LOCK(arg[0]) ? 0 : 1;
 
-	if (!cda_sendcmd(CDA_LOCK, arg))
-		cda_daemon_death();
+	if (!cda_sendcmd(CDA_LOCK, arg, &retcode))
+		cda_v_errhandler(retcode);
 }
 
 
@@ -1241,7 +1446,8 @@ cda_v_shufprog(int inp, word32_t arg[])
 {
 	word32_t	astat0 = arg[0];
 	int		i,
-			j;
+			j,
+			retcode;
 	char		inbuf[80],
 			*p;
 
@@ -1261,24 +1467,24 @@ cda_v_shufprog(int inp, word32_t arg[])
 
 	/* See if program on */
 	arg[0] = 1;
-	if (!cda_sendcmd(CDA_PROGRAM, arg)) {
-		cda_daemon_death();
+	if (!cda_sendcmd(CDA_PROGRAM, arg, &retcode)) {
+		cda_v_errhandler(retcode);
 		return;
 	}
 
 	/* If neither program nor shuffle, set shuffle */
 	if (!RD_ARG_SHUF(astat0) && arg[0] == 0) {
 		arg[0] = 1;
-		if (!cda_sendcmd(CDA_SHUFFLE, arg)) {
-			cda_daemon_death();
+		if (!cda_sendcmd(CDA_SHUFFLE, arg, &retcode)) {
+			cda_v_errhandler(retcode);
 			return;
 		}
 	}
 	else if (RD_ARG_SHUF(astat0)) {
 		/* If shuffle, turn it off and prompt for program */
 		arg[0] = 0;
-		if (!cda_sendcmd(CDA_SHUFFLE, arg)) {
-			cda_daemon_death();
+		if (!cda_sendcmd(CDA_SHUFFLE, arg, &retcode)) {
+			cda_v_errhandler(retcode);
 			return;
 		}
 
@@ -1292,7 +1498,7 @@ cda_v_shufprog(int inp, word32_t arg[])
 
 		cda_clrstatus();
 		wprintw(status_win, "Program: ");
-		cda_echo();
+		cda_v_echo(TRUE);
 		putp(cursor_normal);
 		wrefresh(status_win);
 
@@ -1327,13 +1533,13 @@ cda_v_shufprog(int inp, word32_t arg[])
 			}
 
 			arg[0] = (word32_t) -j;
-			if (!cda_sendcmd(CDA_PROGRAM, arg)) {
+			if (!cda_sendcmd(CDA_PROGRAM, arg, &retcode)) {
 				errmsg = NULL;
 				beep();
 			}
 		}
 
-		cda_noecho();
+		cda_v_echo(FALSE);
 		putp(cursor_invisible);
 
 		refresh_sts = TRUE;
@@ -1341,8 +1547,8 @@ cda_v_shufprog(int inp, word32_t arg[])
 	else {
 		/* Program is on - reset it */
 		arg[0] = 0;
-		if (!cda_sendcmd(CDA_PROGRAM, arg)) {
-			cda_daemon_death();
+		if (!cda_sendcmd(CDA_PROGRAM, arg, &retcode)) {
+			cda_v_errhandler(retcode);
 			return;
 		}
 		refresh_sts = TRUE;
@@ -1365,14 +1571,16 @@ cda_v_shufprog(int inp, word32_t arg[])
 STATIC void
 cda_v_repeat(int inp, word32_t arg[])
 {
+	int	retcode;
+
 	if (!stat_on) {
 		beep();
 		return;
 	}
 
 	arg[0] = RD_ARG_REPT(arg[0]) ? 0 : 1;
-	if (!cda_sendcmd(CDA_REPEAT, arg))
-		cda_daemon_death();
+	if (!cda_sendcmd(CDA_REPEAT, arg, &retcode))
+		cda_v_errhandler(retcode);
 }
 
 
@@ -1390,14 +1598,16 @@ cda_v_repeat(int inp, word32_t arg[])
 STATIC void
 cda_v_disc(int inp, word32_t arg[])
 {
+	int	retcode;
+
 	if (!stat_on) {
 		beep();
 		return;
 	}
 
 	arg[0] = (inp == 'd') ? 2 : 3;
-	if (!cda_sendcmd(CDA_DISC, arg))
-		cda_daemon_death();
+	if (!cda_sendcmd(CDA_DISC, arg, &retcode))
+		cda_v_errhandler(retcode);
 
 	loaddb = TRUE;
 }
@@ -1417,6 +1627,8 @@ cda_v_disc(int inp, word32_t arg[])
 STATIC void
 cda_v_track(int inp, word32_t arg[])
 {
+	int	retcode;
+
 	if (!stat_on ||
 	    RD_ARG_MODE(arg[0]) == MOD_BUSY ||
 	    RD_ARG_MODE(arg[0]) == MOD_NODISC ||
@@ -1426,8 +1638,8 @@ cda_v_track(int inp, word32_t arg[])
 	}
 
 	arg[0] = (inp == KEY_LEFT || inp == 'C') ? 0 : 1;
-	if (!cda_sendcmd(CDA_TRACK, arg))
-		cda_daemon_death();
+	if (!cda_sendcmd(CDA_TRACK, arg, &retcode))
+		cda_v_errhandler(retcode);
 }
 
 
@@ -1445,6 +1657,8 @@ cda_v_track(int inp, word32_t arg[])
 STATIC void
 cda_v_index(int inp, word32_t arg[])
 {
+	int	retcode;
+
 	if (!stat_on ||
 	    RD_ARG_MODE(arg[0]) == MOD_BUSY ||
 	    RD_ARG_MODE(arg[0]) == MOD_NODISC) {
@@ -1453,8 +1667,8 @@ cda_v_index(int inp, word32_t arg[])
 	}
 
 	arg[0] = (inp == '<') ? 0 : 1;
-	if (!cda_sendcmd(CDA_INDEX, arg))
-		cda_daemon_death();
+	if (!cda_sendcmd(CDA_INDEX, arg, &retcode))
+		cda_v_errhandler(retcode);
 }
 
 
@@ -1472,14 +1686,16 @@ cda_v_index(int inp, word32_t arg[])
 STATIC void
 cda_v_volume(int inp, word32_t arg[])
 {
+	int	retcode;
+
 	if (!stat_on) {
 		beep();
 		return;
 	}
 
 	arg[0] = 0;
-	if (!cda_sendcmd(CDA_VOLUME, arg)) {
-		cda_daemon_death();
+	if (!cda_sendcmd(CDA_VOLUME, arg, &retcode)) {
+		cda_v_errhandler(retcode);
 		return;
 	}
 
@@ -1497,8 +1713,8 @@ cda_v_volume(int inp, word32_t arg[])
 	}
 
 	arg[0] = 1;
-	if (!cda_sendcmd(CDA_VOLUME, arg)) {
-		cda_daemon_death();
+	if (!cda_sendcmd(CDA_VOLUME, arg, &retcode)) {
+		cda_v_errhandler(retcode);
 		return;
 	}
 }
@@ -1518,14 +1734,16 @@ cda_v_volume(int inp, word32_t arg[])
 STATIC void
 cda_v_balance(int inp, word32_t arg[])
 {
+	int	retcode;
+
 	if (!stat_on) {
 		beep();
 		return;
 	}
 
 	arg[0] = 0;
-	if (!cda_sendcmd(CDA_BALANCE, arg)) {
-		cda_daemon_death();
+	if (!cda_sendcmd(CDA_BALANCE, arg, &retcode)) {
+		cda_v_errhandler(retcode);
 		return;
 	}
 
@@ -1543,8 +1761,8 @@ cda_v_balance(int inp, word32_t arg[])
 	}
 
 	arg[0] = 1;
-	if (!cda_sendcmd(CDA_BALANCE, arg))
-		cda_daemon_death();
+	if (!cda_sendcmd(CDA_BALANCE, arg, &retcode))
+		cda_v_errhandler(retcode);
 }
 
 
@@ -1563,6 +1781,8 @@ cda_v_balance(int inp, word32_t arg[])
 STATIC void
 cda_v_route(int inp, word32_t arg[])
 {
+	int	retcode;
+
 	if (!stat_on) {
 		beep();
 		return;
@@ -1573,8 +1793,8 @@ cda_v_route(int inp, word32_t arg[])
 
 	arg[0] = 1;
 	arg[1] = route;
-	if (!cda_sendcmd(CDA_ROUTE, arg))
-		cda_daemon_death();
+	if (!cda_sendcmd(CDA_ROUTE, arg, &retcode))
+		cda_v_errhandler(retcode);
 }
 
 
@@ -1642,7 +1862,7 @@ cda_v_repaint(int inp, word32_t arg[])
 STATIC void
 cda_v_scroll(int inp, word32_t arg[])
 {
-	if (inp == KEY_UP || inp == '^')
+	if (inp == KEY_UP || inp == '^' || inp == '\036')
 		scroll_line--;
 	else
 		scroll_line++;
@@ -1675,6 +1895,8 @@ cda_v_scroll(int inp, word32_t arg[])
 STATIC void
 cda_v_debug(int inp, word32_t arg[])
 {
+	int	retcode;
+
 	if (!stat_on) {
 		beep();
 		return;
@@ -1682,9 +1904,9 @@ cda_v_debug(int inp, word32_t arg[])
 
 	/* Query current debug mode */
 	arg[0] = 0;
-	if (!cda_sendcmd(CDA_DEBUG, arg)) {
+	if (!cda_sendcmd(CDA_DEBUG, arg, &retcode)) {
 		beep();
-		cda_daemon_death();
+		cda_v_errhandler(retcode);
 		return;
 	}
 
@@ -1695,9 +1917,9 @@ cda_v_debug(int inp, word32_t arg[])
 	else
 		arg[1] = 0;
 
-	if (!cda_sendcmd(CDA_DEBUG, arg)) {
+	if (!cda_sendcmd(CDA_DEBUG, arg, &retcode)) {
 		beep();
-		cda_daemon_death();
+		cda_v_errhandler(retcode);
 		return;
 	}
 
@@ -1724,7 +1946,8 @@ cda_v_dirtrk(int inp, word32_t arg[])
 {
 	int	i,
 		mins,
-		secs;
+		secs,
+		retcode;
 	char	inbuf[80],
 		*p;
 
@@ -1740,7 +1963,7 @@ cda_v_dirtrk(int inp, word32_t arg[])
 
 	cda_clrstatus();
 	wprintw(status_win, "Track n [mins secs] : ");
-	cda_echo();
+	cda_v_echo(TRUE);
 	putp(cursor_normal);
 	wrefresh(status_win);
 
@@ -1766,11 +1989,11 @@ cda_v_dirtrk(int inp, word32_t arg[])
 		arg[0] = i;
 		arg[1] = mins;
 		arg[2] = secs;
-		if (!cda_sendcmd(CDA_PLAY, arg))
-			cda_daemon_death();
+		if (!cda_sendcmd(CDA_PLAY, arg, &retcode))
+			cda_v_errhandler(retcode);
 	}
 
-	cda_noecho();
+	cda_v_echo(FALSE);
 	putp(cursor_invisible);
 	refresh_sts = TRUE;
 }
@@ -1924,6 +2147,8 @@ STATIC keytbl_t	ktbl[] = {
 	{ KEY_DOWN,	cda_v_scroll		},
 	{ '^',		cda_v_scroll		},
 	{ 'v',		cda_v_scroll		},
+	{ '\036',	cda_v_scroll		},
+	{ '\026',	cda_v_scroll		},
 	{ 'V',		cda_v_scroll		},
 	{ 'd',		cda_v_debug		},
 	{ '\0',		cda_v_dirtrk		},
@@ -1946,7 +2171,8 @@ cda_visual(void)
 {
 	word32_t	arg[CDA_NARGS];
 	int		inp,
-			i;
+			i,
+			retcode;
 	curstat_t	*s = curstat_addr();
 
 #if defined(SIGTSTP) && defined(SIGCONT)
@@ -2000,8 +2226,8 @@ cda_visual(void)
 		(void) alarm(0);
 
 		/* Get current status */
-		if (stat_on && !cda_sendcmd(CDA_STATUS, arg))
-			cda_daemon_death();
+		if (stat_on && !cda_sendcmd(CDA_STATUS, arg, &retcode))
+			cda_v_errhandler(retcode);
 
 		/* Execute command function */
 		for (i = 0; ktbl[i].keyfunc != NULL; i++) {
