@@ -1,7 +1,7 @@
 /*
  *   cddb - CD Database Management Library
  *
- *   Copyright (C) 1993-1997  Ti Kan
+ *   Copyright (C) 1993-1998  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
  *
  */
 #ifndef LINT
-static char *_cddb_c_ident_ = "@(#)cddb.c	6.163 97/11/05";
+static char *_cddb_c_ident_ = "@(#)cddb.c	6.177 98/05/12";
 #endif
 
 #ifdef __VMS
@@ -182,7 +182,9 @@ cddb_line_filter(char *str)
 STATIC char *
 cddb_strcat(char *s1, char *s2)
 {
+	int	n;
 	char	*cp = s1;
+	bool_t	proc_slash;
 
 	if (s1 == NULL || s2 == NULL)
 		return NULL;
@@ -190,22 +192,35 @@ cddb_strcat(char *s1, char *s2)
 	/* Concatenate two strings, with special handling for newline
 	 * and tab characters.
 	 */
-	for (s1 += strlen(s1); *s2 != '\0'; s1++, s2++) {
+	proc_slash = FALSE;
+	n = strlen(s1);
+	s1 += n;
+	if (n > 0 && *(s1 - 1) == '\\') {
+		proc_slash = TRUE;	/* Handle broken escape sequences */
+		s1--;
+	}
+
+	for (; *s2 != '\0'; s1++, s2++) {
 		if (*s2 == '\\') {
-			switch (*(s2 + 1)) {
+			proc_slash = TRUE;
+			s2++;
+		}
+
+		if (proc_slash) {
+			proc_slash = FALSE;
+
+			switch (*s2) {
 			case 'n':
 				*s1 = '\n';
-				s2++;
 				break;
 			case 't':
 				*s1 = '\t';
-				s2++;
 				break;
 			case '\\':
 				*s1 = '\\';
-				s2++;
 				break;
 			default:
+				*s1++ = '\\';
 				*s1 = *s2;
 				break;
 			}
@@ -219,52 +234,6 @@ cddb_strcat(char *s1, char *s2)
 }
 
 
-/*
- * cddb_strcasecmp
- *	Compare two strings a la strcmp(), except it is case-insensitive.
- *
- * Args:
- *	s1 - The first text string.
- *	s2 - The second text string.
- *
- * Return:
- *	Compare value.  See strcmp(3).
- */
-STATIC int
-cddb_strcasecmp(char *s1, char *s2)
-{
-	char	*buf1,
-		*buf2,
-		*p;
-	int	ret;
-
-	if (s1 == NULL || s2 == NULL)
-		return 0;
-
-	/* Allocate tmp buffers */
-	if ((buf1 = (char *) MEM_ALLOC(strlen(s1)+1)) == NULL ||
-	    (buf2 = (char *) MEM_ALLOC(strlen(s2)+1)) == NULL) {
-		CDDB_FATAL(app_data.str_nomemory);
-		return 0;
-	}
-
-	/* Convert both strings to lower case and store in tmp buffer */
-	for (p = buf1; *s1 != '\0'; s1++, p++)
-		*p = (char) ((isupper(*s1)) ? tolower(*s1) : *s1);
-	*p = '\0';
-	for (p = buf2; *s2 != '\0'; s2++, p++)
-		*p = (char) ((isupper(*s2)) ? tolower(*s2) : *s2);
-	*p = '\0';
-
-	ret = strcmp(buf1, buf2);
-
-	MEM_FREE(buf1);
-	MEM_FREE(buf2);
-
-	return (ret);
-}
-
-
 /*
  * cddb_http_xlat
  *	String translation for special meta-characters on the HTTP command
@@ -319,14 +288,11 @@ cddb_http_xlat(char *s1, char *s2)
 STATIC int
 cddb_sum(int n)
 {
-	char	buf[12],
-		*p;
-	int	ret = 0;
+	int	ret;
 
 	/* For backward compatibility this algorithm must not change */
-	(void) sprintf(buf, "%u", n);
-	for (p = buf; *p != '\0'; p++)
-		ret += (*p - '0');
+	for (ret = 0; n > 0; n /= 10)
+		ret += n % 10;
 
 	return (ret);
 }
@@ -379,7 +345,6 @@ cddb_waitio(int fd, bool_t do_workproc, int tmout)
 				/* Timeout */
 				DBGPRN(errfp, "Timed out waiting for data.\n");
 				ret = -1;
-				errno = ETIME;
 			}
 		}
 
@@ -388,6 +353,7 @@ cddb_waitio(int fd, bool_t do_workproc, int tmout)
 
 	} while (ret == 0);
 
+	errno = 0;
 	return (ret > 0 ? TRUE : FALSE);
 #else
 	return TRUE;
@@ -494,7 +460,6 @@ cddb_gets(cddb_file_t *fp, char *buf, int len, bool_t do_workproc)
 			break;
 	}
 	*buf = '\0';
-
 	return ((bool_t) (tot > 0));
 }
 
@@ -585,7 +550,7 @@ cddb_remote_open(cddb_file_t *fp)
 		conport = (unsigned short) fp->pathp->port;
 	}
 
-	sin.sin_port = bswap16(conport);
+	sin.sin_port = util_bswap16(conport);
 	sin.sin_family = AF_INET;
 
 	/* Set timeout in case remote server is unavailable */
@@ -955,7 +920,7 @@ cddb_cmd_query(
 		for (i = 0; i < (int) s->tot_trks; i++) {
 			(void) sprintf(buf, "%s %d",
 				       buf,
-				       s->trkinfo[i].addr + MSF_OFFSET(s));
+				       s->trkinfo[i].addr + MSF_OFFSET);
 		}
 
 		(void) sprintf(buf, "%s %d\r\n", buf,
@@ -977,7 +942,7 @@ cddb_cmd_query(
 		for (i = 0; i < (int) s->tot_trks; i++) {
 			(void) sprintf(buf, "%s+%d",
 				       buf,
-				       s->trkinfo[i].addr + MSF_OFFSET(s));
+				       s->trkinfo[i].addr + MSF_OFFSET);
 		}
 
 		(void) sprintf(buf,
@@ -1580,7 +1545,7 @@ cddb_add_pathent(char *path)
 		else if (path[0] == '~') {
 			/* Perform tilde expansion a la [ck]sh */
 			if (path[1] == '/') {
-				cp2 = homedir(get_ouid());
+				cp2 = util_homedir(util_get_ouid());
 
 				pp->path = (char *) MEM_ALLOC(
 					n + strlen(cp2)
@@ -1593,7 +1558,7 @@ cddb_add_pathent(char *path)
 				(void) sprintf(pp->path, "%s%s", cp2, &path[1]);
 			}
 			else if (path[1] == '\0') {
-				cp2 = homedir(get_ouid());
+				cp2 = util_homedir(util_get_ouid());
 
 				pp->path = (char *) MEM_ALLOC(
 					strlen(cp2) + 1
@@ -1608,7 +1573,7 @@ cddb_add_pathent(char *path)
 			else {
 				cp = strchr(path, '/');
 				if (cp == NULL) {
-					cp2 = uhomedir(&path[1]);
+					cp2 = util_uhomedir(&path[1]);
 					pp->path = (char *) MEM_ALLOC(
 						strlen(cp2) + 1
 					);
@@ -1623,7 +1588,7 @@ cddb_add_pathent(char *path)
 				}
 				else {
 					*cp = '\0';
-					cp2 = uhomedir(&path[1]);
+					cp2 = util_uhomedir(&path[1]);
 					pp->path = (char *) MEM_ALLOC(
 						n + strlen(cp2)
 					);
@@ -1658,7 +1623,7 @@ cddb_add_pathent(char *path)
 		 * category name is less than FILE_BASE_SZ.
 		 */
 		if ((int) strlen(pp->path) >= (FILE_PATH_SZ - 12) ||
-		    (int) strlen(basename(pp->path)) >= FILE_BASE_SZ) {
+		    (int) strlen(util_basename(pp->path)) >= FILE_BASE_SZ) {
 			CDDB_FATAL(app_data.str_longpatherr);
 			return FALSE;
 		}
@@ -1718,7 +1683,7 @@ cddb_add_linkent(char *dtitle, char *idstr, word32_t offset)
 	else {
 		/* Add to list in sorted order */
 		for (q = cddb_linkhead, r = NULL; q != NULL; q = q->next) {
-			if ((n = cddb_strcasecmp(dtitle, q->dtitle)) == 0) {
+			if ((n = util_strcasecmp(dtitle, q->dtitle)) == 0) {
 				/* Already in list: no need to add */
 				MEM_FREE(p->dtitle);
 				MEM_FREE((char *) p);
@@ -2094,6 +2059,7 @@ cddb_read0(cddb_file_t *fp, cddb_incore_t *dbp, curstat_t *s)
 		if (!cddb_remote_open(fp))
 			return FALSE;
 
+		errno = 0;
 		/*FALLTHROUGH*/
 
 	case CDDB_REMOTE_CDDBP:
@@ -2441,6 +2407,8 @@ cddb_read1(cddb_file_t *fp, cddb_file_t *pfp, cddb_incore_t *dbp)
 			return FALSE;
 		}
 
+		errno = 0;
+
 		/* Set up for remote CDDB server read */
 		if (!cddb_cmd_read(fp, buf, bufsz, dbp)) {
 			MEM_FREE(buf);
@@ -2829,7 +2797,7 @@ cddb_write(cddb_file_t *fp, cddb_incore_t *dbp, curstat_t *s)
 		return FALSE;
 	for (i = 0; i < (int) s->tot_trks; i++) {
 		(void) sprintf(tmpbuf, "#\t%u\n",
-			       s->trkinfo[i].addr + MSF_OFFSET(s));
+			       s->trkinfo[i].addr + MSF_OFFSET);
 		if (!cddb_puts(fp, tmpbuf))
 			return FALSE;
 	}
@@ -3292,7 +3260,7 @@ void
 cddb_halt(curstat_t *s)
 {
 	if (curfile[0] != '\0' && s->devlocked)
-		(void) unlink(curfile);
+		(void) UNLINK(curfile);
 }
 
 
@@ -3359,7 +3327,8 @@ cddb_load(cddb_incore_t *dbp, curstat_t *s)
 
 #else	/* __VMS */
 	int		pfd[2],
-			ret;
+			ret,
+			retcode;
 	pid_t		cpid;
 	waitret_t	stat_val;
 	cddb_file_t	*fp,
@@ -3421,11 +3390,19 @@ cddb_load(cddb_incore_t *dbp, curstat_t *s)
 		/* Close read pipe */
 		(void) cddb_close(rfp);
 
+		if (dbp->flags & CDDB_INEXACT)
+			retcode = CDDB_SET_CODE(MATCH_ERR, 0);
+		else if (dbp->flags & CDDB_AUTHFAIL)
+			retcode = CDDB_SET_CODE(AUTH_ERR, 0);
+		else
+			retcode = 0;
+
 		/* Parent process: wait for child to exit */
 		while ((ret = WAITPID(cpid, &stat_val, 0)) != cpid) {
 			if (ret < 0) {
 				child_pid = 0;
-				return 0;
+				stat_val = 0;
+				break;
 			}
 
 			/* Do some work */
@@ -3437,25 +3414,19 @@ cddb_load(cddb_incore_t *dbp, curstat_t *s)
 
 		if (WIFEXITED(stat_val)) {
 			if (WEXITSTATUS(stat_val) == 0) {
-				if (dbp->flags & CDDB_INEXACT)
-					return CDDB_SET_CODE(MATCH_ERR, 0);
-				else if (dbp->flags & CDDB_AUTHFAIL)
-					return CDDB_SET_CODE(AUTH_ERR, 0);
-				else {
-					switch (dbp->type) {
-					case CDDB_REMOTE_CDDBP:
-					case CDDB_REMOTE_HTTP:
-						if (app_data.cddb_rmtautosave)
-							/* Auto-save */
-							cddb_autosave(dbp, s);
-						break;
+				switch (dbp->type) {
+				case CDDB_REMOTE_CDDBP:
+				case CDDB_REMOTE_HTTP:
+					if (app_data.cddb_rmtautosave)
+						/* Auto-save */
+						cddb_autosave(dbp, s);
+					break;
 
-					default:
-						break;
-					}
-
-					return 0;
+				default:
+					break;
 				}
+
+				return (retcode);
 			}
 			else
 				return CDDB_SET_CODE(WEXITSTATUS(stat_val), 0);
@@ -3470,10 +3441,10 @@ cddb_load(cddb_incore_t *dbp, curstat_t *s)
 	(void) signal(SIGTERM, onterm);
 
 	DBGPRN(errfp, "\nSetting uid to %d, gid to %d\n",
-		(int) get_ouid(), (int) get_ogid());
+		(int) util_get_ouid(), (int) util_get_ogid());
 
 	/* Force uid and gid to original setting */
-	if (setuid(get_ouid()) < 0 || setgid(get_ogid()) < 0)
+	if (setuid(util_get_ouid()) < 0 || setgid(util_get_ogid()) < 0)
 		exit(SETUID_ERR);
 
 	/* Close un-needed pipe descriptor */
@@ -3578,7 +3549,7 @@ cddb_save(cddb_incore_t *dbp, curstat_t *s)
 		/* If discid != queryid, perform a link */
 		if (retcode == 0 && dbp->discid != dbp->queryid) {
 			(void) sprintf(dbfile, CDDBFILE_PATH,
-					dirname(dbp->dbfile), dbp->discid);
+					util_dirname(dbp->dbfile), dbp->discid);
 			(void) cddb_link(dbp, dbp->dbfile, dbfile);
 			(void) strcpy(dbp->dbfile, dbfile);
 		}
@@ -3587,10 +3558,10 @@ cddb_save(cddb_incore_t *dbp, curstat_t *s)
 	}
 
 	DBGPRN(errfp, "\nSetting uid to %d, gid to %d\n",
-		(int) get_ouid(), (int) get_ogid());
+		(int) util_get_ouid(), (int) util_get_ogid());
 
 	/* Force uid and gid to original setting */
-	if (setuid(get_ouid()) < 0 || setgid(get_ogid()) < 0)
+	if (setuid(util_get_ouid()) < 0 || setgid(util_get_ogid()) < 0)
 		exit(SETUID_ERR);
 
 	DBGPRN(errfp, "\nWriting CD database file %s\n", dbp->dbfile);
@@ -3680,17 +3651,17 @@ cddb_link(cddb_incore_t *dbp, char *origpath, char *newpath)
 	}
 
 	DBGPRN(errfp, "\nSetting uid to %d, gid to %d\n",
-		(int) get_ouid(), (int) get_ogid());
+		(int) util_get_ouid(), (int) util_get_ogid());
 
 	/* Force uid and gid to original setting */
-	if (setuid(get_ouid()) < 0 || setgid(get_ogid()) < 0)
+	if (setuid(util_get_ouid()) < 0 || setgid(util_get_ogid()) < 0)
 		exit(SETUID_ERR);
 
 	DBGPRN(errfp, "\ncddb_link: origpath=%s newpath=%s\n",
 		origpath, newpath);
 
-	(void) strncpy(origid, basename(origpath), 8);
-	(void) strncpy(newid, basename(newpath), 8);
+	(void) strncpy(origid, util_basename(origpath), 8);
+	(void) strncpy(newid, util_basename(newpath), 8);
 	origid[8] = newid[8] = '\0';
 
 	/*
@@ -3795,15 +3766,13 @@ cddb_link(cddb_incore_t *dbp, char *origpath, char *newpath)
 	/*
 	 * Remove new file
 	 */
-	if (unlink(newpath) < 0) {
-		(void) unlink(newpath);
+	if (UNLINK(newpath) < 0)
 		exit(LINK_ERR);
-	}
 
 	/*
 	 * Link original file to new file
 	 */
-	if (link(origpath, newpath) < 0)
+	if (LINK(origpath, newpath) < 0)
 		exit(LINK_ERR);
 
 	/* Child exits here. */
@@ -3867,10 +3836,10 @@ cddb_send(cddb_incore_t *dbp, curstat_t *s)
 	}
 
 	DBGPRN(errfp, "\nSetting uid to %d, gid to %d\n",
-		(int) get_ouid(), (int) get_ogid());
+		(int) util_get_ouid(), (int) util_get_ogid());
 
 	/* Force uid and gid to original setting */
-	if (setuid(get_ouid()) < 0 || setgid(get_ogid()) < 0)
+	if (setuid(util_get_ouid()) < 0 || setgid(util_get_ogid()) < 0)
 		exit(SETUID_ERR);
 #endif	/* __VMS */
 
@@ -3930,9 +3899,8 @@ cddb_send(cddb_incore_t *dbp, curstat_t *s)
 				break;
 
 			default:
-				p++;
-				q++;
-				(void) sprintf(p, "%%%c", *q);
+				*p++ = *q++;	/* '%' */
+				*p = *q;
 				break;
 			}
 		}
@@ -4132,7 +4100,7 @@ cddb_category(char *path)
 
 	return (retbuf);
 #else
-	return (basename(path));
+	return (util_basename(path));
 #endif
 }
 
@@ -4151,14 +4119,18 @@ cddb_category(char *path)
 void
 cddb_set_auth(char *name, char *passwd)
 {
-	int	n1,
+	int	i,
+		j,
+		n1,
 		n2;
 	char	*buf;
 
 	if (name == NULL || passwd == NULL)
 		return;
 
-	n1 = strlen(name) + strlen(passwd) + 2;
+	i = strlen(name);
+	j = strlen(passwd);
+	n1 = i + j + 2;
 	n2 = (n1 * 4 / 3) + 8;
 
 	if ((buf = (char *) MEM_ALLOC(n1)) == NULL) {
@@ -4168,6 +4140,9 @@ cddb_set_auth(char *name, char *passwd)
 
 	(void) sprintf(buf, "%s:%s", name, passwd);
 
+	(void) memset(name, 0, i);
+	(void) memset(passwd, 0, j);
+
 	if (auth_buf != NULL)
 		MEM_FREE(auth_buf);
 
@@ -4177,7 +4152,15 @@ cddb_set_auth(char *name, char *passwd)
 	}
 
 	/* Base64 encode the name/password pair */
-	b64encode((byte_t *) buf, strlen(buf), (byte_t *) auth_buf, FALSE);
+	util_b64encode(
+		(byte_t *) buf,
+		strlen(buf),
+		(byte_t *) auth_buf,
+		FALSE
+	);
+
+	(void) memset(buf, 0, n1);
+	MEM_FREE(buf);
 }
 
 
@@ -4237,10 +4220,10 @@ cddb_init_linkopts(cddb_incore_t *dbp, curstat_t *s)
 		/* Error */
 		return FALSE;
 
-	dbdir = dirname(dbp->dbfile);
-	bname = basename(dbp->dbfile);
+	dbdir = util_dirname(dbp->dbfile);
+	bname = util_basename(dbp->dbfile);
 
-	if ((dp = opendir(dbdir)) == NULL)
+	if ((dp = OPENDIR(dbdir)) == NULL)
 		return FALSE;
 
 	if ((buf = (char *) MEM_ALLOC(bufsz)) == NULL) {
@@ -4248,7 +4231,7 @@ cddb_init_linkopts(cddb_incore_t *dbp, curstat_t *s)
 		return FALSE;
 	}
 
-	for (n = 0; (de = readdir(dp)) != NULL; n++) {
+	for (n = 0; (de = READDIR(dp)) != NULL; n++) {
 		/* Handle some events to avoid GUI freeze-up
 		 * (once every 50 loops)
 		 */
@@ -4298,7 +4281,7 @@ cddb_init_linkopts(cddb_incore_t *dbp, curstat_t *s)
 					trk_off[ntrk] =
 						(trkaddr * FRAME_PER_SEC) -
 						s->trkinfo[ntrk].addr -
-						MSF_OFFSET(s);
+						MSF_OFFSET;
 				}
 				else {
 					/* File format error */
@@ -4312,7 +4295,7 @@ cddb_init_linkopts(cddb_incore_t *dbp, curstat_t *s)
 			    (i = sscanf(buf, "# %u\n", &trkaddr)) > 0) {
 				trk_off[ntrk] =
 					trkaddr - s->trkinfo[ntrk].addr -
-					MSF_OFFSET(s);
+					MSF_OFFSET;
 				ntrk++;
 			}
 
@@ -4354,7 +4337,7 @@ cddb_init_linkopts(cddb_incore_t *dbp, curstat_t *s)
 
 		(void) fclose(fp);
 	}
-	(void) closedir(dp);
+	(void) CLOSEDIR(dp);
 
 	MEM_FREE(buf);
 	return TRUE;
@@ -4458,7 +4441,7 @@ cddb_curfileupd(cddb_incore_t *dbp)
 			changed;
 
 	if (!app_data.write_curfile ||			/* disabled */
-	    strcmp(app_data.device, "(none)") == 0)	/* demo */
+	    strncmp(app_data.device, "(sim", 4) == 0)	/* demo */
 		return;
 
 	if (cddb_clinfo->curstat_addr == NULL)
@@ -4569,7 +4552,7 @@ cddb_curfileupd(cddb_incore_t *dbp)
 	}
 
 	/* Remove original file */
-	if (unlink(curfile) < 0 && errno != ENOENT) {
+	if (UNLINK(curfile) < 0 && errno != ENOENT) {
 		DBGPRN(errfp, "\nCannot unlink old %s\n", curfile);
 		return;
 	}
