@@ -6,7 +6,7 @@
  *	libdi - CD Audio Player Device Interface Library
  *
  *
- *   Copyright (C) 1993-1997  Ti Kan
+ *   Copyright (C) 1993-1998  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -28,7 +28,7 @@
 #define __APPENV_H__
 
 #ifndef LINT
-static char *_appenv_h_ident_ = "@(#)appenv.h	6.74 97/08/12";
+static char *_appenv_h_ident_ = "@(#)appenv.h	6.99 98/05/14";
 #endif
 
 
@@ -52,35 +52,12 @@ static char *_appenv_h_ident_ = "@(#)appenv.h	6.74 97/08/12";
 #define MAXTRACK	100			/* Max number of tracks */
 #define LEAD_OUT_TRACK	0xaa			/* Lead-out track number */
 #define FRAME_PER_SEC	75			/* Frames per second */
+#define MSF_OFFSET	150			/* Starting MSF offset */
+
 #define CLIP_FRAMES	10			/* Frames to clip from end */
 #define MAX_VOL		100			/* Max logical audio volume */
 
 
-#ifdef _XMINC_
-
-/*
- * Include needed X11 and Motif headers
- */
-#include <X11/keysym.h>
-#include <Xm/Xm.h>
-#include <Xm/Form.h>
-#include <Xm/Frame.h>
-#include <Xm/Label.h>
-#include <Xm/List.h>
-#include <Xm/MessageB.h>
-#include <Xm/PushB.h>
-#include <Xm/RowColumn.h>
-#include <Xm/Scale.h>
-#include <Xm/Separator.h>
-#include <Xm/Text.h>
-#include <Xm/ToggleB.h>
-#include <Xm/Protocols.h>
-#include <X11/cursorfont.h>
-#include <X11/Xatom.h>
-
-#endif
-
-
 /*
  * Common header files to be included for all modules
  */
@@ -105,6 +82,7 @@ static char *_appenv_h_ident_ = "@(#)appenv.h	6.74 97/08/12";
 #include <memory.h>
 #include <dirent.h>
 #include <pwd.h>
+#include <grp.h>
 
 #ifdef BSDCOMPAT
 #include <strings.h>
@@ -112,7 +90,12 @@ static char *_appenv_h_ident_ = "@(#)appenv.h	6.74 97/08/12";
 #include <string.h>
 #endif
 
-#define PIPE(fds)	pipe(fds)
+#define PIPE		pipe
+#define LINK		link
+#define UNLINK		unlink
+#define OPENDIR		opendir
+#define READDIR		readdir
+#define CLOSEDIR	closedir
 
 #else
 /* OpenVMS */
@@ -163,7 +146,22 @@ typedef uint_t		mode_t;
 #define S_IRUSR		0000400
 #endif
 
+/* Prototype some external library functions */
+extern uid_t		getuid(void);
+extern gid_t		getgid(void);
+extern int		setuid(uid_t);
+extern int		setgid(gid_t);
+extern void		*malloc(int);
+extern void		free(void *);
+extern void		exit(int);
+extern char		*getenv(char *);
+
 #define PIPE(fds)	pipe(fds, O_RDWR, 1024)
+#define LINK		util_link
+#define UNLINK		util_unlink
+#define OPENDIR		util_opendir
+#define READDIR		util_readdir
+#define CLOSEDIR	util_closedir
 
 struct dirent {
 	char		d_name[FILE_PATH_SZ];
@@ -244,13 +242,17 @@ typedef int		waitret_t;
 #endif	/* macII */
 
 
+#ifdef __VMS
+#define WAITPID			util_waitpid
+#else
 #ifdef sony_news
 typedef int			pid_t;
 typedef int			mode_t;
 #define WAITPID(pid, svp, op)	wait4(pid, svp, 0, 0)
 #else
-#define WAITPID(pid, svp, op)	waitpid(pid, svp, op)
-#endif
+#define WAITPID			waitpid
+#endif	/* sony_news */
+#endif	/* __VMS */
 
 
 #if defined(macII) || defined(sony_news)
@@ -319,9 +321,11 @@ typedef int			mode_t;
 #define USR_CMCFG_PATH	"%s/.xmcdcfg/common.cfg"/* User common cfg file */
 #define SYS_DSCFG_PATH	"%s/config/%s"		/* System devspec cfg file */
 #define USR_DSCFG_PATH	"%s/.xmcdcfg/%s"	/* User devspec cfg file */
+#define USR_VINIT_PATH	"%s/.xmcdcfg/%s"	/* User version init file */
 #define REL_DBDIR_PATH	"%s/cddb/%s"		/* CDDB rel dir path */
 #define CDDBFILE_PATH	"%s/%08x"		/* CDDB file path */
 #define HELPFILE_PATH	"%s/help/%s"		/* Help file path */
+#define DOCFILE_PATH	"%s/docs/"		/* Documentation file path */
 #define CONCAT_PATH	"%s/%s"			/* Concatenation of path */
 #else
 #define TEMP_DIR	"SYS$LOGIN:TMP.DIR"	/* Temporary directory */
@@ -329,19 +333,27 @@ typedef int			mode_t;
 #define USR_CMCFG_PATH	"%s.xmcdcfg]common.cfg"	/* User common cfg file */
 #define SYS_DSCFG_PATH	"%s]device.cfg"		/* System devspec cfg file */
 #define USR_DSCFG_PATH	"%s.xmcdcfg]device.cfg"	/* User devspec cfg file */
+#define USR_VINIT_PATH	"%s.xmcdcfg]%s"		/* User version init file */
 #define REL_DBDIR_PATH	"%s.cddb.%s]"		/* CDDB rel dir path */
 #define CDDBFILE_PATH	"%s%08x."		/* CDDB file path */
 #define HELPFILE_PATH	"%s.help]%s"		/* Help file path */
+#define DOCFILE_PATH	"%s.docs]"		/* Documentation file path */
 #define CONCAT_PATH	"%s%s"			/* Concatenation of path */
 #endif
 
 
-/*
- * Macros
+/* 
+ * Web search engine information 
  */
-#define MSF_OFFSET(s)	((s)->trkinfo[0].frame + \
-	(((s)->trkinfo[0].min * 60 + (s)->trkinfo[0].sec) * FRAME_PER_SEC))
-						/* Starting MSF offset */
+typedef struct srcheng_ent {
+	char		*name;			/* Search engine name */
+	char		*action;		/* Action URL+args */
+	void		*aux;			/* Auxiliary pointer */
+	int		acnt;			/* Number of %A s */
+	int		dcnt;			/* Number of %D s */
+	int		tcnt;			/* Number of %T s */
+	struct srcheng_ent *next;		/* Link to next entry */
+} srcheng_ent_t;
 
 
 /*
@@ -377,9 +389,11 @@ typedef int			mode_t;
 /*
  * Defines for the time_dpy field in curstat_t
  */
-#define T_ELAPSED	0			/* Per-track elapsed time */
-#define T_REMAIN_TRACK	1			/* Per-track remaining time */
-#define T_REMAIN_DISC	2			/* Whole-disc remaining time */
+#define TIMEDPY_MAX_MODES	4		/* Max number of modes */
+#define T_ELAPSED		0		/* Per-track elapsed time */
+#define T_ELAPSED_DISC		1		/* Whole-disc elapsed time */
+#define T_REMAIN_TRACK		2		/* Per-track remaining time */
+#define T_REMAIN_DISC		3		/* Whole-disc remaining time */
 
 
 /*
@@ -461,8 +475,8 @@ typedef struct {
 /*
  * Default Send CDDB site and command
  */
-#define CDDB_MAILSITE		"xmcd-cddb@amb.org"
-#define CDDB_MAILCMD		"mailx -s '%s' %s < %s"
+#define CDDB_MAILSITE		"cddb-submit@submit.cddb.com"
+#define CDDB_MAILCMD		"mailx -s '%S' %A < %F"
 
 /*
  * Default message strings
@@ -473,6 +487,7 @@ typedef struct {
 #define STR_CDDBQUERY		"query"
 #define STR_PROGMODE		"prog"
 #define STR_ELAPSE		"elapse"
+#define STR_ELAPSE_DISC		"e-disc"
 #define STR_REMAIN_TRK		"r-trac"
 #define STR_REMAIN_DISC		"r-disc"
 #define STR_PLAY		"play"
@@ -495,7 +510,7 @@ typedef struct {
 #define STR_NOMEMORY		"Out of memory!"
 #define STR_TMPDIRERR		"Cannot create or access directory %s!"
 #define STR_LIBDIRERR		\
-"Neither the XMcd*libdir resource nor the XMCD_LIBDIR environment is defined!"
+"Neither the XMcd.libdir resource nor the XMCD_LIBDIR environment is defined!"
 #define STR_LONGPATHERR		"Path or message too long."
 #define STR_NOMETHOD		"Unsupported deviceInterfaceMethod parameter!"
 #define STR_NOVU		"Unsupported driveVendorCode parameter!"
@@ -550,7 +565,23 @@ Running as a single-disc player"
 #define STR_AUTHFAIL		\
 "Proxy authorization failed.\n\
 Do you want to re-enter your user name and password?"
-
+#define STR_BROWSER_START	"Starting web browser..."
+#define STR_BROWSER_FAIL	\
+"Cannot access web browser.\n\
+Check your configuration."
+#ifdef __VMS
+#define STR_APPDEF		\
+"The XMCD.DAT resource file cannot be located, or is the wrong\n\
+version.  A correct version of this file must be present in the\n\
+appropriate directory in order for xmcd to run.  Please check\n\
+your xmcd installation."
+#else
+#define STR_APPDEF		\
+"The app-defaults/XMcd file cannot be located, or is the wrong\n\
+version.  A correct version of this file must be present in the\n\
+appropriate directory in order for xmcd to run.  Please check\n\
+your xmcd installation."
+#endif
 
 /*
  * Application resource/option data
@@ -558,6 +589,17 @@ Do you want to re-enter your user name and password?"
 typedef struct {
 	char		*libdir;		/* Library path */
 
+	/* X resources */
+	char		*version;		/* app-defaults file version */
+	int		main_mode;		/* Default main window mode */
+	int		normal_width;		/* Normal mode width */
+	int		normal_height;		/* Normal mode height */
+	int		basic_width;		/* Basic mode width */
+	int		basic_height;		/* Basic mode height */
+	int		blinkon_interval;	/* Display blink on (ms) */
+	int		blinkoff_interval;	/* Display blink off (ms) */
+	bool_t		main_showfocus;		/* Highlight kbd focus? */
+
 	/*
 	 * Common config parameters
 	 */
@@ -567,6 +609,12 @@ typedef struct {
 	char		*cddb_mailsite;		/* Send CDDB mail site */
 	char		*cddb_mailcmd;		/* Send CDDB mail command */
 	char		*proxy_server;		/* http proxy host:port */
+	char		*browser_dir;		/* Web browser direct cmd */
+	char		*browser_rmt;		/* Web browser remote cmd */
+	char		*site1_url;		/* Web site 1 URL */
+	char		*site2_url;		/* Web site 2 URL */
+	char		*site3_url;		/* Web site 3 URL */
+	char		*exclude_words;		/* Keywords to exclude */
 	int		cddb_port;		/* Remote CDDB default port */
 	int		cddb_timeout;		/* Remote CDDB timeout (secs) */
 	int		stat_interval;		/* Status poll interval (ms) */
@@ -575,15 +623,16 @@ typedef struct {
 						 * threshold (blocks)
 						 */
 	int		sample_blks;		/* Sample play blocks */
-	int		blinkon_interval;	/* Display blink on (ms) */
-	int		blinkoff_interval;	/* Display blink off (ms) */
+	int		timedpy_mode;		/* Default time display mode */
+	int		tooltip_delay;		/* Tool-tip delay interval */
+	int		tooltip_time;		/* Tool-tip active interval */
 	bool_t		ins_disable;		/* Insert poll disable */
 	bool_t		cddb_rmtautosave;	/* CDDB remote auto-save */
 	bool_t		http_proxy;		/* Using http proxy server */
-	bool_t		main_showfocus;		/* Highlight kbd focus? */
 	bool_t		scsierr_msg;		/* Print SCSI error msg? */
 	bool_t		sol2_volmgt;		/* Solaris 2.x Vol Mgr */
 	bool_t		write_curfile;		/* Enable curr.XXX output */
+	bool_t		tooltip_enable;		/* Enable tool-tips */
 	bool_t		debug;			/* Verbose debug output */
 
 	/*
@@ -646,6 +695,7 @@ typedef struct {
 	char		*str_cddbquery;		/* query */
 	char		*str_progmode;		/* prog */
 	char		*str_elapse;		/* elapse */
+	char		*str_elapsedisc;	/* e-disc */
 	char		*str_remaintrk;		/* r-trac */
 	char		*str_remaindisc;	/* r-disc */
 	char		*str_play;		/* play */
@@ -697,24 +747,27 @@ typedef struct {
 	char		*str_lnkerr_suid;	/* File link err: setuid */
 	char		*str_lnkerr_link;	/* File link err: link */
 	char		*str_lnkerr_killed;	/* File link err: child kill */
-	char		*str_srchdb;		/* link search dialog msg */
+	char		*str_srchdb;		/* Link search dialog msg */
 	char		*str_rmtmatch;		/* CDDB match dialog msg */
 	char		*str_chgsave;		/* Save change dialog msg */
 	char		*str_devlist_undef;	/* deviceList no defined */
 	char		*str_devlist_count;	/* deviceList count incorrect */
-	char		*str_medchg_noinit;	/* cannot init medium changer */
-	char		*str_proxyerr;		/* proxy server config error */
-	char		*str_authfail;		/* proxy auth failure */
+	char		*str_medchg_noinit;	/* Cannot init medium changer */
+	char		*str_proxyerr;		/* Proxy server config error */
+	char		*str_authfail;		/* Proxy auth failure */
+	char		*str_browser_start;	/* Starting web browser */
+	char		*str_browser_fail;	/* Cannot start browser */
+	char		*str_appdef;		/* app-defaults file error */
 
 	/*
 	 * Short-cut key definitions
 	 */
-	char		*btnlbl_key;		/* Button label */
+	char		*mode_key;		/* Basic/Normal mode */
 	char		*lock_key;		/* Lock */
 	char		*repeat_key;		/* Repeat */
 	char		*shuffle_key;		/* Shuffle */
 	char		*eject_key;		/* Eject */
-	char		*poweroff_key;		/* Quit */
+	char		*quit_key;		/* Quit */
 	char		*dbprog_key;		/* Database/Program popup */
 	char		*help_key;		/* Help popup */
 	char		*options_key;		/* Options */
