@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# @(#)install.sh	6.44 97/09/21
+# @(#)install.sh	6.66 98/05/22
 #
 # Script to install the software binary and support files on
 # the target system.
@@ -8,7 +8,7 @@
 #    xmcd  - Motif(tm) CD Audio Player
 #    cda   - Command-line CD Audio Player
 #
-#    Copyright (C) 1993-1997  Ti Kan
+#    Copyright (C) 1993-1998  Ti Kan
 #    E-mail: ti@amb.org
 #
 #    This program is free software; you can redistribute it and/or modify
@@ -29,12 +29,10 @@
 PATH=/bin:/usr/bin:/sbin:/usr/sbin:/etc:/usr/local/bin
 export PATH
 
-CATEGORIES="rock jazz blues newage classical reggae folk country soundtrack misc data"
-XMCD_VER=2.2
+XMCD_VER=2.3
 DEMODB=4102560a
 WWWURL=http://sunsite.unc.edu/~cddb/xmcd/
 DIRPERM=755
-CDIRPERM=777
 SCRPERM=755
 FILEPERM=444
 BINPERM=711
@@ -56,7 +54,8 @@ doexit()
 {
 	if [ $1 -eq 0 ]
 	then
-		$ECHO "\nInstallation of xmcd is now complete."
+		$ECHO "\nInstallation of xmcd is now complete.  Please read"
+		$ECHO "$XMCDLIB/docs/README for further information."
 	else
 		$ECHO "\nErrors have occurred in the installation."
 		if [ $ERRFILE != /dev/null ]
@@ -281,8 +280,9 @@ link_dynlibs()
 
 	if fgrep '=>' $TMPFILE >/dev/null 2>&1
 	then
-		# BSD/SunOS style ldd output
+		# BSD/SunOS/SVR5 style ldd output
 		DYNLIBS="`fgrep -v 'not found' $TMPFILE | \
+			fgrep -v 'xmcd needs:' | \
 			sed -e 's/^.*=> *//' -e 's/(.*)//' | tr '\015' ' '`"
 		ERRLIBS="`fgrep 'not found' $TMPFILE | $AWK '{ print $1 }' | \
 			tr '\015' ' '`"
@@ -466,6 +466,14 @@ then
 		# UNIX SVR4 MP
 		LINKLIBS=True
 		;;
+	5)
+		# UNIX SVR5
+		LINKLIBS=True
+		;;
+	5.*)
+		# UNIX SVR5.x
+		LINKLIBS=True
+		;;
 	*)
 		;;
 	esac
@@ -518,7 +526,7 @@ fi
 # Check existence of binaries
 
 MISSING=
-for i in xmcd_d/xmcd cda_d/cda dbconv_d/wm2xmcd
+for i in xmcd_d/xmcd cda_d/cda util_d/cddbcmd util_d/wm2xmcd
 do
 	if [ ! -f $i ]
 	then
@@ -611,7 +619,7 @@ then
 			/usr/X11R6/lib/X11 \
 			/usr/X11R5/lib/X11 \
 			/usr/X386/lib \
-			/usr/openwin/lib/X11 \
+			/usr/openwin/lib \
 			/usr/local/lib/X11
 	do
 		if [ -d $i ]
@@ -765,21 +773,18 @@ then
 	fi
 fi
 
-
 # Make all necessary directories
 
 $ECHO "\n\nMaking directories..."
 
 makedir $XMCDLIB $DIRPERM $OWNER $GROUP
 makedir $XMCDLIB/cddb $DIRPERM $OWNER $GROUP
-for i in $CATEGORIES
-do
-	makedir $XMCDLIB/cddb/$i $CDIRPERM $OWNER $GROUP
-done
 makedir $XMCDLIB/config $DIRPERM $OWNER $GROUP
 makedir $XMCDLIB/config/.tbl $DIRPERM $OWNER $GROUP
+makedir $XMCDLIB/docs $DIRPERM $OWNER $GROUP
 makedir $XMCDLIB/help $DIRPERM $OWNER $GROUP
 makedir $XMCDLIB/pixmaps $DIRPERM $OWNER $GROUP
+makedir $XMCDLIB/scripts $DIRPERM $OWNER $GROUP
 if [ -n "$MANDIR" ]
 then
 	makedir $MANDIR $DIRPERM $OWNER $GROUP
@@ -792,9 +797,15 @@ $ECHO "\nInstalling xmcd files..."
 # Binaries
 instfile xmcd_d/xmcd $BINDIR/xmcd $XBINPERM $XBINOWNER $GROUP
 instfile cda_d/cda $BINDIR/cda $XBINPERM $XBINOWNER $GROUP
-instfile dbconv_d/wm2xmcd $BINDIR/wm2xmcd $BINPERM $BINOWNER $GROUP
+instfile util_d/cddbcmd $BINDIR/cddbcmd $BINPERM $BINOWNER $GROUP
+instfile util_d/wm2xmcd $BINDIR/wm2xmcd $BINPERM $BINOWNER $GROUP
 
 # X resource defaults file
+if [ $LIBDIR = /usr/openwin/lib -a ! -d /usr/lib/X11 ]
+then
+	ln -s /usr/openwin/lib /usr/lib/X11 2>/dev/null
+fi
+
 if instfile xmcd_d/XMcd.ad $APPDEFAULTSDIR/XMcd $FILEPERM $OWNER $GROUP &&
    [ $LIBDIR != "/usr/lib/X11" -a -d /usr/lib/X11/app-defaults ]
 then
@@ -812,19 +823,43 @@ then
 	rm -f $APPDEFAULTSDIR/._junk_
 fi
 
+# Special hack for some Solaris 2.x versions which have two
+# app-defaults directories
+if [ $APPDEFAULTSDIR = "/usr/openwin/lib/app-defaults" -a \
+     -d /usr/openwin/lib/X11/app-defaults ]
+then
+	rm -f /usr/openwin/lib/X11/app-defaults/XMcd
+	dolink $APPDEFAULTSDIR/XMcd /usr/openwin/lib/X11/app-defaults/XMcd
+fi
+
+# Documentation
+rm -f $XMCDLIB/docs/*
+(cd docs_d ;\
+for i in * ;\
+do \
+	case $i in
+	SCCS)
+		;;
+	RCS)
+		;;
+	CVS)
+		;;
+	*)
+		instfile $i $XMCDLIB/docs/$i $FILEPERM $OWNER $GROUP ;\
+		;;
+	esac
+done)
+
 # Help files
 rm -f $XMCDLIB/help/*
 for i in xmcd_d/hlpfiles/*.btn xmcd_d/hlpfiles/*.lbl xmcd_d/hlpfiles/*.txw \
 	 xmcd_d/hlpfiles/*.scl xmcd_d/hlpfiles/*.lsw xmcd_d/hlpfiles/*.rbx \
-	 xmcd_d/hlpfiles/*.cbx
+	 xmcd_d/hlpfiles/*.cbx xmcd_d/hlpfiles/*.opt
 do
 	j=`echo $i | sed 's/xmcd\_d\/hlpfiles\///'`
 	instfile $i $XMCDLIB/help/$j $FILEPERM $OWNER $GROUP
 done
 
-# Demo cddb file
-instfile misc_d/demo.db $XMCDLIB/cddb/misc/$DEMODB $FILEPERM $OWNER $GROUP
-
 # Icon/pixmap files
 for i in xmcd.icon xmcd_a.px xmcd_b.px xmcd.xpm
 do
@@ -836,6 +871,8 @@ instfile libdi_d/common.cfg $XMCDLIB/config/common.cfg \
 	$FILEPERM $OWNER $GROUP
 instfile libdi_d/device.cfg $XMCDLIB/config/device.cfg \
 	$FILEPERM $OWNER $GROUP
+instfile misc_d/sites $XMCDLIB/config/sites \
+	$FILEPERM $OWNER $GROUP
 
 rm -f $XMCDLIB/config/.tbl/*
 ENTRIES=`(cd libdi_d/cfgtbl; echo * | \
@@ -851,20 +888,26 @@ done
 
 # Configuration script
 SHELL_S=`echo $SHELL | sed 's/\//\\\\\//g'`
+BINDIR_S=`echo $BINDIR | sed 's/\//\\\\\//g'`
 LIBDIR_S=`echo $LIBDIR | sed 's/\//\\\\\//g'`
 XMCDLIB_S=`echo $XMCDLIB | sed 's/\//\\\\\//g'`
 APPDEFAULTSDIR_S=`echo $APPDEFAULTSDIR | sed 's/\//\\\\\//g'`
 sed -e "s/^#!\/bin\/sh.*/#!$SHELL_S/" \
+    -e "s/^BINDIR=.*/BINDIR=$BINDIR_S/" \
     -e "s/^LIBDIR=.*/LIBDIR=$LIBDIR_S/" \
     -e "s/^XMCDLIB=.*/XMCDLIB=$XMCDLIB_S/" \
     -e "s/^APPDEFAULTSDIR=.*/APPDEFAULTSDIR=$APPDEFAULTSDIR_S/" \
     <libdi_d/config.sh >/tmp/xmcdcfg.$$
 
 rm -f $XMCDLIB/config/configure.sh
-instfile /tmp/xmcdcfg.$$ $XMCDLIB/config/config.sh \
-	$SCRPERM $OWNER $GROUP
+instfile /tmp/xmcdcfg.$$ $XMCDLIB/config/config.sh $SCRPERM $OWNER $GROUP
 rm -f /tmp/xmcdcfg.$$
 
+# Shell scripts
+instfile misc_d/ncsarmt.sh $XMCDLIB/scripts/ncsarmt $SCRPERM $OWNER $GROUP
+instfile misc_d/ncsawrap.sh $XMCDLIB/scripts/ncsawrap $SCRPERM $OWNER $GROUP
+instfile misc_d/nswrap.sh $XMCDLIB/scripts/nswrap $SCRPERM $OWNER $GROUP
+
 # Convenience link to config.sh
 if [ "$SCO" = True ]
 then
@@ -889,7 +932,9 @@ then
 		$FILEPERM $OWNER $GROUP
 	instfile cda_d/cda.man $MANDIR/cda.$MANSUFFIX \
 		$FILEPERM $OWNER $GROUP
-	instfile dbconv_d/wm2xmcd.man $MANDIR/wm2xmcd.$MANSUFFIX \
+	instfile util_d/cddbcmd.man $MANDIR/cddbcmd.$MANSUFFIX \
+		$FILEPERM $OWNER $GROUP
+	instfile util_d/wm2xmcd.man $MANDIR/wm2xmcd.$MANSUFFIX \
 		$FILEPERM $OWNER $GROUP
 fi
 
@@ -905,17 +950,25 @@ else
 	logerr -p "Cannot execute $XMCDLIB/config/config.sh"
 fi
 
+# Demo cddb file
+if [ -d $XMCDLIB/cddb/misc ]
+then
+	$ECHO "Installing demo mode CDDB file..."
+	instfile misc_d/demo.db $XMCDLIB/cddb/misc/$DEMODB \
+		$FILEPERM $OWNER $GROUP
+fi
+
+# Set up dynamic library links
 if [ $LINKLIBS = True ]
 then
 	link_dynlibs
 fi
 
+# Clean up
 if [ $NOREMOVE = 0 ]
 then
-	rm -rf common_d libdi_d misc_d xmcd_d cda_d dbconv_d install.sh \
+	rm -rf common_d libdi_d misc_d xmcd_d cda_d util_d docs_d install.sh \
 		xmcd.tar
-	mv README README.xmcd 2>/dev/null
-	mv FAQ FAQ.xmcd 2>/dev/null
 fi
 
 doexit $ERROR
