@@ -1,7 +1,7 @@
 /*
  *   libdi - CD Audio Player Device Interface Library
  *
- *   Copyright (C) 1993-1997  Ti Kan
+ *   Copyright (C) 1993-1998  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -31,7 +31,7 @@
  */
 
 #ifndef LINT
-static char *_aixioc_c_ident_ = "@(#)aixioc.c	6.31 97/10/28";
+static char *_aixioc_c_ident_ = "@(#)aixioc.c	6.45 98/05/18";
 #endif
 
 #include "common_d/appenv.h"
@@ -380,7 +380,7 @@ aixioc_disc_present(bool_t savstat)
 	if (!aixioc_send(IDE_CDAUDIO, (void *) &cmd, TRUE))
 		return FALSE;
 
-	msftoblk(
+	util_msftoblk(
 		cmd.indexing.track_msf.mins,
 		cmd.indexing.track_msf.secs,
 		cmd.indexing.track_msf.frames,
@@ -394,7 +394,7 @@ aixioc_disc_present(bool_t savstat)
 	if (!aixioc_send(IDE_CDAUDIO, (void *) &cmd, TRUE))
 		return FALSE;
 
-	msftoblk(
+	util_msftoblk(
 		cmd.indexing.track_msf.mins,
 		cmd.indexing.track_msf.secs,
 		cmd.indexing.track_msf.frames,
@@ -444,9 +444,9 @@ aixioc_playmsf(msf_t *start, msf_t *end)
 	 * don't try to play frame 85 somewhere), we convert it to a block
 	 * address, add the fudge factor, and then reconvert to MSF.  Yuck!
 	 */
-	msftoblk(start->min, start->sec, start->frame, &tmpblk, 0);
+	util_msftoblk(start->min, start->sec, start->frame, &tmpblk, 0);
 	tmpblk += 16;
-	blktomsf(tmpblk, &(start->min), &(start->sec), &(start->frame), 0);
+	util_blktomsf(tmpblk, &(start->min), &(start->sec), &(start->frame), 0);
 #endif
 
 	cmd.msf_flag = TRUE;
@@ -657,15 +657,14 @@ aixioc_do_playaudio(
  *	some frames.
  *
  * Args:
- *	s - Pointer to the curstat_t structure
  *	blk - Fault frame address
  *
  * Return:
  *	TRUE - success
  *	FALSE - failure
  */
-bool_t
-aixioc_fail_recov(curstat_t *s, word32_t blk)
+STATIC bool_t
+aixioc_fail_recov(word32_t blk)
 {
 	msf_t		recov_start_msf;
 	word32_t	recov_start_addr;
@@ -674,12 +673,12 @@ aixioc_fail_recov(curstat_t *s, word32_t blk)
 	ret = TRUE;
 
 	recov_start_addr = blk + ERR_SKIPBLKS;
-	blktomsf(
+	util_blktomsf(
 		recov_start_addr,
 		&recov_start_msf.min,
 		&recov_start_msf.sec,
 		&recov_start_msf.frame,
-		MSF_OFFSET(s)
+		MSF_OFFSET
 	);
 
 	/* Check to see if we have skipped past
@@ -702,7 +701,7 @@ aixioc_fail_recov(curstat_t *s, word32_t blk)
 
 	if (ret) {
 		/* Restart playback */
-		(void) fprintf(errfp, "CD audio: %s (%02u:%02u.%02u%d)\n",
+		(void) fprintf(errfp, "CD audio: %s (%02u:%02u.%02u)\n",
 				app_data.str_recoverr,
 				recov_start_msf.min,
 				recov_start_msf.sec,
@@ -778,35 +777,21 @@ aixioc_get_playstatus(curstat_t *s)
 	}
 
 	/* Check the CD_INFO_AUDIO data */
-	audio_status = cmd.status;
-	curtrk = cmd.indexing.info_audio.current_track;
-	curidx = cmd.indexing.info_audio.current_index;
-
-	if (curtrk != s->cur_trk) {
-		s->cur_trk = curtrk;
-		DPY_TRACK(s);
-	}
-
-	if (curidx != s->cur_idx) {
-		s->cur_idx = curidx;
-		s->sav_iaddr = s->cur_tot_addr;
-		DPY_INDEX(s);
-	}
 
 	/* Figure out the current absolute block */
-	msftoblk(
+	util_msftoblk(
 		cmd.indexing.info_audio.current_mins,
 		cmd.indexing.info_audio.current_secs,
 		cmd.indexing.info_audio.current_frames,
 		&tmpblk,
-		MSF_OFFSET(s)
+		MSF_OFFSET
 	);
 
 	/* Figure out MSF within the current track from the relative block */
 	s->cur_trk_addr = tmpblk - s->trkinfo[s->cur_trk - s->first_trk].addr;
 	if (s->cur_trk_addr < 0)
 		s->cur_trk_addr = 0;
-	blktomsf(
+	util_blktomsf(
 		s->cur_trk_addr,
 		&(s->cur_trk_min),
 		&(s->cur_trk_sec),
@@ -822,11 +807,27 @@ aixioc_get_playstatus(curstat_t *s)
 	/* Update time display */
 	DPY_TIME(s, FALSE);
 
+	curtrk = cmd.indexing.info_audio.current_track;
+	curidx = cmd.indexing.info_audio.current_index;
+
+	if (curtrk != s->cur_trk) {
+		s->cur_trk = curtrk;
+		DPY_TRACK(s);
+	}
+
+	if (curidx != s->cur_idx) {
+		s->cur_idx = curidx;
+		s->sav_iaddr = s->cur_tot_addr;
+		DPY_INDEX(s);
+	}
+
 	/* Hack: to work around the fact that some CD-ROM drives
 	 * return CD_PAUSE_AUDIO status after issuing a CD_STOP_AUDIO command.
 	 * Just treat the status as completed if we get a paused status
 	 * and we don't expect the drive to be paused.
 	 */
+	audio_status = cmd.status;
+
 	if (audio_status == CD_PAUSE_AUDIO && s->mode != MOD_PAUSE &&
 	    !aixioc_idx_pause)
 		audio_status = CD_COMPLETED;
@@ -879,7 +880,7 @@ aixioc_get_playstatus(curstat_t *s)
 
 		errblk = s->cur_tot_addr;
 
-		if (!done && aixioc_fail_recov(s, errblk)) {
+		if (!done && aixioc_fail_recov(errblk)) {
 			in_aixioc_get_playstatus = FALSE;
 			return TRUE;
 		}
@@ -1176,12 +1177,12 @@ aixioc_get_toc(curstat_t *s)
 		s->trkinfo[i].min = cmdbuf.indexing.track_msf.mins;
 		s->trkinfo[i].sec = cmdbuf.indexing.track_msf.secs;
 		s->trkinfo[i].frame = cmdbuf.indexing.track_msf.frames;
-		msftoblk(
+		util_msftoblk(
 			s->trkinfo[i].min,
 			s->trkinfo[i].sec,
 			s->trkinfo[i].frame,
 			&s->trkinfo[i].addr,
-			MSF_OFFSET(s)
+			MSF_OFFSET
 		);
 
 		if (s->trkinfo[i-1].trkno == s->last_trk ||
@@ -1200,12 +1201,12 @@ aixioc_get_toc(curstat_t *s)
 
 	(void) aixioc_send(IOCINFO, (void *) &cdinfo, TRUE);
 	/* Fake up a lead-out track */
-	blktomsf(
+	util_blktomsf(
 		cdinfo.un.idecd.numblks,
 		&(s->trkinfo[i].min),
 		&(s->trkinfo[i].sec),
 		&(s->trkinfo[i].frame),
-		MSF_OFFSET(s)
+		MSF_OFFSET
 	);
 	s->trkinfo[i].trkno = LEAD_OUT_TRACK;
 
@@ -1588,38 +1589,44 @@ aixioc_run_rew(curstat_t *s)
 	if (aixioc_start_search) {
 		aixioc_start_search = FALSE;
 		seq = 0;
-		if (skip_blks < addr)
-			start_addr = addr - skip_blks;
-		else
-			start_addr = 0;
+		i = (int) (addr - skip_blks);
 	}
 	else {
 		if (app_data.skip_spdup > 0 && seq > app_data.skip_spdup)
 			/* Speed up search */
 			skip_blks *= 3;
 
-		if ((int) (start_addr - skip_blks) > 0)
-			start_addr -= skip_blks;
-		else
-			start_addr = 0;
+		i = (int) (start_addr - skip_blks);
 	}
 
+	start_addr = (word32_t) (i > CLIP_FRAMES) ? i : CLIP_FRAMES;
+
 	seq++;
 
 	if (s->shuffle || s->program) {
 		if ((i = di_curtrk_pos(s)) < 0)
 			i = 0;
-	}
-	else
-		i = 0;
 
-	if (start_addr < s->trkinfo[i].addr)
-		start_addr = s->trkinfo[i].addr;
+		if (start_addr < s->trkinfo[i].addr)
+			start_addr = s->trkinfo[i].addr;
+	}
 
 	end_addr = start_addr + MAX_SRCH_BLKS;
 
-	blktomsf(start_addr, &smsf.min, &smsf.sec, &smsf.frame, MSF_OFFSET(s));
-	blktomsf(end_addr, &emsf.min, &emsf.sec, &emsf.frame, MSF_OFFSET(s));
+	util_blktomsf(
+		start_addr,
+		&smsf.min,
+		&smsf.sec,
+		&smsf.frame,
+		MSF_OFFSET
+	);
+	util_blktomsf(
+		end_addr,
+		&emsf.min,
+		&emsf.sec,
+		&emsf.frame,
+		MSF_OFFSET
+	);
 
 	/* Play next search interval */
 	(void) aixioc_do_playaudio(
@@ -1723,8 +1730,20 @@ aixioc_run_ff(curstat_t *s)
 		start_addr = end_addr - skip_blks;
 	}
 
-	blktomsf(start_addr, &smsf.min, &smsf.sec, &smsf.frame, MSF_OFFSET(s));
-	blktomsf(end_addr, &emsf.min, &emsf.sec, &emsf.frame, MSF_OFFSET(s));
+	util_blktomsf(
+		start_addr,
+		&smsf.min,
+		&smsf.sec,
+		&smsf.frame,
+		MSF_OFFSET
+	);
+	util_blktomsf(
+		end_addr,
+		&emsf.min,
+		&emsf.sec,
+		&emsf.frame,
+		MSF_OFFSET
+	);
 
 	/* Play next search interval */
 	(void) aixioc_do_playaudio(
@@ -1812,19 +1831,19 @@ aixioc_run_sample(curstat_t *s)
 		saddr = s->trkinfo[aixioc_next_sam].addr;
 		eaddr = saddr + app_data.sample_blks,
 
-		blktomsf(
+		util_blktomsf(
 			saddr,
 			&smsf.min,
 			&smsf.sec,
 			&smsf.frame,
-			MSF_OFFSET(s)
+			MSF_OFFSET
 		);
-		blktomsf(
+		util_blktomsf(
 			eaddr,
 			&emsf.min,
 			&emsf.sec,
 			&emsf.frame,
-			MSF_OFFSET(s)
+			MSF_OFFSET
 		);
 
 		if (aixioc_do_playaudio(ADDR_BLK | ADDR_MSF,
@@ -1904,12 +1923,12 @@ aixioc_run_prog(curstat_t *s)
 	s->cur_idx = 1;
 
 	start_addr = s->trkinfo[i].addr + s->cur_trk_addr;
-	blktomsf(
+	util_blktomsf(
 		start_addr,
 		&s->cur_tot_min,
 		&s->cur_tot_sec,
 		&s->cur_tot_frame,
-		MSF_OFFSET(s)
+		MSF_OFFSET
 	);
 	start_msf.min = s->cur_tot_min;
 	start_msf.sec = s->cur_tot_sec;
@@ -2182,9 +2201,13 @@ void
 aixioc_repeat(curstat_t *s, bool_t enable)
 {
 	s->repeat = enable;
-	DPY_RPTCNT(s);
-	if (!enable)
+
+	if (!enable && aixioc_new_progshuf) {
 		aixioc_new_progshuf = FALSE;
+		if (s->rptcnt > 0)
+			s->rptcnt--;
+	}
+	DPY_RPTCNT(s);
 }
 
 
@@ -2354,12 +2377,12 @@ aixioc_ab(curstat_t *s)
 		    app_data.min_playblks) {
 			aixioc_ab_end_addr = aixioc_ab_start_addr +
 					    app_data.min_playblks;
-			blktomsf(
+			util_blktomsf(
 				aixioc_ab_end_addr,
 				&aixioc_ab_end_msf.min,
 				&aixioc_ab_end_msf.sec,
 				&aixioc_ab_end_msf.frame,
-				MSF_OFFSET(s)
+				MSF_OFFSET
 			);
 		}
 
@@ -2551,11 +2574,12 @@ aixioc_play_pause(curstat_t *s)
 				s->cur_trk = s->first_trk;
 				start_addr = s->trkinfo[0].addr +
 					     s->cur_trk_addr;
-				blktomsf(start_addr,
-					 &start_msf.min,
-					 &start_msf.sec,
-					 &start_msf.frame,
-					 MSF_OFFSET(s)
+				util_blktomsf(
+					start_addr,
+					&start_msf.min,
+					&start_msf.sec,
+					&start_msf.frame,
+					MSF_OFFSET
 				);
 			}
 			else {
@@ -2564,11 +2588,12 @@ aixioc_play_pause(curstat_t *s)
 					     s->cur_trk_addr;
 			}
 
-			blktomsf(start_addr,
-				 &start_msf.min,
-				 &start_msf.sec,
-				 &start_msf.frame,
-				 MSF_OFFSET(s)
+			util_blktomsf(
+				start_addr,
+				&start_msf.min,
+				&start_msf.sec,
+				&start_msf.frame,
+				MSF_OFFSET
 			);
 
 			end_msf.min = s->tot_min;
@@ -2778,24 +2803,34 @@ aixioc_prevtrk(curstat_t *s)
 		else
 			i = di_curtrk_pos(s);
 
-		if (i < 0)
-			return;
-
-		start_addr = s->trkinfo[i].addr;
-		start_msf.min = s->trkinfo[i].min;
-		start_msf.sec = s->trkinfo[i].sec;
-		start_msf.frame = s->trkinfo[i].frame;
-		s->cur_trk = s->trkinfo[i].trkno;
-		s->cur_idx = 1;
-
-		/* If the current track has been playing for less
-		 * than app_data.prev_threshold blocks, then go
-		 * to the beginning of the previous track (if we
-		 * are not already on the first track).
-		 */
 		go_prev = FALSE;
-		if ((s->cur_tot_addr - start_addr) <= app_data.prev_threshold)
-			go_prev = TRUE;
+
+		if (i == 0 && s->cur_idx == 0) {
+			i = 0;
+			start_addr = CLIP_FRAMES;
+			start_msf.min = 0;
+			start_msf.sec = 2;
+			start_msf.frame = CLIP_FRAMES;
+			s->cur_trk = s->trkinfo[i].trkno;
+			s->cur_idx = 0;
+		}
+		else {
+			start_addr = s->trkinfo[i].addr;
+			start_msf.min = s->trkinfo[i].min;
+			start_msf.sec = s->trkinfo[i].sec;
+			start_msf.frame = s->trkinfo[i].frame;
+			s->cur_trk = s->trkinfo[i].trkno;
+			s->cur_idx = 1;
+
+			/* If the current track has been playing for less
+			 * than app_data.prev_threshold blocks, then go
+			 * to the beginning of the previous track (if we
+			 * are not already on the first track).
+			 */
+			if ((s->cur_tot_addr - start_addr) <=
+			    app_data.prev_threshold)
+				go_prev = TRUE;
+		}
 
 		if (go_prev) {
 			if (s->shuffle || s->program) {
@@ -2812,6 +2847,16 @@ aixioc_prevtrk(curstat_t *s)
 				start_msf.frame = s->trkinfo[i].frame;
 				s->cur_trk = s->trkinfo[i].trkno;
 			}
+			else if (i == 0) {
+				/* Go to the very beginning: this may be
+				 * a lead-in area before the start of track 1.
+				 */
+				start_addr = CLIP_FRAMES;
+				start_msf.min = 0;
+				start_msf.sec = 2;
+				start_msf.frame = CLIP_FRAMES;
+				s->cur_trk = s->trkinfo[i].trkno;
+			}
 			else if (i > 0) {
 				i--;
 
@@ -2979,7 +3024,9 @@ aixioc_nexttrk(curstat_t *s)
 		/* Find next track */
 		if ((i = di_curtrk_pos(s)) < 0)
 			return;
-		i++;
+
+		if (i > 0 || s->cur_idx > 0)
+			i++;
 
 		/* Skip over data tracks */
 		while (i < MAXTRACK && s->trkinfo[i].type == TYP_DATA)
@@ -3892,13 +3939,13 @@ aixioc_wait_display(word32_t offset, curstat_t *s)
 	if (!aixioc_send(IDE_CDAUDIO, (void *) &cmd, TRUE))
 		return; 
 
-	/* If we subtract MSF_OFFSET(s) from 0, it wraps back around to some
+	/* If we subtract MSF_OFFSET from 0, it wraps back around to some
 	 * large value (under AIX) since it's unsigned, so just add
-	 * MSF_OFFSET(s) to the desired location and forget the translation.
+	 * MSF_OFFSET to the desired location and forget the translation.
 	 */
-	offset += MSF_OFFSET(s);
+	offset += MSF_OFFSET;
 
-	msftoblk(
+	util_msftoblk(
 		cmd.indexing.info_audio.current_mins,
 		cmd.indexing.info_audio.current_secs,
 		cmd.indexing.info_audio.current_frames,
@@ -3913,12 +3960,12 @@ aixioc_wait_display(word32_t offset, curstat_t *s)
 
 	/* Wait loop */
 	do {
-		delayms(10);	/* Sleep for 10ms to reduce load */
+		util_delayms(10);	/* Sleep for 10ms to reduce load */
 
 		if (!aixioc_send(IDE_CDAUDIO, (void *) &cmd, TRUE))
 			break;	/* shrug */
 
-		msftoblk(
+		util_msftoblk(
 			cmd.indexing.info_audio.current_mins,
 			cmd.indexing.info_audio.current_secs,
 			cmd.indexing.info_audio.current_frames,
