@@ -1,13 +1,13 @@
 #!/bin/sh
 #
-# @(#)config.sh	6.88 97/09/05
+# @(#)config.sh	6.111 98/05/17
 #
 # Script to set up the device-dependent configuration files.
 #
 #    xmcd  - Motif(tm) CD Audio Player
 #    cda   - Command-line CD Audio Player
 #
-#    Copyright (C) 1993-1997  Ti Kan
+#    Copyright (C) 1993-1998  Ti Kan
 #    E-mail: ti@amb.org
 #
 #    This program is free software; you can redistribute it and/or modify
@@ -28,17 +28,27 @@
 PATH=/bin:/usr/bin:/sbin:/usr/sbin:/etc:/usr/local/bin:/usr/ucb
 export PATH
 
-ERRFILE=/tmp/xmcd.err
-TMPFILE=/tmp/xmcdcfg.$$
+VER=2.3
 
 # Change the following directory to fit your local configuration
+BINDIR=/usr/bin/X11
 LIBDIR=/usr/lib/X11
 XMCDLIB=/usr/lib/X11/xmcd
 APPDEFAULTSDIR=/usr/lib/X11/app-defaults
 
-VER=2.2
+ERRFILE=/tmp/xmcd.err
+TMPFILE=/tmp/.xmcdcfg.$$
+TMPSITES=/tmp/.sites.$$
 
 CFGDIR=$XMCDLIB/config
+SCRDIR=$XMCDLIB/scripts
+SITES=$CFGDIR/sites
+OWNER=bin
+GROUP=bin
+CDIRPERM=777
+DFLT_CGIPATH='/~cddb/cddb.cgi'
+CDDBCATS="rock jazz blues newage classical reggae folk country soundtrack misc data"
+BROWSERS="netscape Mosaic"
 
 # Utility functions
 
@@ -141,6 +151,29 @@ dolink()
 }
 
 
+makedir()
+{
+	$ECHO "\t$1"
+	if [ ! -d $1 ]
+	then
+		mkdir -p $1
+	fi
+	if [ $3 != _default_ ]
+	then
+		chown $3 $1 2>/dev/null
+	fi
+	if [ $4 != _default_ ]
+	then
+		chgrp $4 $1 2>/dev/null
+	fi
+	if [ $2 != _default_ ]
+	then
+		chmod $2 $1 2>/dev/null
+	fi
+	return 0
+}
+
+
 ask_scsi_config()
 {
 	$ECHO "\n  Since you have an unlisted drive, I will assume"
@@ -172,207 +205,218 @@ ask_scsi_config()
 
 	while :
 	do
-		YNDEF=n
-		if getyn "\n  Is your drive on $XMCD_DEV a multi-disc changer"
-		then
-			$ECHO "\n  Select one of the following changer methods:\n"
-			$ECHO "  1.\tSCSI LUN addressing method"
-			$ECHO "  2.\tSCSI medium changer method"
-			$ECHO "  3.\tother"
-			$ECHO "  q.\tquit"
+	    YNDEF=n
+	    if getyn "\n  Is your drive on $XMCD_DEV a multi-disc changer"
+	    then
+		$ECHO "\n  Select one of the following changer methods:\n"
+		$ECHO "  1.\tSCSI LUN addressing method"
+		$ECHO "  2.\tSCSI medium changer method"
+		$ECHO "  3.\tother"
+		$ECHO "  q.\tquit"
 
-			while :
-			do
-				$ECHO "\n  Enter choice: \c"
-				read ANS
-
-				if [ "$ANS" = q ]
-				then
-					return 1
-				fi
-
-				if [ -z "$ANS" -o "$ANS" -lt 1 -o "$ANS" -gt 4 ]
-				then
-					$ECHO "  Please answer 1 to 3."
-				else
-					break
-				fi
-			done	
-
-			if [ "$ANS" -eq 3 ]
-			then
-				#
-				# Unsupported changer method: just treat it
-				# as a single-disc drive
-				#
-				$ECHO "\n  Your drive will be treated as a single-disc unit."
-				CHGMETHOD=0
-				NUMDISCS=1
-				MULTIPLAY=0
-			else
-				CHGMETHOD="$ANS"
-				while :
-				do
-					$ECHO "\n  How many discs does the drive support?: \c"
-					read ANS
-
-					if [ -z "$ANS" -o "$ANS" -lt 1 ]
-					then
-						$ECHO "  Invalid input.  Try again."
-					else
-						NUMDISCS="$ANS"
-						break
-					fi
-				done
-
-				if [ "$NUMDISCS" -gt 1 ]
-				then
-					MULTIPLAY=True
-				else
-					CHGMETHOD=0
-					MULTIPLAY=False
-				fi
-			fi
-		else
-			CHGMETHOD=0
-			NUMDISCS=1
-			MULTIPLAY=False
-		fi
-
-		$ECHO "\n  Does your drive on $XMCD_DEV support the following:\n"
-
-		YNDEF=y
-		if getyn "  - The Play_Audio_MSF SCSI command"
-		then
-			PLAYMSF=True
-		else
-			PLAYMSF=False
-		fi
-
-		YNDEF=n
-		if getyn "  - The Play_Audio(12) SCSI command"
-		then
-			PLAY12=True
-		else
-			PLAY12=False
-		fi
-
-		YNDEF=y
-		if getyn "  - The Play_Audio(10) SCSI command"
-		then
-			PLAY10=True
-		else
-			PLAY10=False
-		fi
-
-		YNDEF=y
-		if getyn "  - The Play_Audio_Track/Index command"
-		then
-			PLAYTI=True
-		else
-			PLAYTI=False
-		fi
-
-		YNDEF=n
-		if getyn "  - Caddy load via the Start_Stop_Unit SCSI command"
-		then
-			LOAD=True
-		else
-			LOAD=False
-		fi
-
-		YNDEF=y
-		if getyn "  - Caddy eject via the Start_Stop_Unit SCSI command"
-		then
-			EJECT=True
-		else
-			EJECT=False
-		fi
-
-		YNDEF=y
-		if getyn "  - Disable block descriptor in the Mode_Sense SCSI command"
-		then
-			MODEDBD=True
-		else
-			MODEDBD=False
-		fi
-
-		YNDEF=y
-		if getyn "  - Audio volume control via the Mode_Select SCSI command"
-		then
-			YNDEF=y
-			if getyn "  - Independent SCSI Mode_Select volume control for each channel"
-			then
-				VOLSUPP=True
-				BALSUPP=True
-			else
-				VOLSUPP=True
-				BALSUPP=False
-			fi
-
-			YNDEF=y
-			if getyn "  - Audio channel routing via SCSI Mode_Select"
-			then
-				CHRSUPP=True
-			else
-				CHRSUPP=False
-			fi
-		else
-			VOLCTL=0
-			VOLSUPP=False
-			BALSUPP=False
-			CHRSUPP=False
-		fi
-
-		YNDEF=y
-		if getyn "  - The Pause/Resume SCSI command"
-		then
-			PAUSE=True
-		else
-			PAUSE=False
-		fi
-
-		YNDEF=y
-		if getyn "  - The Prevent/Allow_Medium_Removal SCSI command"
-		then
-			CADDYLOCK=True
-		else
-			CADDYLOCK=False
-		fi
-
-		YNDEF=n
-		if getyn "  - Data Format 1 of the Read_Subchannel SCSI command"
-		then
-			CURPOSFMT=True
-		else
-			CURPOSFMT=False
-		fi
-
-		$ECHO "\n  This is the configuration for ${XMCD_DEV}:\n"
-		$ECHO "  logicalDriverNumber:   $DRVNO"
-		$ECHO "  mediumChangeMethod:    $CHGMETHOD"
-		$ECHO "  numDiscs:              $NUMDISCS"
-		$ECHO "  playAudio12Support:    $PLAY12"
-		$ECHO "  playAudioMSFSupport:   $PLAYMSF"
-		$ECHO "  playAudio10Support:    $PLAY10"
-		$ECHO "  playAudioTISupport:    $PLAYTI"
-		$ECHO "  loadSupport:           $LOAD"
-		$ECHO "  ejectSupport:          $EJECT"
-		$ECHO "  modeSenseSetDBD:       $MODEDBD"
-		$ECHO "  volumeControlSupport:  $VOLSUPP"
-		$ECHO "  balanceControlSupport: $BALSUPP"
-		$ECHO "  pauseResumeSupport:    $PAUSE"
-		$ECHO "  caddyLockSupport:      $CADDYLOCK"
-		$ECHO "  curposFormat:          $CURPOSFMT"
-		$ECHO "  multiPlay:             $MULTIPLAY"
-
-		YNDEF=y
-		if getyn "\n  Is this acceptable"
-		then
+		while :
+		do
+		    if getstr "\n  Enter choice: [1]"
+		    then
+			case "$ANS" in
+			[1-3])
+			    break
+			    ;;
+			q)
+			    return 1
+			    ;;
+			*)
+			    $ECHO "  Please answer 1 to 3."
+			    ;;
+			esac
+		    else
+			ANS=1
 			break
-		fi
+		    fi
+		done	
 
-		$ECHO "  Try again..."
+		if [ "$ANS" -eq 3 ]
+		then
+		    #
+		    # Unsupported changer method: just treat it
+		    # as a single-disc drive
+		    #
+		    $ECHO "\n  Your drive will be treated as a single-disc unit."
+		    CHGMETHOD=0
+		    NUMDISCS=1
+		    MULTIPLAY=0
+		else
+		    CHGMETHOD="$ANS"
+		    while :
+		    do
+			if getstr \
+			    "\n  How many discs does the drive support?:"
+			then
+			    case "$ANS" in
+			    [1-9]*)
+				NUMDISCS="$ANS"
+				break
+				;;
+			    *)
+				$ECHO "  Invalid input.  Try again."
+				;;
+			    esac
+			else
+			    $ECHO "  Invalid input.  Try again."
+			fi
+		    done
+
+		    if [ "$NUMDISCS" -gt 1 ]
+		    then
+			MULTIPLAY=True
+		    else
+			CHGMETHOD=0
+			MULTIPLAY=False
+		    fi
+		fi
+	    else
+		CHGMETHOD=0
+		NUMDISCS=1
+		MULTIPLAY=False
+	    fi
+
+	    $ECHO "\n  Does your drive on $XMCD_DEV support the following:\n"
+
+	    YNDEF=y
+	    if getyn "  - The Play_Audio_MSF SCSI command"
+	    then
+		    PLAYMSF=True
+	    else
+		    PLAYMSF=False
+	    fi
+
+	    YNDEF=n
+	    if getyn "  - The Play_Audio(12) SCSI command"
+	    then
+		    PLAY12=True
+	    else
+		    PLAY12=False
+	    fi
+
+	    YNDEF=y
+	    if getyn "  - The Play_Audio(10) SCSI command"
+	    then
+		    PLAY10=True
+	    else
+		    PLAY10=False
+	    fi
+
+	    YNDEF=y
+	    if getyn "  - The Play_Audio_Track/Index command"
+	    then
+		    PLAYTI=True
+	    else
+		    PLAYTI=False
+	    fi
+
+	    YNDEF=n
+	    if getyn "  - Caddy load via the Start_Stop_Unit SCSI command"
+	    then
+		    LOAD=True
+	    else
+		    LOAD=False
+	    fi
+
+	    YNDEF=y
+	    if getyn "  - Caddy eject via the Start_Stop_Unit SCSI command"
+	    then
+		    EJECT=True
+	    else
+		    EJECT=False
+	    fi
+
+	    YNDEF=y
+	    if getyn \
+		"  - Disable block descriptor in the Mode_Sense SCSI command"
+	    then
+		    MODEDBD=True
+	    else
+		    MODEDBD=False
+	    fi
+
+	    YNDEF=y
+	    if getyn \
+		"  - Audio volume control via the Mode_Select SCSI command"
+	    then
+		    YNDEF=y
+		    if getyn \
+		    "  - Independent SCSI Mode_Select volume control for each channel"
+		    then
+			    VOLSUPP=True
+			    BALSUPP=True
+		    else
+			    VOLSUPP=True
+			    BALSUPP=False
+		    fi
+
+		    YNDEF=y
+		    if getyn "  - Audio channel routing via SCSI Mode_Select"
+		    then
+			    CHRSUPP=True
+		    else
+			    CHRSUPP=False
+		    fi
+	    else
+		    VOLCTL=0
+		    VOLSUPP=False
+		    BALSUPP=False
+		    CHRSUPP=False
+	    fi
+
+	    YNDEF=y
+	    if getyn "  - The Pause/Resume SCSI command"
+	    then
+		    PAUSE=True
+	    else
+		    PAUSE=False
+	    fi
+
+	    YNDEF=y
+	    if getyn "  - The Prevent/Allow_Medium_Removal SCSI command"
+	    then
+		    CADDYLOCK=True
+	    else
+		    CADDYLOCK=False
+	    fi
+
+	    YNDEF=n
+	    if getyn "  - Data Format 1 of the Read_Subchannel SCSI command"
+	    then
+		    CURPOSFMT=True
+	    else
+		    CURPOSFMT=False
+	    fi
+
+	    $ECHO "\n  This is the configuration for ${XMCD_DEV}:\n"
+	    $ECHO "  logicalDriverNumber:   $DRVNO"
+	    $ECHO "  mediumChangeMethod:    $CHGMETHOD"
+	    $ECHO "  numDiscs:              $NUMDISCS"
+	    $ECHO "  playAudio12Support:    $PLAY12"
+	    $ECHO "  playAudioMSFSupport:   $PLAYMSF"
+	    $ECHO "  playAudio10Support:    $PLAY10"
+	    $ECHO "  playAudioTISupport:    $PLAYTI"
+	    $ECHO "  loadSupport:           $LOAD"
+	    $ECHO "  ejectSupport:          $EJECT"
+	    $ECHO "  modeSenseSetDBD:       $MODEDBD"
+	    $ECHO "  volumeControlSupport:  $VOLSUPP"
+	    $ECHO "  balanceControlSupport: $BALSUPP"
+	    $ECHO "  pauseResumeSupport:    $PAUSE"
+	    $ECHO "  caddyLockSupport:      $CADDYLOCK"
+	    $ECHO "  curposFormat:          $CURPOSFMT"
+	    $ECHO "  multiPlay:             $MULTIPLAY"
+
+	    YNDEF=y
+	    if getyn "\n  Is this acceptable"
+	    then
+		    break
+	    fi
+
+	    $ECHO "  Try again..."
 	done
 
 	return 0
@@ -381,7 +425,7 @@ ask_scsi_config()
 
 ask_nonscsi_config()
 {
-	$ECHO "\n  Non-SCSI drives are currently supported only on the"
+	$ECHO "\n  Non-SCSI drives are currently supported only on the BSDI,"
 	$ECHO "  Digital UNIX, FreeBSD, IBM AIX, Linux, NetBSD, OpenBSD, QNX,"
 	$ECHO "  SCO Open Server and Sun Solaris/x86 platforms.  You must"
 	$ECHO "  have the appropriate CD-ROM driver configured in your kernel."
@@ -402,25 +446,28 @@ ask_nonscsi_config()
 	$ECHO "  1.\tSunOS/Solaris/Linux/QNX ioctl method"
 	$ECHO "  2.\tFreeBSD/NetBSD/OpenBSD ioctl method"
 	$ECHO "  3.\tIBM AIX IDE ioctl method"
-	$ECHO "  4.\tSCO Open Server ATAPI BTLD"
-	$ECHO "  5.\tDigital UNIX ATAPI"
+	$ECHO "  4.\tBSDI BSD/OS ATAPI"
+	$ECHO "  5.\tSCO Open Server ATAPI BTLD"
+	$ECHO "  6.\tDigital UNIX ATAPI"
 	$ECHO "  q.\tquit"
 
 	while :
 	do
-		$ECHO "\n  Enter choice: \c"
-		read ANS
-
-		if [ "$ANS" = q ]
+		if getstr "\n  Enter choice:"
 		then
-			return 1
-		fi
-
-		if [ -z "$ANS" -o "$ANS" -lt 1 -o "$ANS" -gt 5 ]
-		then
-			$ECHO "  Please answer 1 to 5."
+			case "$ANS" in
+			[1-6])
+				break
+				;;
+			q)
+				return 1
+				;;
+			*)
+				$ECHO "  Please answer 1 to 6."
+				;;
+			esac
 		else
-			break
+			$ECHO "  Please answer 1 to 6."
 		fi
 	done
 
@@ -433,15 +480,20 @@ ask_nonscsi_config()
 		then
 			while :
 			do
-				$ECHO "\n  How many discs does the drive support?: \c"
-				read ANS
-
-				if [ -z "$ANS" -o "$ANS" -lt 1 ]
+				if getstr \
+				"\n  How many discs does the drive support?:"
 				then
-					$ECHO "  Invalid input.  Try again."
-				else
+				    case "$ANS" in
+				    [1-9]*)
 					NUMDISCS="$ANS"
 					break
+					;;
+				    *)
+					$ECHO "  Invalid input.  Try again."
+					;;
+				    esac
+				else
+				    $ECHO "  Invalid input.  Try again."
 				fi
 			done
 
@@ -525,7 +577,32 @@ ask_nonscsi_config()
 		CURPOSFMT=False
 		PLAYNOTUR=False
 		;;
-	4)	# SCO Open Server ATAPI BTLD (SCSI emulation)
+	4)	# BSDI BSD/OS ATAPI (SCSI emulation)
+		# Set up as a "generic" SCSI-2 drive
+		METHOD=0
+		VENDOR=0
+		VOLBASE=0
+		VOLTAPER=0
+		PLAYMSF=True
+		PLAY12=False
+		PLAY10=False
+		PLAYTI=True
+		LOAD=True
+		EJECT=True
+		MODEDBD=False
+		VOLSUPP=False
+		BALSUPP=False
+		CHRSUPP=False
+		VOLCTL=0
+		PAUSE=True
+		CADDYLOCK=False
+		CURPOSFMT=True
+		PLAYNOTUR=False
+		CHGMETHOD=0
+		NUMDISCS=1
+		MULTIPLAY=False
+		;;
+	5)	# SCO Open Server ATAPI BTLD (SCSI emulation)
 		# Set up as a "generic" SCSI-2 drive
 		METHOD=0
 		VENDOR=0
@@ -550,7 +627,7 @@ ask_nonscsi_config()
 		NUMDISCS=1
 		MULTIPLAY=False
 		;;
-	5)	# Digital UNIX ATAPI (SCSI emulation)
+	6)	# Digital UNIX ATAPI (SCSI emulation)
 		# Set up as a "generic" SCSI-2 drive
 		METHOD=0
 		VENDOR=0
@@ -568,7 +645,7 @@ ask_nonscsi_config()
 		CHRSUPP=True
 		VOLCTL=7
 		PAUSE=True
-		CADDYLOCK=False
+		CADDYLOCK=True
 		CURPOSFMT=True
 		PLAYNOTUR=False
 		CHGMETHOD=0
@@ -651,39 +728,39 @@ config_drive()
 
 	while :
 	do
-		$ECHO "\n  Enter choice: \c"
-		read ANS
+		if getstr "\n  Enter choice:"
+		then
+			if [ "$ANS" = q ]
+			then
+				return 1
+			elif [ "$ANS" -lt 1 -o "$ANS" -gt $j ]
+			then
+				$ECHO "  Please answer 1 to $j."
+			elif [ "$ANS" = "`expr $j - 1`" ]
+			then
+				ask_scsi_config
+				return $?
+			elif [ "$ANS" = "$j" ]
+			then
+				ask_nonscsi_config
+				return $?
+			else
+				k=1
+				for i in $ENTRIES
+				do
+					if [ $k = $ANS ]
+					then
+						model_sel $i $CFGDIR/.tbl/$i
+						return $?
+					fi
+					k=`expr $k + 1`
+				done
 
-		if [ "$ANS" = q ]
-		then
-			return 1
-		fi
-
-		if [ -z "$ANS" -o "$ANS" -lt 1 -o "$ANS" -gt $j ]
-		then
-			$ECHO "  Please answer 1 to $j."
-		elif [ "$ANS" = "`expr $j - 1`" ]
-		then
-			ask_scsi_config
-			return $?
-		elif [ "$ANS" = "$j" ]
-		then
-			ask_nonscsi_config
-			return $?
+				# Should not get here.
+				return 1
+			fi
 		else
-			k=1
-			for i in $ENTRIES
-			do
-				if [ $k = $ANS ]
-				then
-					model_sel $i $CFGDIR/.tbl/$i
-					return $?
-				fi
-				k=`expr $k + 1`
-			done
-
-			# Should not get here.
-			return 1
+			$ECHO "  Please answer 1 to $j."
 		fi
 	done
 
@@ -740,35 +817,35 @@ model_sel()
 
 	while :
 	do
-		$ECHO "\n  Enter choice: \c"
-		read ANS
+		if getstr "\n  Enter choice:"
+		then
+			j=`grep -v "^#" $2 | wc -l | sed 's/^[ 	]*//'`
+			j=`expr $j + 2`
 
-		j=`grep -v "^#" $2 | wc -l | sed 's/^[ 	]*//'`
-		j=`expr $j + 2`
-
-		if [ "$ANS" = q ]
-		then
-			return 1
-		fi
-
-		if [ -z "$ANS" -o "$ANS" -lt 1 -o "$ANS" -gt $j ]
-		then
-			$ECHO "  Please answer 1 to $j."
-		elif [ "$ANS" = "`expr $j - 1`" ]
-		then
-			ask_scsi_config
-			return $?
-		elif [ "$ANS" = "$j" ]
-		then
-			ask_nonscsi_config
-			return $?
-		else
-			read_config $2 $ANS
-			if [ "$METHOD" != 0 ]
+			if [ "$ANS" = q ]
+			then
+				return 1
+			elif [ "$ANS" -lt 1 -o "$ANS" -gt $j ]
+			then
+				$ECHO "  Please answer 1 to $j."
+			elif [ "$ANS" = "`expr $j - 1`" ]
+			then
+				ask_scsi_config
+				return $?
+			elif [ "$ANS" = "$j" ]
 			then
 				ask_nonscsi_config
+				return $?
+			else
+				read_config $2 $ANS
+				if [ "$METHOD" != 0 ]
+				then
+					ask_nonscsi_config
+				fi
+				return $?
 			fi
-			return $?
+		else
+			$ECHO "  Please answer 1 to $j."
 		fi
 	done
 
@@ -861,26 +938,27 @@ read_config()
 
 get_protocol()
 {
+	$ECHO "\nYou should choose HTTP if your system is behind a"
+	$ECHO "firewall that only allows web access to the Internet."
+	$ECHO "Otherwise, choose CDDBP.  There are more public CDDB"
+	$ECHO "servers supporting CDDBP than HTTP."
 	while :
 	do
 		$ECHO "\n  Protocol Selection\n"
-		$ECHO "    1.\tCDDBP (CD Database Protocol)"
-		$ECHO "    2.\tHTTP  (Hyper-Text Transport Protocol)"
-		$ECHO "    q.\tquit"
-		$ECHO "\n  Enter Choice: [1] \c"
+		$ECHO "  1.\tCDDBP (CD Database Protocol)"
+		$ECHO "  2.\tHTTP  (Hyper-Text Transport Protocol)"
+		$ECHO "  q.\tquit"
 
-		read ANS
-		if [ -z "$ANS" ]
+		if getstr "\n  Enter Choice: [1]"
 		then
-			PROTOCOL=cddbp
-			return
-		else
 			case "$ANS" in
 			1)
 				PROTOCOL=cddbp
+				break
 				;;
 			2)
 				PROTOCOL=http
+				break
 				;;
 			q)
 				$ECHO "\nConfiguration aborted." >&2
@@ -888,135 +966,21 @@ get_protocol()
 				doexit 3
 				;;
 			*)
-				$ECHO "  Invalid selection.  Try again." >&2
-				continue
+				$ECHO "\nInvalid selection.  Try again."
 				;;
 			esac
-			break
-		fi
-	done
-}
-
-
-get_port()
-{
-	if [ "$PROTOCOL" = "http" ]
-	then
-		DFLT_PORT=80
-	else
-		DFLT_PORT=888
-	fi
-
-	$ECHO "  Enter TCP port number: [$DFLT_PORT] \c"
-	read ANS
-
-	PORT="$ANS"
-}
-
-
-get_cgipath()
-{
-	DFLT_CGIPATH='~cddb/cddb.cgi'
-	$ECHO "  Enter CGI path: [$DFLT_CGIPATH] \c"
-	read ANS
-
-	if [ -n "$ANS" ]
-	then
-		CGIPATH="$ANS"
-	else
-		CGIPATH="$DFLT_CGIPATH"
-	fi
-}
-
-
-add_cddbpath()
-{
-	for i in $*
-	do
-		$ECHO "\nConfiguring CD database server \"${i}\"..."
-
-		get_protocol
-		get_port
-
-		case "$PROTOCOL" in
-		cddbp)
-			if [ -n "$PORT" ]
-			then
-			    CDDBPATH="$CDDBPATH;cddbp://${i}:${PORT}"
-			else
-			    CDDBPATH="$CDDBPATH;cddbp://${i}"
-			fi
-			;;
-		http)
-			get_cgipath
-
-			if [ -n "$PORT" ]
-			then
-			    CDDBPATH="$CDDBPATH;http://${i}:${PORT}/${CGIPATH}"
-			else
-			    CDDBPATH="$CDDBPATH;http://${i}/${CGIPATH}"
-			fi
-			;;
-		*)
-			;;
-		esac
-	done
-}
-
-
-cddb_server_config()
-{
-	$ECHO "\nAnswer 'y' to the next question only if your system is"
-	$ECHO "connected to a TCP/IP network and there is an accessible"
-	$ECHO "xmcd CD database server on the network.  A dialup or"
-	$ECHO "dedicated connetion to the Internet will allow you to"
-	$ECHO "access the public xmcd CD Database servers."
-
-	$ECHO "\nA list of Internet public xmcd CD Database servers can"
-	$ECHO "be found on the CD Database Server web site:"
-	$ECHO "\n\thttp://www.cddb.com/"
-	$ECHO "\nThis web site also describes the two supported protocols,"
-	$ECHO "CDDBP and HTTP, and the use of proxy servers."
-
-	$ECHO "\nWould you like to use remote CD database servers"
-	YNDEF=y
-	getyn "in addition to the local CD database directories"
-	if [ $? -ne 0 ]
-	then
-		return
-	fi
-
-	$ECHO "\nYou may specify multiple CD database servers.  Enter"
-	$ECHO "one server host per line, and just press <Return> to"
-	$ECHO "end the list.\n"
-
-	CDDB_SERVERS=
-	cnt=0
-	while :
-	do
-		if getstr "Enter CD database server host name or IP number:"
-		then
-			if ($ECHO "$ANS" | grep "[ 	]") >/dev/null 2>&1
-			then
-				$ECHO "Invalid input.  Try again."
-				continue
-			else
-				CDDB_SERVERS="$CDDB_SERVERS $ANS"
-			fi
 		else
-			if [ $cnt -eq 0 ]
-			then
-				$ECHO "No database server configured."
-			else
-				add_cddbpath $CDDB_SERVERS
-			fi
-			break
+			PROTOCOL=cddbp
+			return
 		fi
-		cnt=`expr $cnt + 1`
 	done
+}
 
+
+get_proxy()
+{
 	YNDEF=n
-	getyn "\nDo you want to use a proxy server"
+	getyn "\n  Do you want to use a proxy server"
 	if [ $? -ne 0 ]
 	then
 		USEPROXY=False
@@ -1027,35 +991,659 @@ cddb_server_config()
 	USEPROXY=True
 	while :
 	do
-		if getstr "Enter the proxy server host name or IP number: \c"
+		if getstr "  Enter the proxy server host name or IP number:"
 		then
 			if ($ECHO "$ANS" | grep "[ 	]") >/dev/null 2>&1
 			then
-				$ECHO "Invalid input.  Try again."
+				$ECHO "  Invalid input.  Try again."
 				continue
 			else
 				PROXYSERVER="$ANS"
 			fi
 
-			if getstr "Enter the proxy server port number: [80] \c"
+			if getstr "  Enter the proxy server port number: [80]"
 			then
 				PROXYSERVER="${PROXYSERVER}:${ANS}"
 			fi
 
 			break
 		else
-			$ECHO "Invalid input.  Try again."
+			$ECHO "  Invalid input.  Try again."
 		fi
 	done
 }
 
 
+get_useserver()
+{
+	$ECHO "\nThis configuration procedure can connect to the central"
+	$ECHO "CD database server to retrieve the latest list of categories"
+	$ECHO "and public server sites.  A built-in list can also be used"
+	$ECHO "that is correct at the time this version of xmcd is released."
+
+	USESERVER=0
+	while :
+	do
+		$ECHO "\n  CDDB Information\n"
+		$ECHO "  1.\tConnect to CDDB server and get current list"
+		$ECHO "  2.\tUse built-in list"
+		$ECHO "  q.\tQuit\n"
+
+		getstr "  Enter choice: [1]"
+		if [ $? -ne 0 ]
+		then
+			ANS=1
+		fi
+
+		case "$ANS" in
+		1)
+			if [ -x $BINDIR/cddbcmd ]
+			then
+				return 0
+			else
+				$ECHO "\nCannot find the cddbcmd(1) utility."
+				$ECHO "This is needed to connect to the server."
+			fi
+			;;
+		2)
+			return 1
+			;;
+		q)
+			$ECHO "\nConfiguration aborted." >&2
+			doexit 3
+			;;
+		*)
+			$ECHO "\nInvalid input.  Please try again."
+			;;
+		esac
+	done
+	return 1
+}
+
+
+get_host()
+{
+	if getstr "    Server host name or IP number:"
+	then
+		HOST="$ANS"
+		return 0
+	else
+		$ECHO "    No host specified."
+		return 1
+	fi
+}
+
+
+get_port()
+{
+	if [ "$PROTOCOL" = http ]
+	then
+		DFLT_PORT=80
+	else
+		DFLT_PORT=888
+	fi
+
+	if getstr "    TCP port number: [$DFLT_PORT]"
+	then
+		PORT="$ANS"
+	else
+		PORT=$DFLT_PORT
+	fi
+}
+
+
+get_cgipath()
+{
+	if getstr "    Enter CGI path: [$DFLT_CGIPATH]"
+	then
+		CGIPATH="$ANS"
+	else
+		CGIPATH="$DFLT_CGIPATH"
+	fi
+}
+
+
+print_servers()
+{
+	$ECHO "\n  CDDB server sites selection"
+	$ECHO "\n\tServer host               Proto Port   Location\n"
+	$AWK '
+	BEGIN	{
+		n = 1
+	}
+	{
+		printf("  %d.\t%-25s %-5s %-6s ", n, $1, $2, $3)
+		for (i = 7; i <= NF; i++)
+			printf("%s ", $i);
+		printf("\n")
+		n++;
+	}
+	END	{
+		printf("  %d.\tother site not listed\n", n++)
+		printf("  ?.\tshow this list again\n")
+		printf("  c.\tdone adding servers\n")
+		printf("  q.\tquit\n\n")
+	}
+	' $TMPFILE
+}
+
+
+set_server_entry()
+{
+	if [ ! $1 -eq 0 ]
+	then
+		eval `\
+		(
+			$AWK '
+			BEGIN	{
+				n = 1
+			}
+			{
+				if (n == i) {
+					printf("HOST=%s\n", $1)
+					printf("PORT=%s\n", $3)
+					printf("CGIPATH=%s\n", $4)
+				}
+				n++;
+			}
+			' i=$1 $TMPFILE
+		)`
+	fi
+
+	if [ -z "$HOST" -o -z "$PORT" ]
+	then
+		logerr -p "Internal error: undefined HOST or PORT!"
+		doexit 1
+	fi
+
+	case "$PROTOCOL" in
+	cddbp)
+		if [ $PORT -eq 888 ]
+		then
+			ENTRY="cddbp://${HOST}"
+		else
+			ENTRY="cddbp://${HOST}:${PORT}"
+		fi
+		;;
+	http)
+		if [ $PORT -eq 80 ]
+		then
+			ENTRY="http://${HOST}${CGIPATH}"
+		else
+			ENTRY="http://${HOST}:${PORT}${CGIPATH}"
+		fi
+		;;
+	*)
+		return
+		;;
+	esac
+
+	CDDBPATH="$CDDBPATH;$ENTRY"
+	$ECHO "    Added \"$ENTRY\" to cddbPath."
+}
+
+
+prompt_server()
+{
+	get_host
+	if [ $? -ne 0 ]
+	then
+		return 1
+	fi
+
+	get_port
+
+	if [ "$PROTOCOL" = http ]
+	then
+		get_cgipath
+	fi
+
+	set_server_entry 0
+	return 0
+}
+
+
+sel_servers()
+{
+	$AWK '
+	/^#/ || /^$/ { next }
+	{ if ($2 == protocol) print }
+	' protocol="$PROTOCOL" $SITES >$TMPFILE
+
+	NUMSERVERS=`wc -l $TMPFILE | $AWK '{ print $1 }'`
+	USRSPEC=`expr $NUMSERVERS + 1`
+
+	$ECHO "\nYou may configure more than one server site.  Choose the"
+	$ECHO "servers closest to you.  Enter the selections one at a time."
+	$ECHO "Type 'c' when done."
+
+	cnt=0
+	print_servers
+	while :
+	do
+		if getstr "  Enter CDDB server selection:"
+		then
+			case "$ANS" in
+			[0-9]*)
+			    if [ "$ANS" -lt 1 -o "$ANS" -gt $NUMSERVERS ]
+			    then
+				if [ "$ANS" -eq $USRSPEC ]
+				then
+				    if prompt_server
+				    then
+					cnt=`expr $cnt + 1`
+				    fi
+				else
+				    $ECHO "\nInvalid selection.  Try again."
+				fi
+			    else
+				set_server_entry $ANS
+				cnt=`expr $cnt + 1`
+			    fi
+			    ;;
+			'?')
+			    print_servers
+			    ;;
+			c)
+			    $ECHO "\nTotal CDDB servers configured: $cnt"
+			    break
+			    ;;
+			q)
+			    $ECHO "\nConfiguration aborted." >&2
+			    logerr -n "Configuration aborted by user"
+			    doexit 3
+			    ;;
+			*)
+			    $ECHO "  Please enter 1-${USRSPEC}, c, ? or q."
+			    ;;
+			esac
+		else
+			$ECHO "  Please enter 1-${USRSPEC}, c, ? or q."
+		fi
+	done
+	rm -f $TMPSITES $TMPFILE
+}
+
+
+check_local_cddb()
+{
+	$ECHO "\nChecking local CD database...\n"
+	if [ -z "$CDDBCATS" ]
+	then
+		$ECHO "\nNo CD database categories defined!"
+		return
+	fi
+
+	for i in $CDDBCATS
+	do
+		makedir $XMCDLIB/cddb/$i $CDIRPERM $OWNER $GROUP
+	done
+}
+
+
+cddb_config()
+{
+	# Set up default cddbPath
+	cnt=0
+	for i in $CDDBCATS
+	do
+		if [ $cnt -eq 0 ]
+		then
+			CDDBPATH="$i"
+		else
+			CDDBPATH="${CDDBPATH};$i"
+		fi
+		cnt=`expr $cnt + 1`
+	done
+
+	$ECHO "\nIf your system has Internet connectivity and functional"
+	$ECHO "domain name service (DNS), you should answer 'y' to the next"
+	$ECHO "question.  If you have a dial-up Internet connection that"
+	$ECHO "needs to be manually started, you should start up the link"
+	$ECHO "now before you continue with this configuration procedure."
+	$ECHO "If this system is not linked to the Internet, then answer 'n'."
+
+	YNDEF=y
+	getyn "\n  Would you like to use Internet CD database servers"
+	if [ $? -ne 0 ]
+	then
+		$ECHO "\nNo Internet CD database servers are configured."
+
+		# Check and set up local CDDB directories
+		check_local_cddb
+		return
+	fi
+
+	# Get which protocol to use
+	get_protocol
+	if [ "$PROTOCOL" = http ]
+	then
+		get_proxy
+	fi
+
+	# Get current CDDB categories and servers list
+	if get_useserver
+	then
+	    $ECHO "\nConnecting to the central CDDB server."
+
+	    OPTS=""
+	    if [ "$PROTOCOL" = http ]
+	    then
+		OPTS="$OPTS -m http"
+		if [ $USEPROXY = True ]
+		then
+		    OPTS="$OPTS -p $PROXYSERVER"
+		fi
+	    fi
+
+	    # Categories
+	    $ECHO "Getting CDDB categories... \c"
+	    cnt=0
+	    while [ $cnt -lt 5 ]
+	    do
+		CATS=`$BINDIR/cddbcmd $OPTS cddb lscat 2>/dev/null`
+		stat=$?
+		if [ $stat -eq 0 ]
+		then
+		    break
+		fi
+		sleep 5
+		$ECHO "retrying \c"
+		cnt=`expr $cnt + 1`
+	    done
+	    if [ $stat -ne 0 -o -z "$CATS" ]
+	    then
+		$ECHO "Failed"
+		$ECHO "Cannot obtain CDDB categories list from central server."
+		$ECHO "Using built-in list."
+	    else
+		$ECHO "Success"
+		CDDBCATS="$CATS"
+
+		# Set up cddbPath
+		cnt=0
+		for i in $CDDBCATS
+		do
+		    if [ $cnt -eq 0 ]
+		    then
+			CDDBPATH="$i"
+		    else
+			CDDBPATH="${CDDBPATH};$i"
+		    fi
+		    cnt=`expr $cnt + 1`
+		done
+	    fi
+
+	    # Sites
+	    $ECHO "Getting CDDB public server sites list... \c"
+	    cnt=0
+	    while [ $cnt -lt 5 ]
+	    do
+		$BINDIR/cddbcmd $OPTS sites 2>/dev/null | \
+		    grep -v unofficial >$TMPSITES
+		stat=$?
+		if [ $stat -eq 0 ]
+		then
+		    break
+		fi
+		sleep 5
+		$ECHO "retrying \c"
+		cnt=`expr $cnt + 1`
+	    done
+	    if [ $stat -ne 0 -o ! -s $TMPSITES ]
+	    then
+		$ECHO "Failed"
+		$ECHO "Cannot obtain server sites list from central server."
+		$ECHO "Using built-in list."
+	    else
+		$ECHO "Success"
+		SITES=$TMPSITES
+	    fi
+	fi
+
+	if [ ! -r $SITES ]
+	then
+		$ECHO "Warning: Cannot read $SITES."
+		SITES=/dev/null
+	fi
+
+	$ECHO "\nPress <RETURN> to continue. \c"
+	read ANS
+
+	# Let user choose the CDDB servers
+	sel_servers
+
+	# Check and set up local CDDB directories
+	check_local_cddb
+}
+
+
+browser_config()
+{
+	$ECHO "\nSearching for installed web browsers..."
+	BROWSER=
+	WORD=A
+	for j in $BROWSERS
+	do
+		for i in	/usr/bin/X11 \
+				/usr/X/bin \
+				/usr/X11/bin \
+				/usr/X11R6/bin \
+				/usr/X11R5/bin \
+				/usr/X386/bin \
+				/usr/openwin/bin \
+				/usr/local/bin/X11 \
+				/usr/local/bin \
+				/usr/lbin
+		do
+		    if [ -x $i/$j ]
+		    then
+			$ECHO "\nThe following is found on your system:"
+			$ECHO "    $i/$j"
+			YNDEF=y
+			if getyn "\n  Do you want xmcd to use this browser"
+			then
+				BDIR=$i
+				BROWSER=$j
+				break
+			else
+				WORD=Another
+			fi
+		    fi
+		done
+		if [ -n "$BROWSER" ]
+		then
+		    break
+		fi
+	done
+
+	if [ -z "$BROWSER" ]
+	then
+		$ECHO "\n$WORD web browser cannot be found on your system."
+		$ECHO "You must have Netscape Navigator/Communicator or"
+		$ECHO "or NCSA Mosaic in order to use the wwwWarp feature"
+		$ECHO "in xmcd.  If you have one of these browsers, please"
+		$ECHO "enter its full path name."
+
+		while :
+		do
+		    BDIR=/usr/bin/X11
+		    i=netscape
+
+		    $ECHO "\n  Enter web browser path, type \"c\" to \c"
+		    $ECHO "continue without\n  specifying a browser path, \c"
+		    $ECHO "or type \"q\" to abort."
+		    if getstr "  [$BDIR/$i]:"
+		    then
+			    if [ "$ANS" = c ]
+			    then
+				break
+			    elif [ "$ANS" = q ]
+			    then
+				$ECHO "\nConfiguration aborted." >&2
+				logerr -n "Configuration aborted by user"
+				doexit 3
+			    fi
+
+			    if [ -x "$ANS" ]
+			    then
+				BDIR=`dirname $ANS`
+				i=`basename $ANS`
+				for j in $BROWSERS
+				do
+					if [ $j = $i ]
+					then
+						BROWSER=$i
+						break
+					fi
+				done
+				if [ -z "$BROWSER" ]
+				then
+					$ECHO "\n$i is not a supported browser."
+				else
+					break
+				fi
+			    else
+				$ECHO "\n$ANS does not exist!"
+			    fi
+			else
+			    BROWSER=$i
+			    break
+			fi
+		done
+	fi
+
+	#
+	# Make netscape the default setting
+	#
+	BROWSER_RMT="netscape -remote 'openURL(%U)' >/dev/null 2>&1"
+	BROWSER_DIR="netscape '%U' >/dev/null 2>&1 &"
+
+	if [ -n "$BROWSER" ]
+	then
+	    #
+	    # Set up browser configuration
+	    #
+	    case $BROWSER in
+	    netscape)
+		    BROWSER_RMT="$BDIR/$BROWSER_RMT"
+		    BROWSER_DIR="$BDIR/$BROWSER_DIR"
+		    ;;
+	    Mosaic)
+		    BROWSER_RMT="$SCRDIR/ncsarmt '%U'"
+		    BROWSER_DIR="$BDIR/Mosaic '%U' >/dev/null 2>&1 &"
+		    ;;
+	    esac
+
+	    $ECHO "\nXmcd has been configured to use the $BROWSER browser."
+	else
+	    $ECHO "\nNo web browser specified."
+	    $ECHO "\n  Do you have a supported web browser that can be run"
+	    YNDEF=n
+	    if getyn "  from a remote system"
+	    then
+		while :
+		do
+		    $ECHO "\n  Enter the remote system host name, \c"
+		    $ECHO "type \"c\" to continue without\n  specifying \c"
+		    $ECHO "a host name, \c"
+		    if getstr "or type \"q\" to abort:"
+		    then
+			case "$ANS" in
+			c)
+			    break
+			    ;;
+			q)
+			    $ECHO "\nConfiguration aborted." >&2
+			    logerr -n "Configuration aborted by user"
+			    doexit 3
+			    ;;
+			*)
+			    RMTSYS="$ANS"
+			    break
+			esac
+		    fi
+		done
+
+		if [ -n "$RMTSYS" ]
+		then
+		    while :
+		    do
+			BDIR=/usr/bin/X11
+			i=netscape
+
+			$ECHO "\n  Enter web browser path on \"$RMTSYS\", \c"
+			$ECHO "type \"c\" to\n  continue without specifying \c"
+			$ECHO "a path, or type \"q\" to abort."
+			if getstr "  [$BDIR/$i]: "
+			then
+			    if [ "$ANS" = c ]
+			    then
+				break
+			    elif [ "$ANS" = q ]
+			    then
+				$ECHO "\nConfiguration aborted." >&2
+				logerr -n "Configuration aborted by user"
+				doexit 3
+			    fi
+
+			    BDIR=`dirname $ANS`
+			    i=`basename $ANS`
+			    for j in $BROWSERS
+			    do
+				if [ $j = $i ]
+				then
+				    BROWSER=$i
+				    break
+				fi
+			    done
+
+			    if [ -z "$BROWSER" ]
+			    then
+				$ECHO "\n$i is not a supported browser."
+			    else
+				break
+			    fi
+			else
+			    BROWSER=$i
+			    break
+			fi
+		    done
+		fi
+	    fi
+
+	    if [ -n "$BROWSER" ]
+	    then
+		case $BROWSER in
+		netscape)
+			SCR=$SCRDIR/nswrap
+			;;
+		Mosaic)
+			SCR=$SCRDIR/ncsawrap
+			;;
+		esac
+		BROWSER_RMT="$SCR -remote $RMTSYS $BDIR/$BROWSER '%U'"
+		BROWSER_DIR="$SCR -direct $RMTSYS $BDIR/$BROWSER '%U' &"
+
+		$ECHO "\nXmcd has been configured to use the $BROWSER"
+		$ECHO "browser remotely on host \"$RMTSYS\".  Xmcd"
+		$ECHO "users must have rsh(1) permissions on $RMTSYS"
+		$ECHO "in order to use the wwwWarp feature.  See"
+		$ECHO "rhosts(4) and hosts.equiv(4) for details."
+	    else
+		$ECHO "\nNo web browser is configured."
+		$ECHO "The xmcd wwwWarp feature will not function."
+	    fi
+	fi
+}
+
+
 #
 # Main starts here
 #
 
 # Catch some signals
-trap "rm -f $TMPFILE; exit 1" 1 2 3 5 15
+trap "rm -f $TMPFILE $TMPSITES; exit 1" 1 2 3 5 15
 
 # Get platform information
 OS_SYS=`(uname -s) 2>/dev/null`
@@ -1129,9 +1717,9 @@ STOPONEXIT=True
 EXITONEJECT=False
 CLOSEONEJECT=False
 MAILCMD="mailx -s '%S' %A <%F >/dev/null 2>&1"
-CDDBPATH="rock;jazz;blues;newage;classical;reggae;folk;country;soundtrack;misc;data"
 USEPROXY=False
 PROXYSERVER="yourproxyhost:80"
+CDDBPATH=
 
 # Determine what platform we are running on
 
@@ -1176,7 +1764,12 @@ then
 		DEVSUFF=s0
 		FIRST=4
 		;;
-	B.10*)	# HP-UX 10.x
+	B.10*)		# HP-UX 10.x
+		DEVPREF=/dev/rdsk/c0t
+		DEVSUFF=s0
+		FIRST=4
+		;;
+	B.11*)		# HP-UX 11.x
 		DEVPREF=/dev/rdsk/c0t
 		DEVSUFF=s0
 		FIRST=4
@@ -1330,7 +1923,7 @@ then
 elif [ -x /bin/i386 -o -x /sbin/i386 ] && i386
 then
 	case $OS_VER in
-	3.2)	# SCO ODT
+	3.2)	# SCO UNIX/ODT/Open Server
 		DEVPREF=/dev/rcd
 		DEVSUFF=
 		FIRST=0
@@ -1345,12 +1938,22 @@ then
 		DEVSUFF=
 		FIRST=1
 		;;
-	4.2)	# UNIX SVR4.2/x86
+	4.2)	# UNIX SVR4.2/x86 (UnixWare 1.x)
 		DEVPREF=/dev/rcdrom/cdrom
 		DEVSUFF=
 		FIRST=1
 		;;
-	4*MP)	# UNIX SVR4.2MP/x86
+	4*MP)	# UNIX SVR4.2MP/x86 (UnixWare 2.x)
+		DEVPREF=/dev/rcdrom/cdrom
+		DEVSUFF=
+		FIRST=1
+		;;
+	5)	# UNIX SVR5/x86 (UnixWare 7)
+		DEVPREF=/dev/rcdrom/cdrom
+		DEVSUFF=
+		FIRST=1
+		;;
+	5.*)	# UNIX SVR5.*/x86 (UnixWare 7.x)
 		DEVPREF=/dev/rcdrom/cdrom
 		DEVSUFF=
 		FIRST=1
@@ -1398,15 +2001,24 @@ then
 fi
 
 #
-# Configure remote CD database server access
+# Configure CD database server access
 #
-$ECHO "\n*** NETWORK CONFIGURATION ***"
-cddb_server_config
+$ECHO "\n*** CDDB CONFIGURATION ***"
+cddb_config
+
+#
+# Web browser setup
+#
+$ECHO "\n\n*** WEB BROWSER CONFIGURATION ***"
+browser_config
 
 #
 # Drive configuration
 #
 $ECHO "\n\n*** DRIVE CONFIGURATION ***"
+$ECHO "\nNote that the default device shown is usually a SCSI CD-ROM."
+$ECHO "You will probably need to specify a difference device if your"
+$ECHO "CD-ROM drive is not SCSI."
 EXITSTAT=0
 DRVNO=0
 DEVNO=$FIRST
@@ -1452,12 +2064,12 @@ do
 		if [ -w $APPDEFAULTSDIR/XMcd ]
 		then
 			$AWK '
-			/^XMcd\*libdir:/ {
-				printf("XMcd*libdir:\t\t\t%s\n", xmcdlib)
+			/^XMcd\.libdir:/ {
+				printf("XMcd.libdir:\t\t\t%s\n", xmcdlib)
 				next
 			}
-			/^XMcd\*cddbMailCmd:/ {
-				printf("XMcd*cddbMailCmd:\t\t%s\n", mailcmd)
+			/^XMcd\.cddbMailCmd:/ {
+				printf("XMcd.cddbMailCmd:\t\t%s\n", mailcmd)
 				next
 			}
 			{
@@ -1515,6 +2127,14 @@ do
 				printf("proxyServer:\t\t%s\n", proxyserver)
 				next
 			}
+			/^browserRemote:/ {
+				printf("browserRemote:\t%s\n", browserrmt)
+				next
+			}
+			/^browserDirect:/ {
+				printf("browserDirect:\t%s\n", browserdir)
+				next
+			}
 			{
 				print $0
 			}' \
@@ -1523,6 +2143,8 @@ do
 				cddbpath="$CDDBPATH" \
 				useproxy="$USEPROXY" \
 				proxyserver="$PROXYSERVER" \
+				browserrmt="$BROWSER_RMT" \
+				browserdir="$BROWSER_DIR" \
 				$CFGDIR/common.cfg > /tmp/xmcd.$$
 
 			cp /tmp/xmcd.$$ $CFGDIR/common.cfg
@@ -1600,10 +2222,7 @@ do
 					SAVDEV=$LUNDEV
 					LUNDEV="${LUNPREF}${LUNNUM}${LUNSUFF}"
 
-					$ECHO "  Disc $n: [$LUNDEV] \c"
-					read ANS
-
-					if [ -n "$ANS" ]
+					if getstr "  Disc $n: [$LUNDEV]"
 					then
 						LUNDEV=$ANS
 					fi
@@ -1648,37 +2267,36 @@ do
 			YNDEF=y
 			if getyn "  Does your OS platforms support this?"
 			then
-				while :
-				do
-					LUNDEV=/dev/changer
-					$ECHO "\n  Enter the medium changer device: [$LUNDEV] \c"
-					read ANS
+			    while :
+			    do
+				LUNDEV=/dev/changer
+				if getstr \
+				"\n  Enter the medium changer device: [$LUNDEV]"
+				then
+					LUNDEV=$ANS
+				fi
 
-					if [ -n "$ANS" ]
-					then
-						LUNDEV=$ANS
-					fi
+				if [ $BLKDEV = 0 -a ! -c $LUNDEV ]
+				then
+				    $ECHO "  $LUNDEV is invalid.  Try again."
+				    continue
+				fi
+				if [ $BLKDEV = 1 -a ! -b $LUNDEV ]
+				then
+				    $ECHO "  $LUNDEV is invalid.  Try again."
+				    continue
+				fi
 
-					if [ $BLKDEV = 0 -a ! -c $LUNDEV ]
-					then
-						$ECHO "  $LUNDEV is invalid.  Try again."
-						continue
-					fi
-					if [ $BLKDEV = 1 -a ! -b $LUNDEV ]
-					then
-						$ECHO "  $LUNDEV is invalid.  Try again."
-						continue
-					fi
-
-					DEVLIST="${DEVLIST};${LUNDEV}"
-					break
-				done
+				DEVLIST="${DEVLIST};${LUNDEV}"
+				break
+			    done
 			else
-				$ECHO "\n  The drive will be treated as a single disc player."
-				CHGMETHOD=0
-				NUMDISCS=1
-				MULTIPLAY=False
-				DEVLIST=$XMCD_DEV
+			    $ECHO \
+			    "\n  The drive will be treated as a single disc player."
+			    CHGMETHOD=0
+			    NUMDISCS=1
+			    MULTIPLAY=False
+			    DEVLIST=$XMCD_DEV
 			fi
 			;;
 		3)
