@@ -1,7 +1,7 @@
 /*
  *   libdi - CD Audio Player Device Interface Library
  *
- *   Copyright (C) 1993-1997  Ti Kan
+ *   Copyright (C) 1993-1998  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -29,7 +29,7 @@
  */
 
 #ifndef LINT
-static char *_fbioc_c_ident_ = "@(#)fbioc.c	6.81 97/11/30";
+static char *_fbioc_c_ident_ = "@(#)fbioc.c	6.94 98/05/18";
 #endif
 
 #include "common_d/appenv.h"
@@ -394,7 +394,7 @@ fbioc_rdtoc(struct ioc_toc_header *h, struct cd_toc_entry **e, int start)
 			continue;
 
 		j = i - start;
-		k = (fbioc_bcd_hack ? bcdtol(i) : i) - start;
+		k = (fbioc_bcd_hack ? util_bcdtol(i) : i) - start;
 
 		if (i == (h->ending_track + 1)) {
 			/* FreeBSD HACK: Fix up lead-out track number */
@@ -417,14 +417,15 @@ fbioc_rdtoc(struct ioc_toc_header *h, struct cd_toc_entry **e, int start)
 		}
 		else {
 			if (fbioc_bcd_hack)
-				(*e)[k].track = (byte_t) bcdtol(t[j].track);
+				(*e)[k].track = (byte_t)
+					util_bcdtol(t[j].track);
 			else
 				(*e)[k].track = t[j].track;
 		}
 	}
 
 	if (fbioc_bcd_hack)
-		h->ending_track = (byte_t) bcdtol(h->ending_track);
+		h->ending_track = (byte_t) util_bcdtol(h->ending_track);
 
 	MEM_FREE(t);
 
@@ -630,10 +631,10 @@ fbioc_play_trkidx(int start_trk, int start_idx, int end_trk, int end_idx)
 
 	if (fbioc_bcd_hack) {
 		/* Hack: BUGLY CD-ROM firmware */
-		t.start_track = ltobcd(start_trk);
-		t.start_index = ltobcd(start_idx);
-		t.end_track = ltobcd(end_trk);
-		t.end_index = ltobcd(end_idx);
+		t.start_track = util_ltobcd(start_trk);
+		t.start_index = util_ltobcd(start_idx);
+		t.end_track = util_ltobcd(end_trk);
+		t.end_index = util_ltobcd(end_idx);
 	}
 	else {
 		t.start_track = start_trk;
@@ -820,15 +821,14 @@ fbioc_do_playaudio(
  *	some frames.
  *
  * Args:
- *	s - Pointer to the curstat_t structure
  *	blk - Fault frame address
  *
  * Return:
  *	TRUE - success
  *	FALSE - failure
  */
-bool_t
-fbioc_fail_recov(curstat_t *s, word32_t blk)
+STATIC bool_t
+fbioc_fail_recov(word32_t blk)
 {
 	msf_t		recov_start_msf;
 	word32_t	recov_start_addr;
@@ -837,12 +837,12 @@ fbioc_fail_recov(curstat_t *s, word32_t blk)
 	ret = TRUE;
 
 	recov_start_addr = blk + ERR_SKIPBLKS;
-	blktomsf(
+	util_blktomsf(
 		recov_start_addr,
 		&recov_start_msf.min,
 		&recov_start_msf.sec,
 		&recov_start_msf.frame,
-		MSF_OFFSET(s)
+		MSF_OFFSET
 	);
 
 	/* Check to see if we have skipped past
@@ -865,7 +865,7 @@ fbioc_fail_recov(curstat_t *s, word32_t blk)
 
 	if (ret) {
 		/* Restart playback */
-		(void) fprintf(errfp, "CD audio: %s (%02u:%02u.%02u%d)\n",
+		(void) fprintf(errfp, "CD audio: %s (%02u:%02u.%02u)\n",
 				app_data.str_recoverr,
 				recov_start_msf.min,
 				recov_start_msf.sec,
@@ -939,15 +939,51 @@ fbioc_get_playstatus(curstat_t *s)
 	}
 
 	/* Check the subchannel data */
-	audio_status = sub.header.audio_status;
+
+#ifdef NETBSD_OLDIOC
+	s->cur_tot_min = sub.what.position.absaddr[1];
+	s->cur_tot_sec = sub.what.position.absaddr[2];
+	s->cur_tot_frame = sub.what.position.absaddr[3];
+#else
+	s->cur_tot_min = sub.what.position.absaddr.msf.minute;
+	s->cur_tot_sec = sub.what.position.absaddr.msf.second;
+	s->cur_tot_frame = sub.what.position.absaddr.msf.frame;
+#endif
+	util_msftoblk(
+		s->cur_tot_min,
+		s->cur_tot_sec,
+		s->cur_tot_frame,
+		&s->cur_tot_addr,
+		MSF_OFFSET
+	);
+
+#ifdef NETBSD_OLDIOC
+	s->cur_trk_min = sub.what.position.reladdr[1];
+	s->cur_trk_sec = sub.what.position.reladdr[2];
+	s->cur_trk_frame = sub.what.position.reladdr[3];
+#else
+	s->cur_trk_min = sub.what.position.reladdr.msf.minute;
+	s->cur_trk_sec = sub.what.position.reladdr.msf.second;
+	s->cur_trk_frame = sub.what.position.reladdr.msf.frame;
+#endif
+	util_msftoblk(
+		s->cur_trk_min,
+		s->cur_trk_sec,
+		s->cur_trk_frame,
+		&s->cur_trk_addr,
+		0
+	);
+
+	/* Update time display */
+	DPY_TIME(s, FALSE);
 
 	/* Hack: to work around firmware anomalies
 	 * in some CD-ROM drives.
 	 */
 	if (fbioc_bcd_hack) {
 		/* Hack: BUGLY CD-ROM firmware */
-		curtrk = bcdtol(sub.what.position.track_number);
-		curidx = bcdtol(sub.what.position.index_number);
+		curtrk = util_bcdtol(sub.what.position.track_number);
+		curidx = util_bcdtol(sub.what.position.index_number);
 	}
 	else {
 		curtrk = sub.what.position.track_number;
@@ -967,42 +1003,7 @@ fbioc_get_playstatus(curstat_t *s)
 		DPY_INDEX(s);
 	}
 
-#ifdef NETBSD_OLDIOC
-	s->cur_tot_min = sub.what.position.absaddr[1];
-	s->cur_tot_sec = sub.what.position.absaddr[2];
-	s->cur_tot_frame = sub.what.position.absaddr[3];
-#else
-	s->cur_tot_min = sub.what.position.absaddr.msf.minute;
-	s->cur_tot_sec = sub.what.position.absaddr.msf.second;
-	s->cur_tot_frame = sub.what.position.absaddr.msf.frame;
-#endif
-	msftoblk(
-		s->cur_tot_min,
-		s->cur_tot_sec,
-		s->cur_tot_frame,
-		&s->cur_tot_addr,
-		MSF_OFFSET(s)
-	);
-
-#ifdef NETBSD_OLDIOC
-	s->cur_trk_min = sub.what.position.reladdr[1];
-	s->cur_trk_sec = sub.what.position.reladdr[2];
-	s->cur_trk_frame = sub.what.position.reladdr[3];
-#else
-	s->cur_trk_min = sub.what.position.reladdr.msf.minute;
-	s->cur_trk_sec = sub.what.position.reladdr.msf.second;
-	s->cur_trk_frame = sub.what.position.reladdr.msf.frame;
-#endif
-	msftoblk(
-		s->cur_trk_min,
-		s->cur_trk_sec,
-		s->cur_trk_frame,
-		&s->cur_trk_addr,
-		0
-	);
-
-	/* Update time display */
-	DPY_TIME(s, FALSE);
+	audio_status = sub.header.audio_status;
 
 	/* Hack: to work around the fact that some CD-ROM drives
 	 * return CD_AS_PLAY_PAUSED status after issuing a Stop Unit command.
@@ -1061,7 +1062,7 @@ fbioc_get_playstatus(curstat_t *s)
 
 		errblk = s->cur_tot_addr;
 
-		if (!done && fbioc_fail_recov(s, errblk)) {
+		if (!done && fbioc_fail_recov(errblk)) {
 			in_fbioc_get_playstatus = FALSE;
 			return TRUE;
 		}
@@ -1306,12 +1307,12 @@ fbioc_get_toc(curstat_t *s)
 		s->trkinfo[i].sec = p->addr.msf.second;
 		s->trkinfo[i].frame = p->addr.msf.frame;
 #endif
-		msftoblk(
+		util_msftoblk(
 			s->trkinfo[i].min,
 			s->trkinfo[i].sec,
 			s->trkinfo[i].frame,
 			&s->trkinfo[i].addr,
-			MSF_OFFSET(s)
+			MSF_OFFSET
 		);
 
 		if (p->track == LEAD_OUT_TRACK || i == (MAXTRACK - 1)) {
@@ -1700,38 +1701,44 @@ fbioc_run_rew(curstat_t *s)
 	if (fbioc_start_search) {
 		fbioc_start_search = FALSE;
 		seq = 0;
-		if (skip_blks < addr)
-			start_addr = addr - skip_blks;
-		else
-			start_addr = 0;
+		i = (int) (addr - skip_blks);
 	}
 	else {
 		if (app_data.skip_spdup > 0 && seq > app_data.skip_spdup)
 			/* Speed up search */
 			skip_blks *= 3;
 
-		if ((int) (start_addr - skip_blks) > 0)
-			start_addr -= skip_blks;
-		else
-			start_addr = 0;
+		i = (int) (start_addr - skip_blks);
 	}
 
+	start_addr = (word32_t) (i > CLIP_FRAMES) ? i : CLIP_FRAMES;
+
 	seq++;
 
 	if (s->shuffle || s->program) {
 		if ((i = di_curtrk_pos(s)) < 0)
 			i = 0;
-	}
-	else
-		i = 0;
 
-	if (start_addr < s->trkinfo[i].addr)
-		start_addr = s->trkinfo[i].addr;
+		if (start_addr < s->trkinfo[i].addr)
+			start_addr = s->trkinfo[i].addr;
+	}
 
 	end_addr = start_addr + MAX_SRCH_BLKS;
 
-	blktomsf(start_addr, &smsf.min, &smsf.sec, &smsf.frame, MSF_OFFSET(s));
-	blktomsf(end_addr, &emsf.min, &emsf.sec, &emsf.frame, MSF_OFFSET(s));
+	util_blktomsf(
+		start_addr,
+		&smsf.min,
+		&smsf.sec,
+		&smsf.frame,
+		MSF_OFFSET
+	);
+	util_blktomsf(
+		end_addr,
+		&emsf.min,
+		&emsf.sec,
+		&emsf.frame,
+		MSF_OFFSET
+	);
 
 	/* Play next search interval */
 	(void) fbioc_do_playaudio(
@@ -1835,8 +1842,20 @@ fbioc_run_ff(curstat_t *s)
 		start_addr = end_addr - skip_blks;
 	}
 
-	blktomsf(start_addr, &smsf.min, &smsf.sec, &smsf.frame, MSF_OFFSET(s));
-	blktomsf(end_addr, &emsf.min, &emsf.sec, &emsf.frame, MSF_OFFSET(s));
+	util_blktomsf(
+		start_addr,
+		&smsf.min,
+		&smsf.sec,
+		&smsf.frame,
+		MSF_OFFSET
+	);
+	util_blktomsf(
+		end_addr,
+		&emsf.min,
+		&emsf.sec,
+		&emsf.frame,
+		MSF_OFFSET
+	);
 
 	/* Play next search interval */
 	(void) fbioc_do_playaudio(
@@ -1924,19 +1943,19 @@ fbioc_run_sample(curstat_t *s)
 		saddr = s->trkinfo[fbioc_next_sam].addr;
 		eaddr = saddr + app_data.sample_blks,
 
-		blktomsf(
+		util_blktomsf(
 			saddr,
 			&smsf.min,
 			&smsf.sec,
 			&smsf.frame,
-			MSF_OFFSET(s)
+			MSF_OFFSET
 		);
-		blktomsf(
+		util_blktomsf(
 			eaddr,
 			&emsf.min,
 			&emsf.sec,
 			&emsf.frame,
-			MSF_OFFSET(s)
+			MSF_OFFSET
 		);
 
 		if (fbioc_do_playaudio(ADDR_BLK | ADDR_MSF,
@@ -2016,12 +2035,12 @@ fbioc_run_prog(curstat_t *s)
 	s->cur_idx = 1;
 
 	start_addr = s->trkinfo[i].addr + s->cur_trk_addr;
-	blktomsf(
+	util_blktomsf(
 		start_addr,
 		&s->cur_tot_min,
 		&s->cur_tot_sec,
 		&s->cur_tot_frame,
-		MSF_OFFSET(s)
+		MSF_OFFSET
 	);
 	start_msf.min = s->cur_tot_min;
 	start_msf.sec = s->cur_tot_sec;
@@ -2303,9 +2322,13 @@ void
 fbioc_repeat(curstat_t *s, bool_t enable)
 {
 	s->repeat = enable;
-	DPY_RPTCNT(s);
-	if (!enable)
+
+	if (!enable && fbioc_new_progshuf) {
 		fbioc_new_progshuf = FALSE;
+		if (s->rptcnt > 0)
+			s->rptcnt--;
+	}
+	DPY_RPTCNT(s);
 }
 
 
@@ -2476,12 +2499,12 @@ fbioc_ab(curstat_t *s)
 		    app_data.min_playblks) {
 			fbioc_ab_end_addr = fbioc_ab_start_addr +
 					    app_data.min_playblks;
-			blktomsf(
+			util_blktomsf(
 				fbioc_ab_end_addr,
 				&fbioc_ab_end_msf.min,
 				&fbioc_ab_end_msf.sec,
 				&fbioc_ab_end_msf.frame,
-				MSF_OFFSET(s)
+				MSF_OFFSET
 			);
 		}
 
@@ -2673,11 +2696,12 @@ fbioc_play_pause(curstat_t *s)
 				s->cur_trk = s->first_trk;
 				start_addr = s->trkinfo[0].addr +
 					     s->cur_trk_addr;
-				blktomsf(start_addr,
-					 &start_msf.min,
-					 &start_msf.sec,
-					 &start_msf.frame,
-					 MSF_OFFSET(s)
+				util_blktomsf(
+					start_addr,
+					&start_msf.min,
+					&start_msf.sec,
+					&start_msf.frame,
+					MSF_OFFSET
 				);
 			}
 			else {
@@ -2686,11 +2710,12 @@ fbioc_play_pause(curstat_t *s)
 					     s->cur_trk_addr;
 			}
 
-			blktomsf(start_addr,
-				 &start_msf.min,
-				 &start_msf.sec,
-				 &start_msf.frame,
-				 MSF_OFFSET(s)
+			util_blktomsf(
+				start_addr,
+				&start_msf.min,
+				&start_msf.sec,
+				&start_msf.frame,
+				MSF_OFFSET
 			);
 
 			end_msf.min = s->tot_min;
@@ -2899,24 +2924,34 @@ fbioc_prevtrk(curstat_t *s)
 		else
 			i = di_curtrk_pos(s);
 
-		if (i < 0)
-			return;
-
-		start_addr = s->trkinfo[i].addr;
-		start_msf.min = s->trkinfo[i].min;
-		start_msf.sec = s->trkinfo[i].sec;
-		start_msf.frame = s->trkinfo[i].frame;
-		s->cur_trk = s->trkinfo[i].trkno;
-		s->cur_idx = 1;
-
-		/* If the current track has been playing for less
-		 * than app_data.prev_threshold blocks, then go
-		 * to the beginning of the previous track (if we
-		 * are not already on the first track).
-		 */
 		go_prev = FALSE;
-		if ((s->cur_tot_addr - start_addr) <= app_data.prev_threshold)
-			go_prev = TRUE;
+
+		if (i == 0 && s->cur_idx == 0) {
+			i = 0;
+			start_addr = CLIP_FRAMES;
+			start_msf.min = 0;
+			start_msf.sec = 2;
+			start_msf.frame = CLIP_FRAMES;
+			s->cur_trk = s->trkinfo[i].trkno;
+			s->cur_idx = 0;
+		}
+		else {
+			start_addr = s->trkinfo[i].addr;
+			start_msf.min = s->trkinfo[i].min;
+			start_msf.sec = s->trkinfo[i].sec;
+			start_msf.frame = s->trkinfo[i].frame;
+			s->cur_trk = s->trkinfo[i].trkno;
+			s->cur_idx = 1;
+
+			/* If the current track has been playing for less
+			 * than app_data.prev_threshold blocks, then go
+			 * to the beginning of the previous track (if we
+			 * are not already on the first track).
+			 */
+			if ((s->cur_tot_addr - start_addr) <=
+			    app_data.prev_threshold)
+				go_prev = TRUE;
+		}
 
 		if (go_prev) {
 			if (s->shuffle || s->program) {
@@ -2933,6 +2968,16 @@ fbioc_prevtrk(curstat_t *s)
 				start_msf.frame = s->trkinfo[i].frame;
 				s->cur_trk = s->trkinfo[i].trkno;
 			}
+			else if (i == 0) {
+				/* Go to the very beginning: this may be
+				 * a lead-in area before the start of track 1.
+				 */
+				start_addr = CLIP_FRAMES;
+				start_msf.min = 0;
+				start_msf.sec = 2;
+				start_msf.frame = CLIP_FRAMES;
+				s->cur_trk = s->trkinfo[i].trkno;
+			}
 			else if (i > 0) {
 				i--;
 
@@ -3095,7 +3140,9 @@ fbioc_nexttrk(curstat_t *s)
 		/* Find next track */
 		if ((i = di_curtrk_pos(s)) < 0)
 			return;
-		i++;
+
+		if (i > 0 || s->cur_idx > 0)
+			i++;
 
 		/* Skip over data tracks */
 		while (i < MAXTRACK && s->trkinfo[i].type == TYP_DATA)
