@@ -1,7 +1,7 @@
 /*
  *   libdi - CD Audio Player Device Interface Library
  *
- *   Copyright (C) 1993-1997  Ti Kan
+ *   Copyright (C) 1993-1998  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -19,7 +19,7 @@
  *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 #ifndef LINT
-static char *_libdi_c_ident_ = "@(#)libdi.c	6.78 97/07/27";
+static char *_libdi_c_ident_ = "@(#)libdi.c	6.102 98/05/15";
 #endif
 
 #include "common_d/appenv.h"
@@ -58,6 +58,9 @@ di_client_t		*di_clinfo;
 /* Device list table */
 char			**di_devlist;
 
+/* Web search engines list */
+srcheng_ent_t		*srcheng_list_head;
+
 /* Device lock file */
 STATIC char		lockfile[FILE_PATH_SZ];
 
@@ -65,6 +68,7 @@ STATIC char		lockfile[FILE_PATH_SZ];
 STATIC di_devreg_t	*di_reghead;
 
 
+
 /*
  * di_boolstr
  *	Return the string "False" or "True" depending upon the
@@ -106,6 +110,27 @@ di_prncfg(void)
 	(void) fprintf(errfp, "\tcddbMailCmd:\t\t\t%s\n",
 		       app_data.cddb_mailcmd);
 
+	(void) fprintf(errfp, "\nX resources:\n");
+
+	(void) fprintf(errfp, "\tversion:\t\t\t%s\n",
+		       app_data.version == NULL ? "(undef)" : app_data.version);
+	(void) fprintf(errfp, "\tmainWindowMode:\t\t\t%d\n",
+		       app_data.main_mode);
+	(void) fprintf(errfp, "\tnormalMainWidth:\t\t%d\n",
+		       app_data.normal_width);
+	(void) fprintf(errfp, "\tnormalMainHeight:\t\t%d\n",
+		       app_data.normal_height);
+	(void) fprintf(errfp, "\tbasicMainWidth:\t\t\t%d\n",
+		       app_data.basic_width);
+	(void) fprintf(errfp, "\tbasicMainHeight:\t\t%d\n",
+		       app_data.basic_height);
+	(void) fprintf(errfp, "\tdisplayBlinkOnInterval:\t\t%d\n",
+		       app_data.blinkon_interval);
+	(void) fprintf(errfp, "\tdisplayBlinkOffInterval:\t%d\n",
+		       app_data.blinkoff_interval);
+	(void) fprintf(errfp, "\tmainShowFocus:\t\t\t%s\n",
+		       di_boolstr(app_data.main_showfocus));
+
 	(void) fprintf(errfp, "\nCommon parameters:\n");
 
 	(void) fprintf(errfp, "\tdevice:\t\t\t\t%s\n",
@@ -117,6 +142,24 @@ di_prncfg(void)
 	(void) fprintf(errfp, "\tproxyServer:\t\t\t%s\n",
 		       app_data.proxy_server == NULL ?
 		           "(undef)" : app_data.proxy_server);
+	(void) fprintf(errfp, "\tbrowserDirect:\t\t\t%s\n",
+		       app_data.browser_dir == NULL ?
+		           "(undef)" : app_data.browser_dir);
+	(void) fprintf(errfp, "\tbrowserRemote:\t\t\t%s\n",
+		       app_data.browser_rmt == NULL ?
+		           "(undef)" : app_data.browser_rmt);
+	(void) fprintf(errfp, "\twebSite1URL:\t\t\t%s\n",
+		       app_data.site1_url == NULL ?
+		           "(undef)" : app_data.site1_url);
+	(void) fprintf(errfp, "\twebSite2URL:\t\t\t%s\n",
+		       app_data.site2_url == NULL ?
+		           "(undef)" : app_data.site2_url);
+	(void) fprintf(errfp, "\twebSite3URL:\t\t\t%s\n",
+		       app_data.site3_url == NULL ?
+		           "(undef)" : app_data.site3_url);
+	(void) fprintf(errfp, "\texcludeWords:\t\t\t%s\n",
+		       app_data.exclude_words == NULL ?
+		           "(undef)" : app_data.exclude_words);
 	(void) fprintf(errfp, "\tcddbDefaultPort:\t\t%d\n",
 		       app_data.cddb_port);
 	(void) fprintf(errfp, "\tcddbTimeOut:\t\t\t%d\n",
@@ -125,6 +168,12 @@ di_prncfg(void)
 		       app_data.stat_interval);
 	(void) fprintf(errfp, "\tinsertPollInterval:\t\t%d\n",
 		       app_data.ins_interval);
+	(void) fprintf(errfp, "\ttimeDisplayMode:\t\t%d\n",
+		       app_data.timedpy_mode);
+	(void) fprintf(errfp, "\ttooltipDelayInterval:\t\t%d\n",
+		       app_data.tooltip_delay);
+	(void) fprintf(errfp, "\ttooltipActiveInterval:\t\t%d\n",
+		       app_data.tooltip_time);
 	(void) fprintf(errfp, "\tinsertPollDisable:\t\t%s\n",
 		       di_boolstr(app_data.ins_disable));
 	(void) fprintf(errfp, "\tpreviousThreshold:\t\t%d\n",
@@ -143,20 +192,16 @@ di_prncfg(void)
 		       app_data.sample_blks);
 	(void) fprintf(errfp, "\tstartupVolume:\t\t\t%d\n",
 		       app_data.startup_vol);
-	(void) fprintf(errfp, "\tdisplayBlinkOnInterval:\t\t%d\n",
-		       app_data.blinkon_interval);
-	(void) fprintf(errfp, "\tdisplayBlinkOffInterval:\t%d\n",
-		       app_data.blinkoff_interval);
 	(void) fprintf(errfp, "\tcddbRemoteAutoSave:\t\t%s\n",
 		       di_boolstr(app_data.cddb_rmtautosave));
 	(void) fprintf(errfp, "\tcddbUseHttpProxy:\t\t%s\n",
 		       di_boolstr(app_data.http_proxy));
-	(void) fprintf(errfp, "\tmainShowFocus:\t\t\t%s\n",
-		       di_boolstr(app_data.main_showfocus));
 	(void) fprintf(errfp, "\tshowScsiErrMsg:\t\t\t%s\n",
 		       di_boolstr(app_data.scsierr_msg));
 	(void) fprintf(errfp, "\tsolaris2VolumeManager:\t\t%s\n",
 		       di_boolstr(app_data.sol2_volmgt));
+	(void) fprintf(errfp, "\ttooltipEnable:\t\t\t%s\n",
+		       di_boolstr(app_data.tooltip_enable));
 	(void) fprintf(errfp, "\tdebug:\t\t\t\t%s\n",
 		       di_boolstr(app_data.debug));
 
@@ -973,6 +1018,150 @@ di_dump_curstat(curstat_t *s)
 }
 
 
+/*
+ * di_add_srcheng_ent
+ *	Add an entry to the search engines information list.
+ *
+ * Args:
+ *	def - The search engine definition string from common.cfg
+ *
+ * Return:
+ *	Nothing.
+ */
+void
+di_add_srcheng_ent(char *def)
+{
+	char			*cp1,
+				*cp2,
+				*cp3,
+				*name,
+				*action;
+	srcheng_ent_t		*p;
+	bool_t			line_ent,
+				new_ent;
+	static srcheng_ent_t	*lastp;
+
+	if ((cp1 = strchr(def, ':')) == NULL) {
+		/* Bad definition */
+		return;
+	}
+
+	name = def;
+	action = NULL;
+	*cp1 = '\0';
+	p = NULL;
+
+	line_ent = (bool_t) (strcmp(name, "-") == 0);
+	if (!line_ent) {
+		cp2 = cp1 + 1;
+
+		/* Skip white space */
+		while (*cp2 == ' ' || *cp2 == '\t')
+			cp2++;
+
+		if (*cp2 == '\0' || *cp2 == '\n') {
+			/* Bad definition */
+			*cp1 = ':';
+			return;
+		}
+
+		action = cp2;
+
+		/* Find an existing entry that matches the name */
+		for (p = srcheng_list_head; p != NULL; p = p->next) {
+			if (strcmp(name, p->name) == 0)
+				break;
+		}
+	}
+
+	new_ent = FALSE;
+	if (p == NULL) {
+		/* Allocate a new list entry */
+		p = (srcheng_ent_t *)(void *) MEM_ALLOC(sizeof(srcheng_ent_t));
+		if (p == NULL) {
+			DI_FATAL(app_data.str_nomemory);
+			return;
+		}
+		p->action = NULL;
+		p->aux = NULL;
+		new_ent = TRUE;
+	}
+
+	if (app_data.debug) {
+		if (line_ent)
+			(void) fprintf(errfp, "Add    ---\n");
+		else
+			(void) fprintf(errfp, "%s %s: %s\n",
+				new_ent ? "Add   " : "Update",
+				name, action);
+	}
+
+	/* Name */
+	p->name = (char *) MEM_ALLOC(strlen(name) + 1);
+	if (p->name == NULL) {
+		DI_FATAL(app_data.str_nomemory);
+		return;
+	}
+	(void) strcpy(p->name, name);
+
+	if (!line_ent) {
+		/* Action URL */
+		if (p->action != NULL)
+			MEM_FREE(p->action);
+		p->action = (char *) MEM_ALLOC(strlen(action) + 1);
+		if (p->action == NULL) {
+			DI_FATAL(app_data.str_nomemory);
+			return;
+		}
+		(void) strcpy(p->action, action);
+
+		/* Scan URL action string to get a count of the number of
+		 * substitutions needed.
+		 */
+		p->acnt = p->dcnt = p->tcnt = 0;
+		for (cp3 = action; *cp3 != '\0'; cp3++) {
+			if (*cp3 == '%') {
+				switch ((int) *(++cp3)) {
+				case 'A':
+				case 'a':
+					p->acnt++;
+					break;
+				case 'D':
+				case 'd':
+					p->dcnt++;
+					break;
+				case 'T':
+				case 't':
+					p->tcnt++;
+					break;
+				case 'B':
+				case 'b':
+					p->acnt++;
+					p->dcnt++;
+					break;
+				default:
+					break;
+				}
+			}
+		}
+	}
+
+	/* Restore character */
+	*cp1 = ':';
+
+	if (new_ent) {
+		/* Add new entry to list */
+		p->next = NULL;
+		if (srcheng_list_head == NULL)
+			srcheng_list_head = lastp = p;
+		else {
+			lastp->next = p;
+			lastp = p;
+		}
+	}
+}
+
+
 /*
  * di_common_parmload
  *	Load the common configuration file and initialize parameters.
@@ -1002,7 +1191,7 @@ di_common_parmload(char *path, bool_t priv)
 
 
 	if (priv && di_isdemo())
-		app_data.device = "(none)";
+		app_data.device = "(sim1)";
 
 	if (PIPE(pfd) < 0) {
 		DBGPRN(errfp, "di_common_parmload: pipe failed (errno=%d)\n",
@@ -1022,10 +1211,11 @@ di_common_parmload(char *path, bool_t priv)
 		(void) close(pfd[0]);
 
 		DBGPRN(errfp, "Setting uid to %d, gid to %d\n",
-			(int) get_ouid(), (int) get_ogid());
+			(int) util_get_ouid(), (int) util_get_ogid());
 
 		/* Force uid and gid to original setting */
-		if (setuid(get_ouid()) < 0 || setgid(get_ogid()) < 0) {
+		if (setuid(util_get_ouid()) < 0 ||
+		    setgid(util_get_ogid()) < 0) {
 			(void) close(pfd[1]);
 			exit(1);
 		}
@@ -1120,11 +1310,13 @@ di_common_parmload(char *path, bool_t priv)
 
 	/* Read in common parameters */
 	while (fgets(buf, PARM_BUF_SZ, fp) != NULL) {
-		/* Skip comments */
+		/* Skip comments and blank lines */
 		if (buf[0] == '#' || buf[0] == '!' || buf[0] == '\n')
 			continue;
 
-		if (!di_isdemo() && sscanf(buf, "device: %s\n", parm) > 0) {
+		if (sscanf(buf, "device: %[^\n]\n", parm) > 0) {
+			if (di_isdemo())
+				continue;
 			if (priv && app_data.device != NULL)
 				dev_cmdline = TRUE;
 			if (!dev_cmdline) {
@@ -1142,7 +1334,7 @@ di_common_parmload(char *path, bool_t priv)
 			}
 			continue;
 		}
-		if (sscanf(buf, "cddbPath: %s\n", parm) > 0) {
+		if (sscanf(buf, "cddbPath: %[^\n]\n", parm) > 0) {
 			if (!priv || app_data.cddb_path == NULL) {
 				if (app_data.cddb_path != NULL)
 					MEM_FREE(app_data.cddb_path);
@@ -1158,15 +1350,96 @@ di_common_parmload(char *path, bool_t priv)
 			}
 			continue;
 		}
-		if (sscanf(buf, "proxyServer: %s\n", parm) > 0) {
+		if (sscanf(buf, "proxyServer: %[^\n]\n", parm) > 0) {
+			if (app_data.proxy_server != NULL)
+				MEM_FREE(app_data.proxy_server);
 			app_data.proxy_server = (char *) MEM_ALLOC(
 				strlen(parm) + 1
 			);
-			if (app_data.cddb_path == NULL) {
+			if (app_data.proxy_server == NULL) {
 				DI_FATAL(app_data.str_nomemory);
 				return;
 			}
 			(void) strcpy(app_data.proxy_server, parm);
+			continue;
+		}
+		if (sscanf(buf, "browserDirect: %[^\n]\n", parm) > 0) {
+			if (app_data.browser_dir != NULL)
+				MEM_FREE(app_data.browser_dir);
+			app_data.browser_dir = (char *) MEM_ALLOC(
+				strlen(parm) + 1
+			);
+			if (app_data.browser_dir == NULL) {
+				DI_FATAL(app_data.str_nomemory);
+				return;
+			}
+			(void) strcpy(app_data.browser_dir, parm);
+			continue;
+		}
+		if (sscanf(buf, "browserRemote: %[^\n]\n", parm) > 0) {
+			if (app_data.browser_rmt != NULL)
+				MEM_FREE(app_data.browser_rmt);
+			app_data.browser_rmt = (char *) MEM_ALLOC(
+				strlen(parm) + 1
+			);
+			if (app_data.browser_rmt == NULL) {
+				DI_FATAL(app_data.str_nomemory);
+				return;
+			}
+			(void) strcpy(app_data.browser_rmt, parm);
+			continue;
+		}
+		if (sscanf(buf, "webSite1URL: %[^\n]\n", parm) > 0) {
+			if (app_data.site1_url != NULL)
+				MEM_FREE(app_data.site1_url);
+			app_data.site1_url = (char *) MEM_ALLOC(
+				strlen(parm) + 1
+			);
+			if (app_data.site1_url == NULL) {
+				DI_FATAL(app_data.str_nomemory);
+				return;
+			}
+			(void) strcpy(app_data.site1_url, parm);
+			continue;
+		}
+		if (sscanf(buf, "webSite2URL: %[^\n]\n", parm) > 0) {
+			if (app_data.site2_url != NULL)
+				MEM_FREE(app_data.site2_url);
+			app_data.site2_url = (char *) MEM_ALLOC(
+				strlen(parm) + 1
+			);
+			if (app_data.site2_url == NULL) {
+				DI_FATAL(app_data.str_nomemory);
+				return;
+			}
+			(void) strcpy(app_data.site2_url, parm);
+			continue;
+		}
+		if (sscanf(buf, "webSite3URL: %[^\n]\n", parm) > 0) {
+			if (app_data.site3_url != NULL)
+				MEM_FREE(app_data.site3_url);
+			app_data.site3_url = (char *) MEM_ALLOC(
+				strlen(parm) + 1
+			);
+			if (app_data.site3_url == NULL) {
+				DI_FATAL(app_data.str_nomemory);
+				return;
+			}
+			(void) strcpy(app_data.site3_url, parm);
+			continue;
+		}
+		if (sscanf(buf, "excludeWords: %[^\n]\n", parm) > 0) {
+			if (app_data.exclude_words != NULL)
+				MEM_FREE(app_data.exclude_words);
+			app_data.exclude_words = (char *) MEM_ALLOC(
+				strlen(parm) + 1
+			);
+			if (app_data.exclude_words == NULL) {
+				DI_FATAL(app_data.str_nomemory);
+				return;
+			}
+			(void) strcpy(app_data.exclude_words, parm);
+			continue;
 		}
 		if (sscanf(buf, "cddbDefaultPort: %s\n", parm) > 0) {
 			app_data.cddb_port = atoi(parm);
@@ -1177,11 +1450,11 @@ di_common_parmload(char *path, bool_t priv)
 			continue;
 		}
 		if (sscanf(buf, "cddbRemoteAutoSave: %s\n", parm) > 0) {
-			app_data.cddb_rmtautosave = stob(parm);
+			app_data.cddb_rmtautosave = util_stob(parm);
 			continue;
 		}
 		if (sscanf(buf, "cddbUseHttpProxy: %s\n", parm) > 0) {
-			app_data.http_proxy = stob(parm);
+			app_data.http_proxy = util_stob(parm);
 			continue;
 		}
 		if (sscanf(buf, "cddbFileMode: %s\n", parm) > 0) {
@@ -1209,7 +1482,7 @@ di_common_parmload(char *path, bool_t priv)
 			continue;
 		}
 		if (sscanf(buf, "insertPollDisable: %s\n", parm) > 0) {
-			app_data.ins_disable = stob(parm);
+			app_data.ins_disable = util_stob(parm);
 			continue;
 		}
 		if (sscanf(buf, "previousThreshold: %s\n", parm) > 0) {
@@ -1220,29 +1493,60 @@ di_common_parmload(char *path, bool_t priv)
 			app_data.sample_blks = atoi(parm);
 			continue;
 		}
+		if (sscanf(buf, "timeDisplayMode: %s\n", parm) > 0) {
+			app_data.timedpy_mode = atoi(parm);
+			continue;
+		}
 		if (sscanf(buf, "solaris2VolumeManager: %s\n", parm) > 0) {
-			app_data.sol2_volmgt = stob(parm);
+			app_data.sol2_volmgt = util_stob(parm);
 			continue;
 		}
 		if (sscanf(buf, "showScsiErrMsg: %s\n", parm) > 0) {
-			app_data.scsierr_msg = stob(parm);
+			app_data.scsierr_msg = util_stob(parm);
 			continue;
 		}
 		if (sscanf(buf, "curfileEnable: %s\n", parm) > 0) {
-			app_data.write_curfile = stob(parm);
+			app_data.write_curfile = util_stob(parm);
+			continue;
+		}
+		if (sscanf(buf, "tooltipEnable: %s\n", parm) > 0) {
+			app_data.tooltip_enable = util_stob(parm);
+			continue;
+		}
+		if (sscanf(buf, "tooltipDelayInterval: %s\n", parm) > 0) {
+			app_data.tooltip_delay = atoi(parm);
+			continue;
+		}
+		if (sscanf(buf, "tooltipActiveInterval: %s\n", parm) > 0) {
+			app_data.tooltip_time = atoi(parm);
 			continue;
 		}
 		if (sscanf(buf, "debugMode: %s\n", parm) > 0) {
 			if (!force_debug)
-				app_data.debug = stob(parm);
+				app_data.debug = util_stob(parm);
 			continue;
 		}
+
+		/* If we get here, and there are more entries, then we
+		 * assume it's the search engines definitions.
+		 */
+		buf[strlen(buf)-1] = '\0';	/* Eat newline */
+		di_add_srcheng_ent(buf);
 	}
 
 	(void) fclose(fp);
 	MEM_FREE(buf);
 	MEM_FREE(parm);
 
+	/* In case of error */
+	if (app_data.timedpy_mode < 0 ||
+	    app_data.timedpy_mode >= TIMEDPY_MAX_MODES)
+		app_data.timedpy_mode = 0;
+	if (app_data.tooltip_delay < 0)
+		app_data.tooltip_delay = 1000;
+	if (app_data.tooltip_time < 0)
+		app_data.tooltip_delay = 3000;
+
 #ifndef __VMS
 	while ((ret = WAITPID(cpid, &stat_val, 0)) != cpid) {
 		if (ret < 0)
@@ -1255,13 +1559,23 @@ di_common_parmload(char *path, bool_t priv)
 			    "di_common_parmload: child exited (status=%d)\n",
 			    WEXITSTATUS(stat_val));
 			(void) sprintf(errstr, app_data.str_nocfg, path);
-			DI_WARNING(errstr);
+			if (priv && !di_isdemo()) {
+				DI_FATAL(errstr);
+			}
+			else {
+				DI_WARNING(errstr);
+			}
 		}
 	    }
 	    else if (WIFSIGNALED(stat_val)) {
 		DBGPRN(errfp, "di_common_parmload: child killed.\n");
 		(void) sprintf(errstr, app_data.str_nocfg, path);
-		DI_WARNING(errstr);
+		if (priv && !di_isdemo()) {
+			DI_FATAL(errstr);
+		}
+		else {
+			DI_WARNING(errstr);
+		}
 	    }
 	}
 #endif
@@ -1296,8 +1610,15 @@ di_devspec_parmload(char *path, bool_t priv)
 
 
 	if (priv && di_isdemo()) {
-		app_data.devlist =
-		    "(sim1);(sim2);(sim3);(sim4);(sim5);(sim6);(sim7);(sim8)";
+		char	*cp;
+
+		cp = "(sim1);(sim2);(sim3);(sim4);(sim5);(sim6);(sim7);(sim8)";
+		app_data.devlist = MEM_ALLOC(strlen(cp) + 1);
+		if (app_data.devlist == NULL) {
+			DI_FATAL(app_data.str_nomemory);
+			return;
+		}
+		strcpy(app_data.devlist, cp);
 		app_data.numdiscs = 8;
 		app_data.chg_method = CHG_SCSI_LUN;
 		app_data.multi_play = TRUE;
@@ -1321,10 +1642,11 @@ di_devspec_parmload(char *path, bool_t priv)
 		(void) close(pfd[0]);
 
 		DBGPRN(errfp, "Setting uid to %d, gid to %d\n",
-			(int) get_ouid(), (int) get_ogid());
+			(int) util_get_ouid(), (int) util_get_ogid());
 
 		/* Force uid and gid to original setting */
-		if (setuid(get_ouid()) < 0 || setgid(get_ogid()) < 0) {
+		if (setuid(util_get_ouid()) < 0 ||
+		    setgid(util_get_ogid()) < 0) {
 			(void) close(pfd[1]);
 			exit(1);
 		}
@@ -1476,72 +1798,72 @@ di_devspec_parmload(char *path, bool_t priv)
 			}
 			if (sscanf(buf, "playAudio10Support: %s\n",
 				   parm) > 0) {
-				app_data.play10_supp = stob(parm);
+				app_data.play10_supp = util_stob(parm);
 				continue;
 			}
 			if (sscanf(buf, "playAudio12Support: %s\n",
 				   parm) > 0) {
-				app_data.play12_supp = stob(parm);
+				app_data.play12_supp = util_stob(parm);
 				continue;
 			}
 			if (sscanf(buf, "playAudioMSFSupport: %s\n",
 				   parm) > 0) {
-				app_data.playmsf_supp = stob(parm);
+				app_data.playmsf_supp = util_stob(parm);
 				continue;
 			}
 			if (sscanf(buf, "playAudioTISupport: %s\n",
 				   parm) > 0) {
-				app_data.playti_supp = stob(parm);
+				app_data.playti_supp = util_stob(parm);
 				continue;
 			}
 			if (sscanf(buf, "loadSupport: %s\n",
 				   parm) > 0) {
-				app_data.load_supp = stob(parm);
+				app_data.load_supp = util_stob(parm);
 				continue;
 			}
 			if (sscanf(buf, "ejectSupport: %s\n",
 				   parm) > 0) {
-				app_data.eject_supp = stob(parm);
+				app_data.eject_supp = util_stob(parm);
 				continue;
 			}
 			if (sscanf(buf, "modeSenseSetDBD: %s\n",
 				   parm) > 0) {
-				app_data.msen_dbd = stob(parm);
+				app_data.msen_dbd = util_stob(parm);
 				continue;
 			}
 			if (sscanf(buf, "volumeControlSupport: %s\n",
 				   parm) > 0) {
-				app_data.mselvol_supp = stob(parm);
+				app_data.mselvol_supp = util_stob(parm);
 				continue;
 			}
 			if (sscanf(buf, "balanceControlSupport: %s\n",
 				   parm) > 0) {
-				app_data.balance_supp = stob(parm);
+				app_data.balance_supp = util_stob(parm);
 				continue;
 			}
 			if (sscanf(buf, "channelRouteSupport: %s\n",
 				   parm) > 0) {
-				app_data.chroute_supp = stob(parm);
+				app_data.chroute_supp = util_stob(parm);
 				continue;
 			}
 			if (sscanf(buf, "pauseResumeSupport: %s\n",
 				   parm) > 0) {
-				app_data.pause_supp = stob(parm);
+				app_data.pause_supp = util_stob(parm);
 				continue;
 			}
 			if (sscanf(buf, "caddyLockSupport: %s\n",
 				   parm) > 0) {
-				app_data.caddylock_supp = stob(parm);
+				app_data.caddylock_supp = util_stob(parm);
 				continue;
 			}
 			if (sscanf(buf, "curposFormat: %s\n",
 				   parm) > 0) {
-				app_data.curpos_fmt = stob(parm);
+				app_data.curpos_fmt = util_stob(parm);
 				continue;
 			}
 			if (sscanf(buf, "noTURWhenPlaying: %s\n",
 				   parm) > 0) {
-				app_data.play_notur = stob(parm);
+				app_data.play_notur = util_stob(parm);
 				continue;
 			}
 		}
@@ -1582,55 +1904,55 @@ di_devspec_parmload(char *path, bool_t priv)
 			continue;
 		}
 		if (sscanf(buf, "spinDownOnLoad: %s\n", parm) > 0) {
-			app_data.load_spindown = stob(parm);
+			app_data.load_spindown = util_stob(parm);
 			continue;
 		}
 		if (sscanf(buf, "playOnLoad: %s\n", parm) > 0) {
-			app_data.load_play = stob(parm);
+			app_data.load_play = util_stob(parm);
 			continue;
 		}
 		if (sscanf(buf, "ejectOnDone: %s\n", parm) > 0) {
-			app_data.done_eject = stob(parm);
+			app_data.done_eject = util_stob(parm);
 			continue;
 		}
 		if (sscanf(buf, "ejectOnExit: %s\n", parm) > 0) {
-			app_data.exit_eject = stob(parm);
+			app_data.exit_eject = util_stob(parm);
 			continue;
 		}
 		if (sscanf(buf, "stopOnExit: %s\n", parm) > 0) {
-			app_data.exit_stop = stob(parm);
+			app_data.exit_stop = util_stob(parm);
 			continue;
 		}
 		if (sscanf(buf, "exitOnEject: %s\n", parm) > 0) {
-			app_data.eject_exit = stob(parm);
+			app_data.eject_exit = util_stob(parm);
 			continue;
 		}
 		if (sscanf(buf, "closeOnEject: %s\n", parm) > 0) {
-			app_data.eject_close = stob(parm);
+			app_data.eject_close = util_stob(parm);
 			continue;
 		}
 		if (sscanf(buf, "caddyLock: %s\n", parm) > 0) {
-			app_data.caddy_lock = stob(parm);
+			app_data.caddy_lock = util_stob(parm);
 			continue;
 		}
 		if (sscanf(buf, "repeatMode: %s\n", parm) > 0) {
-			app_data.repeat_mode = stob(parm);
+			app_data.repeat_mode = util_stob(parm);
 			continue;
 		}
 		if (sscanf(buf, "shuffleMode: %s\n", parm) > 0) {
-			app_data.shuffle_mode = stob(parm);
+			app_data.shuffle_mode = util_stob(parm);
 			continue;
 		}
 		if (sscanf(buf, "multiPlay: %s\n", parm) > 0) {
-			app_data.multi_play = stob(parm);
+			app_data.multi_play = util_stob(parm);
 			continue;
 		}
 		if (sscanf(buf, "reversePlay: %s\n", parm) > 0) {
-			app_data.reverse = stob(parm);
+			app_data.reverse = util_stob(parm);
 			continue;
 		}
 		if (sscanf(buf, "internalSpeakerEnable: %s\n", parm) > 0) {
-			app_data.internal_spkr = stob(parm);
+			app_data.internal_spkr = util_stob(parm);
 			continue;
 		}
 	}
@@ -1651,13 +1973,23 @@ di_devspec_parmload(char *path, bool_t priv)
 			    "di_devspec_parmload: child exited (status=%d)\n",
 			    WEXITSTATUS(stat_val));
 			(void) sprintf(errstr, app_data.str_nocfg, path);
-			DI_WARNING(errstr);
+			if (priv && !di_isdemo()) {
+				DI_FATAL(errstr);
+			}
+			else {
+				DI_WARNING(errstr);
+			}
 		}
 	    }
 	    else if (WIFSIGNALED(stat_val)) {
 		DBGPRN(errfp, "di_devspec_parmload: child killed.\n");
 		(void) sprintf(errstr, app_data.str_nocfg, path);
-		DI_WARNING(errstr);
+		if (priv && !di_isdemo()) {
+			DI_FATAL(errstr);
+		}
+		else {
+			DI_WARNING(errstr);
+		}
 	    }
 	}
 #endif
@@ -1731,7 +2063,6 @@ di_devspec_parmsave(char *path)
 	int		ret;
 	pid_t		cpid;
 	waitret_t	stat_val;
-	struct stat	stbuf;
 	FILE		*fp;
 	char		*truestr = "True",
 			*falsestr = "False",
@@ -1778,40 +2109,19 @@ di_devspec_parmsave(char *path)
 	}
 
 	DBGPRN(errfp, "Setting uid to %d, gid to %d\n",
-		(int) get_ouid(), (int) get_ogid());
+		(int) util_get_ouid(), (int) util_get_ogid());
 
 	/* Force uid and gid to original setting */
-	if (setuid(get_ouid()) < 0 || setgid(get_ogid()) < 0)
+	if (setuid(util_get_ouid()) < 0 || setgid(util_get_ogid()) < 0)
 		exit(1);
 
 #ifndef __VMS
-	/*
-	 * Make sure directory exists.  If not, create it.
-	 */
-	dirpath = dirname(path);
-	if (stat(dirpath, &stbuf) < 0) {
-		if (errno == ENOENT) {
-			if (mkdir(dirpath, 0777) < 0) {
-				DBGPRN(errfp,
-				    "di_devspec_parmsave: cannot mkdir %s.\n",
-				    dirpath);
-				exit(2);
-			}
-		}
-		else {
-			DBGPRN(errfp,
-				"di_devspec_parmsave: cannot mkdir %s.\n",
-				dirpath);
-			exit(3);
-		}
-	}
-	else if (!S_ISDIR(stbuf.st_mode)) {
+	dirpath = util_dirname(path);
+	if (!util_mkdir(dirpath, 0755)) {
 		DBGPRN(errfp, "di_devspec_parmsave: cannot mkdir %s.\n",
 			dirpath);
-		exit(4);
+		exit(2);
 	}
-
-	(void) chmod(dirpath, 0755);
 #endif
 
 	DBGPRN(errfp, "Writing device-specific file %s\n", path);
@@ -1941,7 +2251,7 @@ di_devlock(curstat_t *s, char *path)
 				if (pid <= 0 ||
 				    (kill(pid, 0) < 0 && errno == ESRCH)) {
 					/* Pid died, steal its lockfile */
-					(void) unlink(lockfile);
+					(void) UNLINK(lockfile);
 				}
 				else {
 					/* Pid still running: clash */
@@ -1963,7 +2273,7 @@ di_devlock(curstat_t *s, char *path)
 	}
 #else
 /* OpenVMS */
-	char			ref_cnt = 1;
+	static char		ref_cnt;
 	int			status;
 
 	struct {
@@ -1985,6 +2295,8 @@ di_devlock(curstat_t *s, char *path)
 	if (path == NULL)
 		return TRUE;
 
+	ref_cnt = 1;
+
 	dev_nam_desc.dsc$b_class = DSC$K_CLASS_S;
 	dev_nam_desc.dsc$b_dtype = DSC$K_DTYPE_T;
 	dev_nam_desc.dsc$a_pointer = path;
@@ -1999,10 +2311,7 @@ di_devlock(curstat_t *s, char *path)
 		return FALSE;
 	}
 	
-	if (ref_cnt == 0)
-		return TRUE;
-	else
-		return FALSE;
+	return ((bool_t) (ref_cnt == 0));
 #endif	/* __VMS */
 }
 
@@ -2023,7 +2332,7 @@ di_devunlock(curstat_t *s)
 {
 #ifndef __VMS
 	if (lockfile[0] != '\0' && s->devlocked)
-		(void) unlink(lockfile);
+		(void) UNLINK(lockfile);
 #endif
 }
 
@@ -2424,10 +2733,10 @@ di_untaper_vol(int v)
 	switch (app_data.vol_taper) {
 	case 1:
 		/* squared taper */
-		return (isqrt(v) * 10);
+		return (util_isqrt(v) * 10);
 	case 2:
 		/* inverse-squared taper */
-		return (MAX_VOL - isqrt(SQR(MAX_VOL) - (MAX_VOL * v)));
+		return (MAX_VOL - util_isqrt(SQR(MAX_VOL) - (MAX_VOL * v)));
 	case 0:
 	default:
 		/* linear taper */
