@@ -1,7 +1,7 @@
 /*
  *   libdi - scsipt SCSI Device Interface Library
  *
- *   Copyright (C) 1993-1997  Ti Kan
+ *   Copyright (C) 1993-1998  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -19,7 +19,7 @@
  *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 #ifndef LINT
-static char *_os_cdsim_c_ident_ = "@(#)os_cdsim.c	6.13 97/04/09";
+static char *_os_cdsim_c_ident_ = "@(#)os_cdsim.c	6.16 98/05/10";
 #endif
 
 #include "common_d/appenv.h"
@@ -188,7 +188,7 @@ cdsim_sig(int sig)
  *	Command completion status code
  */
 /*ARGSUSED*/
-STATIC word32_t
+STATIC byte_t
 cdsim_s_test(simpkt_t *r, simpkt_t *s)
 {
 	if (cdsim_stat.status == CDSIM_NODISC)
@@ -371,15 +371,15 @@ cdsim_m_rdsubq(simpkt_t *r, simpkt_t *s)
 		s1->idxno = cdsim_stat.idxno;
 
 		if (r->cdb[1] & 0x02) {
-			blktomsf(
+			util_blktomsf(
 				cdsim_stat.absaddr,
 				&s1->abs_addr.msf.min,
 				&s1->abs_addr.msf.sec,
 				&s1->abs_addr.msf.frame,
-				FRAME_PER_SEC << 1
+				MSF_OFFSET
 			);
 
-			blktomsf(
+			util_blktomsf(
 				cdsim_stat.reladdr,
 				&s1->rel_addr.msf.min,
 				&s1->rel_addr.msf.sec,
@@ -388,8 +388,8 @@ cdsim_m_rdsubq(simpkt_t *r, simpkt_t *s)
 			);
 		}
 		else {
-			s1->abs_addr.logical = bswap32(cdsim_stat.absaddr);
-			s1->rel_addr.logical = bswap32(cdsim_stat.reladdr);
+			s1->abs_addr.logical = util_bswap32(cdsim_stat.absaddr);
+			s1->rel_addr.logical = util_bswap32(cdsim_stat.reladdr);
 		}
 
 		s->len = sizeof(subq_hdr_t) + sizeof(subq_01_t);
@@ -513,10 +513,20 @@ cdsim_m_play(simpkt_t *r, simpkt_t *s)
 STATIC byte_t
 cdsim_m_playmsf(simpkt_t *r, simpkt_t *s)
 {
-	msftoblk(r->cdb[3], r->cdb[4], r->cdb[5], &cdsim_stat.startaddr,
-		 FRAME_PER_SEC << 1);
-	msftoblk(r->cdb[6], r->cdb[7], r->cdb[8], &cdsim_stat.endaddr,
-		 FRAME_PER_SEC << 1);
+	util_msftoblk(	
+		r->cdb[3],
+		r->cdb[4],
+		r->cdb[5],
+		&cdsim_stat.startaddr,
+		MSF_OFFSET
+	);
+	util_msftoblk(
+		r->cdb[6],
+		r->cdb[7],
+		r->cdb[8],
+		&cdsim_stat.endaddr,
+		MSF_OFFSET
+	);
 
 	if (cdsim_stat.endaddr <= cdsim_stat.startaddr)
 		return CDSIM_PARMERR;
@@ -824,7 +834,7 @@ cdsim_init(void)
 	(void) strncpy((char *) cdsim_inqdata.revnum, CDSIM_VERS, 4);
 
 	/* Initialize TOC data */
-	h1->data_len = h2->data_len = bswap16(
+	h1->data_len = h2->data_len = util_bswap16(
 		((CDSIM_NTRKS + 1) * sizeof(toc_trk_descr_t)) + 2
 	);
 
@@ -841,18 +851,18 @@ cdsim_init(void)
 		t1->audioch = t2->audioch = 0;
 		t1->adr = t2->adr = 0;
 		t1->trkno = t2->trkno = i + 1;
-		t1->abs_addr.logical = bswap32(cdsim_stat.trk[i].addr);
-		blktomsf(
+		t1->abs_addr.logical = util_bswap32(cdsim_stat.trk[i].addr);
+		util_blktomsf(
 			cdsim_stat.trk[i].addr,
 			&t2->abs_addr.msf.min,
 			&t2->abs_addr.msf.sec,
 			&t2->abs_addr.msf.frame,
-			FRAME_PER_SEC << 1
+			MSF_OFFSET
 		);
 
-		t1 = (toc_trk_descr_t *)
+		t1 = (toc_trk_descr_t *)(void *)
 			((byte_t *)(void *) t1 + sizeof(toc_trk_descr_t));
-		t2 = (toc_trk_descr_t *)
+		t2 = (toc_trk_descr_t *)(void *)
 			((byte_t *)(void *) t2 + sizeof(toc_trk_descr_t));
 	}
 
@@ -863,13 +873,13 @@ cdsim_init(void)
 	t1->audioch = t2->audioch = 0;
 	t1->adr = t2->adr = 0;
 	t1->trkno = t2->trkno = LEAD_OUT_TRACK;
-	t1->abs_addr.logical = bswap32(cdsim_stat.trk[i].addr);
-	blktomsf(
+	t1->abs_addr.logical = util_bswap32(cdsim_stat.trk[i].addr);
+	util_blktomsf(
 		cdsim_stat.trk[i].addr,
 		&t2->abs_addr.msf.min,
 		&t2->abs_addr.msf.sec,
 		&t2->abs_addr.msf.frame,
-		FRAME_PER_SEC << 1
+		MSF_OFFSET
 	);
 }
 
