@@ -1,7 +1,7 @@
 /*
  *   libdi - CD Audio Player Device Interface Library
  *
- *   Copyright (C) 1993-1997  Ti Kan
+ *   Copyright (C) 1993-1998  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -24,7 +24,7 @@
 #ifdef DI_SCSIPT
 
 #ifndef LINT
-static char *_scsipt_h_ident_ = "@(#)scsipt.h	6.28 97/06/06";
+static char *_scsipt_h_ident_ = "@(#)scsipt.h	6.30 98/02/27";
 #endif
 
 
@@ -49,8 +49,9 @@ static char *_scsipt_h_ident_ = "@(#)scsipt.h	6.28 97/06/06";
 
 
 /* Data direction code */
-#define READ_OP			0	/* SCSI data-in */
-#define WRITE_OP		1	/* SCSI data-out */
+#define OP_NODATA		0	/* SCSI no data */
+#define OP_DATAIN		1	/* SCSI data-in */
+#define OP_DATAOUT		2	/* SCSI data-out */
 
 
 /* Default command timeout interval */
