@@ -1,7 +1,7 @@
 /*
  *   libdi - CD Audio Player Device Interface Library
  *
- *   Copyright (C) 1993-1997  Ti Kan
+ *   Copyright (C) 1993-1998  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -30,7 +30,7 @@
  */
 
 #ifndef LINT
-static char *_slioc_c_ident_ = "@(#)slioc.c	6.96 97/10/28";
+static char *_slioc_c_ident_ = "@(#)slioc.c	6.110 98/05/18";
 #endif
 
 #include "common_d/appenv.h"
@@ -541,7 +541,7 @@ slioc_rdtoc(struct cdrom_tochdr *h, struct cdrom_tocentry **e, int start)
 			continue;
 
 		j = i - start;
-		k = (slioc_bcd_hack ? bcdtol(i) : i) - start;
+		k = (slioc_bcd_hack ? util_bcdtol(i) : i) - start;
 
 		(*e)[k].cdte_adr = t[j].cdte_adr;
 		(*e)[k].cdte_ctrl = t[j].cdte_ctrl;
@@ -555,13 +555,13 @@ slioc_rdtoc(struct cdrom_tochdr *h, struct cdrom_tocentry **e, int start)
 		else {
 			if (slioc_bcd_hack)
 				(*e)[k].cdte_track = (byte_t)
-					bcdtol(t[j].cdte_track);
+					util_bcdtol(t[j].cdte_track);
 			else
 				(*e)[k].cdte_track = t[j].cdte_track;
 		}
 	}
 	if (slioc_bcd_hack)
-		h->cdth_trk1 = (byte_t) bcdtol(h->cdth_trk1);
+		h->cdth_trk1 = (byte_t) util_bcdtol(h->cdth_trk1);
 
 	MEM_FREE(t);
 
@@ -770,10 +770,10 @@ slioc_play_trkidx(int start_trk, int start_idx, int end_trk, int end_idx)
 
 	if (slioc_bcd_hack) {
 		/* Hack: BUGLY CD-ROM firmware */
-		t.cdti_trk0 = ltobcd(start_trk);
-		t.cdti_ind0 = ltobcd(start_idx);
-		t.cdti_trk1 = ltobcd(end_trk);
-		t.cdti_ind1 = ltobcd(end_idx);
+		t.cdti_trk0 = util_ltobcd(start_trk);
+		t.cdti_ind0 = util_ltobcd(start_idx);
+		t.cdti_trk1 = util_ltobcd(end_trk);
+		t.cdti_ind1 = util_ltobcd(end_idx);
 	}
 	else {
 		t.cdti_trk0 = start_trk;
@@ -969,15 +969,14 @@ slioc_do_playaudio(
  *	some frames.
  *
  * Args:
- *	s - Pointer to the curstat_t structure
  *	blk - Fault frame address
  *
  * Return:
  *	TRUE - success
  *	FALSE - failure
  */
-bool_t
-slioc_fail_recov(curstat_t *s, word32_t blk)
+STATIC bool_t
+slioc_fail_recov(word32_t blk)
 {
 	msf_t		recov_start_msf;
 	word32_t	recov_start_addr;
@@ -986,12 +985,12 @@ slioc_fail_recov(curstat_t *s, word32_t blk)
 	ret = TRUE;
 
 	recov_start_addr = blk + ERR_SKIPBLKS;
-	blktomsf(
+	util_blktomsf(
 		recov_start_addr,
 		&recov_start_msf.min,
 		&recov_start_msf.sec,
 		&recov_start_msf.frame,
-		MSF_OFFSET(s)
+		MSF_OFFSET
 	);
 
 	/* Check to see if we have skipped past
@@ -1014,7 +1013,7 @@ slioc_fail_recov(curstat_t *s, word32_t blk)
 
 	if (ret) {
 		/* Restart playback */
-		(void) fprintf(errfp, "CD audio: %s (%02u:%02u.%02u%d)\n",
+		(void) fprintf(errfp, "CD audio: %s (%02u:%02u.%02u)\n",
 				app_data.str_recoverr,
 				recov_start_msf.min,
 				recov_start_msf.sec,
@@ -1089,12 +1088,36 @@ slioc_get_playstatus(curstat_t *s)
 	}
 
 	/* Check the subchannel data */
-	audio_status = sub.cdsc_audiostatus;
+
+	s->cur_tot_frame = sub.cdsc_absaddr.msf.frame;
+	s->cur_tot_sec = sub.cdsc_absaddr.msf.second;
+	s->cur_tot_min = sub.cdsc_absaddr.msf.minute;
+	util_msftoblk(
+		s->cur_tot_min,
+		s->cur_tot_sec,
+		s->cur_tot_frame,
+		&s->cur_tot_addr,
+		MSF_OFFSET
+	);
+
+	s->cur_trk_frame = sub.cdsc_reladdr.msf.frame;
+	s->cur_trk_sec = sub.cdsc_reladdr.msf.second;
+	s->cur_trk_min = sub.cdsc_reladdr.msf.minute;
+	util_msftoblk(
+		s->cur_trk_min,
+		s->cur_trk_sec,
+		s->cur_trk_frame,
+		&s->cur_trk_addr,
+		0
+	);
+
+	/* Update time display */
+	DPY_TIME(s, FALSE);
 
 	if (slioc_bcd_hack) {
 		/* Hack: BUGLY CD-ROM firmware */
-		curtrk = bcdtol(sub.cdsc_trk);
-		curidx = bcdtol(sub.cdsc_ind);
+		curtrk = util_bcdtol(sub.cdsc_trk);
+		curidx = util_bcdtol(sub.cdsc_ind);
 	}
 	else {
 		curtrk = sub.cdsc_trk;
@@ -1117,30 +1140,7 @@ slioc_get_playstatus(curstat_t *s)
 		DPY_INDEX(s);
 	}
 
-	s->cur_tot_frame = sub.cdsc_absaddr.msf.frame;
-	s->cur_tot_sec = sub.cdsc_absaddr.msf.second;
-	s->cur_tot_min = sub.cdsc_absaddr.msf.minute;
-	msftoblk(
-		s->cur_tot_min,
-		s->cur_tot_sec,
-		s->cur_tot_frame,
-		&s->cur_tot_addr,
-		MSF_OFFSET(s)
-	);
-
-	s->cur_trk_frame = sub.cdsc_reladdr.msf.frame;
-	s->cur_trk_sec = sub.cdsc_reladdr.msf.second;
-	s->cur_trk_min = sub.cdsc_reladdr.msf.minute;
-	msftoblk(
-		s->cur_trk_min,
-		s->cur_trk_sec,
-		s->cur_trk_frame,
-		&s->cur_trk_addr,
-		0
-	);
-
-	/* Update time display */
-	DPY_TIME(s, FALSE);
+	audio_status = sub.cdsc_audiostatus;
 
 	/* Hack: to work around the fact that some CD-ROM drives
 	 * return CDROM_AUDIO_PAUSED status after issuing a Stop Unit command.
@@ -1199,7 +1199,7 @@ slioc_get_playstatus(curstat_t *s)
 
 		errblk = s->cur_tot_addr;
 
-		if (!done && slioc_fail_recov(s, errblk)) {
+		if (!done && slioc_fail_recov(errblk)) {
 			in_slioc_get_playstatus = FALSE;
 			return TRUE;
 		}
@@ -1503,12 +1503,12 @@ slioc_get_toc(curstat_t *s)
 		s->trkinfo[i].min = p->cdte_addr.msf.minute;
 		s->trkinfo[i].sec = p->cdte_addr.msf.second;
 		s->trkinfo[i].frame = p->cdte_addr.msf.frame;
-		msftoblk(
+		util_msftoblk(
 			s->trkinfo[i].min,
 			s->trkinfo[i].sec,
 			s->trkinfo[i].frame,
 			&s->trkinfo[i].addr,
-			MSF_OFFSET(s)
+			MSF_OFFSET
 		);
 
 		if (p->cdte_track == CDROM_LEADOUT || i == (MAXTRACK - 1)) {
@@ -1951,38 +1951,44 @@ slioc_run_rew(curstat_t *s)
 	if (slioc_start_search) {
 		slioc_start_search = FALSE;
 		seq = 0;
-		if (skip_blks < addr)
-			start_addr = addr - skip_blks;
-		else
-			start_addr = 0;
+		i = (int) (addr - skip_blks);
 	}
 	else {
 		if (app_data.skip_spdup > 0 && seq > app_data.skip_spdup)
 			/* Speed up search */
 			skip_blks *= 3;
 
-		if ((int) (start_addr - skip_blks) > 0)
-			start_addr -= skip_blks;
-		else
-			start_addr = 0;
+		i = (int) (start_addr - skip_blks);
 	}
 
+	start_addr = (word32_t) (i > CLIP_FRAMES) ? i : CLIP_FRAMES;
+
 	seq++;
 
 	if (s->shuffle || s->program) {
 		if ((i = di_curtrk_pos(s)) < 0)
 			i = 0;
-	}
-	else
-		i = 0;
 
-	if (start_addr < s->trkinfo[i].addr)
-		start_addr = s->trkinfo[i].addr;
+		if (start_addr < s->trkinfo[i].addr)
+			start_addr = s->trkinfo[i].addr;
+	}
 
 	end_addr = start_addr + MAX_SRCH_BLKS;
 
-	blktomsf(start_addr, &smsf.min, &smsf.sec, &smsf.frame, MSF_OFFSET(s));
-	blktomsf(end_addr, &emsf.min, &emsf.sec, &emsf.frame, MSF_OFFSET(s));
+	util_blktomsf(
+		start_addr,
+		&smsf.min,
+		&smsf.sec,
+		&smsf.frame,
+		MSF_OFFSET
+	);
+	util_blktomsf(
+		end_addr,
+		&emsf.min,
+		&emsf.sec,
+		&emsf.frame,
+		MSF_OFFSET
+	);
 
 	/* Play next search interval */
 	(void) slioc_do_playaudio(
@@ -2086,8 +2092,20 @@ slioc_run_ff(curstat_t *s)
 		start_addr = end_addr - skip_blks;
 	}
 
-	blktomsf(start_addr, &smsf.min, &smsf.sec, &smsf.frame, MSF_OFFSET(s));
-	blktomsf(end_addr, &emsf.min, &emsf.sec, &emsf.frame, MSF_OFFSET(s));
+	util_blktomsf(
+		start_addr,
+		&smsf.min,
+		&smsf.sec,
+		&smsf.frame,
+		MSF_OFFSET
+	);
+	util_blktomsf(
+		end_addr,
+		&emsf.min,
+		&emsf.sec,
+		&emsf.frame,
+		MSF_OFFSET
+	);
 
 	/* Play next search interval */
 	(void) slioc_do_playaudio(
@@ -2175,19 +2193,19 @@ slioc_run_sample(curstat_t *s)
 		saddr = s->trkinfo[slioc_next_sam].addr;
 		eaddr = saddr + app_data.sample_blks,
 
-		blktomsf(
+		util_blktomsf(
 			saddr,
 			&smsf.min,
 			&smsf.sec,
 			&smsf.frame,
-			MSF_OFFSET(s)
+			MSF_OFFSET
 		);
-		blktomsf(
+		util_blktomsf(
 			eaddr,
 			&emsf.min,
 			&emsf.sec,
 			&emsf.frame,
-			MSF_OFFSET(s)
+			MSF_OFFSET
 		);
 
 		if (slioc_do_playaudio(ADDR_BLK | ADDR_MSF,
@@ -2267,12 +2285,12 @@ slioc_run_prog(curstat_t *s)
 	s->cur_idx = 1;
 
 	start_addr = s->trkinfo[i].addr + s->cur_trk_addr;
-	blktomsf(
+	util_blktomsf(
 		start_addr,
 		&s->cur_tot_min,
 		&s->cur_tot_sec,
 		&s->cur_tot_frame,
-		MSF_OFFSET(s)
+		MSF_OFFSET
 	);
 	start_msf.min = s->cur_tot_min;
 	start_msf.sec = s->cur_tot_sec;
@@ -2564,9 +2582,13 @@ void
 slioc_repeat(curstat_t *s, bool_t enable)
 {
 	s->repeat = enable;
-	DPY_RPTCNT(s);
-	if (!enable)
+
+	if (!enable && slioc_new_progshuf) {
 		slioc_new_progshuf = FALSE;
+		if (s->rptcnt > 0)
+			s->rptcnt--;
+	}
+	DPY_RPTCNT(s);
 }
 
 
@@ -2733,12 +2755,12 @@ slioc_ab(curstat_t *s)
 		    app_data.min_playblks) {
 			slioc_ab_end_addr = slioc_ab_start_addr +
 					    app_data.min_playblks;
-			blktomsf(
+			util_blktomsf(
 				slioc_ab_end_addr,
 				&slioc_ab_end_msf.min,
 				&slioc_ab_end_msf.sec,
 				&slioc_ab_end_msf.frame,
-				MSF_OFFSET(s)
+				MSF_OFFSET
 			);
 		}
 
@@ -2930,11 +2952,12 @@ slioc_play_pause(curstat_t *s)
 				s->cur_trk = s->first_trk;
 				start_addr = s->trkinfo[0].addr +
 					     s->cur_trk_addr;
-				blktomsf(start_addr,
-					 &start_msf.min,
-					 &start_msf.sec,
-					 &start_msf.frame,
-					 MSF_OFFSET(s)
+				util_blktomsf(
+					start_addr,
+					&start_msf.min,
+					&start_msf.sec,
+					&start_msf.frame,
+					MSF_OFFSET
 				);
 			}
 			else {
@@ -2943,11 +2966,12 @@ slioc_play_pause(curstat_t *s)
 					     s->cur_trk_addr;
 			}
 
-			blktomsf(start_addr,
-				 &start_msf.min,
-				 &start_msf.sec,
-				 &start_msf.frame,
-				 MSF_OFFSET(s)
+			util_blktomsf(
+				start_addr,
+				&start_msf.min,
+				&start_msf.sec,
+				&start_msf.frame,
+				MSF_OFFSET
 			);
 
 			end_msf.min = s->tot_min;
@@ -3133,7 +3157,7 @@ slioc_chgdisc(curstat_t *s)
 	sav_rptcnt = s->rptcnt;
 
 	/* Stop the CD first */
-	(void) slioc_stop(s, TRUE);
+	slioc_stop(s, TRUE);
 
 	di_reset_curstat(s, TRUE, FALSE);
 	s->mode = MOD_NODISC;
@@ -3196,24 +3220,34 @@ slioc_prevtrk(curstat_t *s)
 		else
 			i = di_curtrk_pos(s);
 
-		if (i < 0)
-			return;
-
-		start_addr = s->trkinfo[i].addr;
-		start_msf.min = s->trkinfo[i].min;
-		start_msf.sec = s->trkinfo[i].sec;
-		start_msf.frame = s->trkinfo[i].frame;
-		s->cur_trk = s->trkinfo[i].trkno;
-		s->cur_idx = 1;
-
-		/* If the current track has been playing for less
-		 * than app_data.prev_threshold blocks, then go
-		 * to the beginning of the previous track (if we
-		 * are not already on the first track).
-		 */
 		go_prev = FALSE;
-		if ((s->cur_tot_addr - start_addr) <= app_data.prev_threshold)
-			go_prev = TRUE;
+
+		if (i == 0 && s->cur_idx == 0) {
+			i = 0;
+			start_addr = CLIP_FRAMES;
+			start_msf.min = 0;
+			start_msf.sec = 2;
+			start_msf.frame = CLIP_FRAMES;
+			s->cur_trk = s->trkinfo[i].trkno;
+			s->cur_idx = 0;
+		}
+		else {
+			start_addr = s->trkinfo[i].addr;
+			start_msf.min = s->trkinfo[i].min;
+			start_msf.sec = s->trkinfo[i].sec;
+			start_msf.frame = s->trkinfo[i].frame;
+			s->cur_trk = s->trkinfo[i].trkno;
+			s->cur_idx = 1;
+
+			/* If the current track has been playing for less
+			 * than app_data.prev_threshold blocks, then go
+			 * to the beginning of the previous track (if we
+			 * are not already on the first track).
+			 */
+			if ((s->cur_tot_addr - start_addr) <=
+			    app_data.prev_threshold)
+				go_prev = TRUE;
+		}
 
 		if (go_prev) {
 			if (s->shuffle || s->program) {
@@ -3230,6 +3264,16 @@ slioc_prevtrk(curstat_t *s)
 				start_msf.frame = s->trkinfo[i].frame;
 				s->cur_trk = s->trkinfo[i].trkno;
 			}
+			else if (i == 0) {
+				/* Go to the very beginning: this may be
+				 * a lead-in area before the start of track 1.
+				 */
+				start_addr = CLIP_FRAMES;
+				start_msf.min = 0;
+				start_msf.sec = 2;
+				start_msf.frame = CLIP_FRAMES;
+				s->cur_trk = s->trkinfo[i].trkno;
+			}
 			else if (i > 0) {
 				i--;
 
@@ -3392,7 +3436,9 @@ slioc_nexttrk(curstat_t *s)
 		/* Find next track */
 		if ((i = di_curtrk_pos(s)) < 0)
 			return;
-		i++;
+
+		if (i > 0 || s->cur_idx > 0)
+			i++;
 
 		/* Skip over data tracks */
 		while (i < MAXTRACK && s->trkinfo[i].type == TYP_DATA)
