@@ -1,7 +1,7 @@
 /*
  *   xmcd - Motif(tm) CD Audio Player
  *
- *   Copyright (C) 1993-1997  Ti Kan
+ *   Copyright (C) 1993-1998  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -20,17 +20,16 @@
  *
  */
 #ifndef LINT
-static char *_cdfunc_c_ident_ = "@(#)cdfunc.c	6.94 97/08/08";
+static char *_cdfunc_c_ident_ = "@(#)cdfunc.c	6.132 98/05/18";
 #endif
 
-#define _XMINC_
-
 #include "common_d/appenv.h"
 #include "common_d/patchlevel.h"
 #include "common_d/util.h"
 #include "xmcd_d/xmcd.h"
 #include "xmcd_d/widget.h"
 #include "xmcd_d/dbprog.h"
+#include "xmcd_d/geom.h"
 #include "xmcd_d/hotkey.h"
 #include "xmcd_d/help.h"
 #include "xmcd_d/cdfunc.h"
@@ -61,14 +60,18 @@ STATIC int		keypad_mode;		/* Keypad mode */
 STATIC long		tm_blinkid = -1,	/* Time dpy blink timer ID */
 			ab_blinkid = -1,	/* A->B dpy blink timer ID */
 			dbmode_blinkid = -1,	/* Dbmode dpy blink timer ID */
-			chgdisc_dlyid = -1;	/* Disc chg delay timer ID */
+			chgdisc_dlyid = -1,	/* Disc chg delay timer ID */
+			tooltip1_id = -1,	/* Tooltip popup timer ID */
+			tooltip2_id = -1;	/* Tooltip popdown timer ID */
 STATIC word32_t		warp_offset = 0;	/* Track warp block offset */
 STATIC bool_t		searching,		/* Running REW or FF */
-			btnlbl_state,		/* Button label state */
 			popup_ok,		/* Are popups allowed? */
 			pseudo_warp,		/* Warp slider only flag */
 			warp_busy,		/* Warp function active */
-			chgdelay;		/* Disc change delay */
+			chgdelay,		/* Disc change delay */
+			mode_chg,		/* Changing main window mode */
+			tooltip_active,		/* Tooltip is active */
+			skip_next_tooltip;	/* Skip the next tooltip */
 STATIC cddb_client_t	cddb_cldata;		/* Client info for libcddb */
 STATIC di_client_t	di_cldata;		/* Client info for libdi */
 
@@ -84,6 +87,68 @@ STATIC void		cd_pause_blink(curstat_t *, bool_t),
  ***********************/
 
 
+/*
+ * disc_etime_norm
+ *	Return the elapsed time of the disc in seconds.  This is
+ *	used during normal playback.
+ *
+ * Args:
+ *	s - Pointer to the curstat_t structure.
+ *
+ * Return:
+ *	The disc elapsed time in seconds.
+ */
+STATIC sword32_t
+disc_etime_norm(curstat_t *s)
+{
+	sword32_t	secs;
+
+	secs = (s->cur_tot_min * 60 + s->cur_tot_sec) -
+	       (MSF_OFFSET / FRAME_PER_SEC);
+	return ((secs >= 0) ? secs : 0);
+}
+
+
+/*
+ * disc_etime_prog
+ *	Return the elapsed time of the disc in seconds.  This is
+ *	used during shuffle or program mode.
+ *
+ * Args:
+ *	s - Pointer to the curstat_t structure.
+ *
+ * Return:
+ *	The disc elapsed time in seconds.
+ */
+STATIC sword32_t
+disc_etime_prog(curstat_t *s)
+{
+	sword32_t	i,
+			secs = 0;
+
+	/* Find the time of all played tracks */
+	for (i = 0; i < ((int) s->prog_cnt) - 1; i++) {
+		secs += ((s->trkinfo[s->playorder[i]+1].min * 60 +
+			 s->trkinfo[s->playorder[i]+1].sec) -
+		         (s->trkinfo[s->playorder[i]].min * 60 +
+			 s->trkinfo[s->playorder[i]].sec));
+	}
+
+	/* Find the elapsed time of the current track */
+	for (i = 0; i < MAXTRACK; i++) {
+		if (s->trkinfo[i].trkno == LEAD_OUT_TRACK)
+			break;
+
+		if (s->trkinfo[i].trkno == s->cur_trk) {
+			secs += (s->cur_trk_min * 60 + s->cur_trk_sec);
+			break;
+		}
+	}
+
+	return ((secs >= 0) ? secs : 0);
+}
+
+
 /*
  * track_rtime
  *	Return the remaining time of the current playing track in seconds.
@@ -324,8 +389,16 @@ dpy_keypad_ind(curstat_t *s)
 	case KPMODE_TRACK:
 		if (keystr[0] == '\0') {
 			if (warp_busy) {
-				blktomsf(warp_offset, &min, &sec, &frame, 0);
+				util_blktomsf(
+					warp_offset,
+					&min,
+					&sec,
+					&frame,
+					0
+				);
+
 				(void) sprintf(trk, "%02u", s->cur_trk);
+
 				(void) sprintf(time, "+%02u:%02u", min, sec);
 			}
 			else if (di_curtrk_pos(s) < 0) {
@@ -337,7 +410,13 @@ dpy_keypad_ind(curstat_t *s)
 				(void) strcpy(time, " --:--");
 			}
 			else {
-				blktomsf(s->cur_trk_addr, &min, &sec, &frame, 0);
+				util_blktomsf(
+					s->cur_trk_addr,
+					&min,
+					&sec,
+					&frame,
+					0
+				);
 				(void) sprintf(trk, "%02u", s->cur_trk);
 				(void) sprintf(time, "%c%02u:%02u",
 				       (s->cur_idx == 0) ? '-' : '+', min, sec);
@@ -349,7 +428,7 @@ dpy_keypad_ind(curstat_t *s)
 				(void) sprintf(str, "%s  %s", trk, time);
 		}
 		else {
-			blktomsf(warp_offset, &min, &sec, &frame, 0);
+			util_blktomsf(warp_offset, &min, &sec, &frame, 0);
 			(void) sprintf(trk, "%02u", atoi(keystr));
 			(void) sprintf(time, "+%02u:%02u", min, sec);
 
@@ -438,13 +517,15 @@ set_btn_color(Widget w, Pixmap px, Pixel color)
  *
  * Args:
  *	w - The scale widget.
+ *	pixmap - not used.
  *	color - The pixel value of the desired color.
  *
  * Return:
  *	Nothing.
  */
+/*ARGSUSED*/
 STATIC void
-set_scale_color(Widget w, Pixel color)
+set_scale_color(Widget w, Pixmap px, Pixel color)
 {
 	XtVaSetValues(w, XmNforeground, color, NULL);
 }
@@ -577,6 +658,302 @@ do_chgdisc(curstat_t *s)
 }
 
 
+/*
+ * cd_tooltip_popdown
+ *	Pop-down the tool-tip.
+ *
+ * Args:
+ *	w - The tooltip shell.
+ *
+ * Return:
+ *	Nothing.
+ */
+STATIC void
+cd_tooltip_popdown(Widget w)
+{
+	/* Cancel pending timers */
+	if (tooltip1_id >= 0) {
+		cd_untimeout(tooltip1_id);
+		tooltip1_id = -1;
+	}
+	if (tooltip2_id >= 0) {
+		cd_untimeout(tooltip2_id);
+		tooltip2_id = -1;
+	}
+
+	/* Pop down the tooltip */
+	if (tooltip_active) {
+		tooltip_active = FALSE;
+		XtPopdown(w);
+	}
+}
+
+
+/*
+ * cd_tooltip_sphandler
+ *	Special widget-specific handler for tool-tips
+ *
+ * Args:
+ *	w - The associated control widget
+ *	ret_tlbl - the return XmString label that will be displayed on the
+ *		   tool-tip.  The caller should XmStringFree() this when
+ *		   done.
+ *
+ * Return:
+ *	0: Don't pop-up tool-tip
+ *	1: Pop-up tooltip with the returned ret_tlbl
+ *	2: This is not a special widget: use the default handler.
+ */
+STATIC int
+cd_tooltip_sphandler(Widget w, XmString *ret_tlbl)
+{
+	char		*ttitle,
+			*dtitle;
+	XmString	xs1,
+			xs2,
+			xs3;
+	curstat_t	*s = curstat_addr();
+
+	if (w == widgets.main.disc_ind) {
+		/* Disc display */
+		xs1 = XmStringCreateSimple("Disc ");
+		XtVaGetValues(w, XmNlabelString, &xs2, NULL);
+		*ret_tlbl = XmStringConcat(xs1, xs2);
+		XmStringFree(xs1);
+		XmStringFree(xs2);
+	}
+	else if (w == widgets.main.track_ind) {
+		/* Track display */
+		xs1 = XmStringCreateSimple("Track ");
+		XtVaGetValues(w, XmNlabelString, &xs2, NULL);
+		*ret_tlbl = XmStringConcat(xs1, xs2);
+		XmStringFree(xs1);
+		XmStringFree(xs2);
+	}
+	else if (w == widgets.main.index_ind) {
+		/* Index display */
+		xs1 = XmStringCreateSimple("Index ");
+		XtVaGetValues(w, XmNlabelString, &xs2, NULL);
+		*ret_tlbl = XmStringConcat(xs1, xs2);
+		XmStringFree(xs1);
+		XmStringFree(xs2);
+	}
+	else if (w == widgets.main.time_ind) {
+		/* Time display */
+		xs1 = XmStringCreateSimple("Time ");
+		XtVaGetValues(widgets.main.timemode_ind,
+			XmNlabelString, &xs2,
+			NULL
+		);
+		xs3 = XmStringConcat(xs1, xs2);
+		XmStringFree(xs1);
+		XmStringFree(xs2);
+
+		switch (geom_main_getmode()) {
+		case MAIN_NORMAL:
+			/* Normal mode */
+			*ret_tlbl = xs3;
+			break;
+
+		case MAIN_BASIC:
+			/* Basic mode */
+
+			/* Add disc title */
+			dtitle = dbprog_curdtitle(s);
+			if (dtitle[0] == '\0') {
+				/* No disc title */
+				*ret_tlbl = xs3;
+				break;
+			}
+
+			xs1 = XmStringSeparatorCreate();
+			xs2 = XmStringConcat(xs3, xs1);
+			XmStringFree(xs1);
+			XmStringFree(xs3);
+
+			xs1 = XmStringCreateLtoR(
+				dtitle,
+				XmSTRING_DEFAULT_CHARSET
+			);
+			xs3 = XmStringConcat(xs2, xs1);
+			XmStringFree(xs1);
+			XmStringFree(xs2);
+
+			/* Add track title */
+			ttitle = dbprog_curttitle(s);
+			if (ttitle[0] == '\0') {
+				/* No track title */
+				*ret_tlbl = xs3;
+			}
+			else {
+				xs1 = XmStringSeparatorCreate();
+				xs2 = XmStringConcat(xs3, xs1);
+				XmStringFree(xs1);
+				XmStringFree(xs3);
+
+				xs1 = XmStringCreateLtoR(
+					ttitle,
+					XmSTRING_DEFAULT_CHARSET
+				);
+				xs3 = XmStringConcat(xs2, xs1);
+				XmStringFree(xs1);
+				XmStringFree(xs2);
+
+				*ret_tlbl = xs3;
+			}
+			break;
+		}
+	}
+	else if (w == widgets.main.dtitle_ind) {
+		dtitle = dbprog_curdtitle(s);
+		if (dtitle[0] == '\0') {
+			/* No disc title: don't popup tooltip */
+			return 0;
+		}
+		/* Use label string */
+		XtVaGetValues(w, XmNlabelString, ret_tlbl, NULL);
+	}
+	else if (w == widgets.main.ttitle_ind) {
+		ttitle = dbprog_curttitle(s);
+		if (ttitle[0] == '\0') {
+			/* No track title: don't popup tooltip */
+			return 0;
+		}
+		/* Use label string */
+		XtVaGetValues(w, XmNlabelString, ret_tlbl, NULL);
+	}
+	else
+		return 2;
+
+	return 1;
+}
+
+
+/*
+ * cd_tooltip_popup
+ *	Timer function to pop-up the tool-tip.
+ *
+ * Args:
+ *	w - The associated control widget
+ *
+ * Return:
+ *	Nothing.
+ */
+STATIC void
+cd_tooltip_popup(Widget w)
+{
+	Display			*display = XtDisplay(widgets.toplevel);
+	int			screen = DefaultScreen(display);
+	Position		end_x,
+				end_y,
+				abs_x,
+				abs_y,
+				x,
+				y;
+	Dimension		width,
+				height,
+				off_x,
+				off_y,
+				swidth,
+				sheight;
+	WidgetClass		wc;
+	XtWidgetGeometry	geom;
+	XmString		tlbl;
+
+	tooltip1_id = -1;
+	tooltip_active = TRUE;
+	swidth = (Dimension) XDisplayWidth(display, screen);
+	sheight = (Dimension) XDisplayHeight(display, screen);
+
+	XtVaGetValues(w,
+		XmNwidth, &width,
+		XmNheight, &height,
+		NULL
+	);
+
+	/* Perform widget-specific handling */
+	switch (cd_tooltip_sphandler(w, &tlbl)) {
+	case 0:
+		/* tooltip popup refused by cd_tooltip_sphandler */
+		tooltip_active = FALSE;
+		return;
+	case 1:
+		/* cd_tooltip_sphandler handled it */
+		break;
+	default:
+		wc = XtClass(w);
+
+		if (wc == xmPushButtonWidgetClass ||
+			 wc == xmToggleButtonWidgetClass) {
+			/* Use label string */
+			XtVaGetValues(w, XmNlabelString, &tlbl, NULL);
+		}
+		else if (wc == xmScaleWidgetClass) {
+			/* Use title string */
+			XtVaGetValues(w, XmNtitleString, &tlbl, NULL);
+		}
+		else {
+			/* Use widget name */
+			tlbl = XmStringCreateSimple(XtName(w));
+		}
+		break;
+	}
+
+	/* Set the tooltip label string and hotkey mnemonic, if any. */
+	XtVaSetValues(widgets.tooltip.tooltip_lbl, XmNlabelString, tlbl, NULL);
+	hotkey_tooltip_mnemonic(w);
+
+	/* Translate to screen absolute coordinates, and add desired offsets */
+	XtTranslateCoords(w, 0, 0, &abs_x, &abs_y);
+
+	/* Make sure that the tooltip window doesn't go beyond
+	 * screen boundaries.
+	 */
+	(void) XtQueryGeometry(widgets.tooltip.tooltip_lbl, NULL, &geom);
+
+	off_x = width / 2;
+	end_x = abs_x + geom.width + off_x + 2;
+	if (end_x > (Position) swidth)
+		off_x = -(end_x - swidth - off_x);
+	x = abs_x + off_x;
+	if (x < 0)
+		x = 2;
+
+	off_y = height + 5;
+	end_y = abs_y + geom.height + off_y + 2;
+	if (end_y > (Position) sheight)
+		off_y = -(geom.height + 5);
+	y = abs_y + off_y;
+
+	/* Move tooltip widget to desired location and pop it up */
+	XtVaSetValues(widgets.tooltip.shell, XmNx, x, XmNy, y, NULL);
+	XtPopup(widgets.tooltip.shell, XtGrabNone);
+
+	/* Set timer for auto-popdown */
+	if (app_data.tooltip_time > 0) {
+		int	n;
+
+		/* If the tooltip label is longer than 40 characters,
+		 * add 50mS to the tooltip time for each additional
+		 * character.
+		 */
+		n = XmStringLength(tlbl);
+		if (n <= 40)
+			n = 0;
+		else
+			n = ((n - 40) + 1) * 50;
+
+		tooltip2_id = cd_timeout(
+			app_data.tooltip_time + n,
+			cd_tooltip_popdown,
+			(byte_t *) widgets.tooltip.shell
+		);
+	}
+
+	XmStringFree(tlbl);
+}
+
+
 /***********************
  *   public routines   *
  ***********************/
@@ -677,7 +1054,8 @@ dpy_track(curstat_t *s)
 
 	sav_trk = s->cur_trk;
 
-	if (s->cur_trk <= 0 || s->mode == MOD_BUSY || s->mode == MOD_NODISC)
+	if (s->cur_trk < 0 || s->mode == MOD_BUSY ||
+		 s->mode == MOD_NODISC)
 		(void) strcpy(str, "--");
 	else if (s->time_dpy == T_REMAIN_DISC) {
 		if (s->shuffle || s->program) {
@@ -694,11 +1072,14 @@ dpy_track(curstat_t *s)
 	else
 		(void) sprintf(str, "%02u", s->cur_trk);
 
-	if (strcmp(str, prev) == 0)
+	if (strcmp(str, prev) == 0 && !mode_chg)
 		/* No change, just return */
 		return;
 
-	xs = XmStringCreateSimple(str);
+	xs = XmStringCreateLtoR(
+		str,
+		(geom_main_getmode() == MAIN_NORMAL) ? CHSET1 : CHSET2
+	);
 
 	XtVaSetValues(
 		widgets.main.track_ind,
@@ -771,9 +1152,12 @@ dpy_time(curstat_t *s, bool_t blank)
 {
 	sword32_t	time_sec;
 	XmString	xs;
+	bool_t		playing;
 	char		str[12];
 	static char	prev[12] = { 'j', 'u', 'n', 'k', '\0' };
 
+	playing = FALSE;
+
 	if (blank)
 		str[0] = '\0';
 	else if (s->mode == MOD_BUSY)
@@ -785,6 +1169,8 @@ dpy_time(curstat_t *s, bool_t blank)
 	else if (s->mode == MOD_STOP)
 		(void) strcpy(str, " --:--");
 	else {
+		playing = TRUE;
+
 		switch (s->time_dpy) {
 		case T_ELAPSED:
 			(void) sprintf(str, "%s%02u:%02u",
@@ -793,17 +1179,36 @@ dpy_time(curstat_t *s, bool_t blank)
 				       s->cur_trk_sec);
 			break;
 
-		case T_REMAIN_TRACK:
-			time_sec = track_rtime(s);
+		case T_ELAPSED_DISC:
+			if (s->shuffle || s->program) {
+				if (s->cur_idx == 0) {
+					(void) strcpy(str, "   :  ");
+					break;
+				}
+				else
+					time_sec = disc_etime_prog(s);
+			}
+			else
+				time_sec = disc_etime_norm(s);
 
-			(void) sprintf(str, "-%02u:%02u",
+			(void) sprintf(str, "+%02u:%02u",
 				       time_sec / 60, time_sec % 60);
 			break;
 
+		case T_REMAIN_TRACK:
+			if (s->cur_idx == 0)
+				(void) strcpy(str, "   :  ");
+			else {
+				time_sec = track_rtime(s);
+				(void) sprintf(str, "-%02u:%02u",
+					       time_sec / 60, time_sec % 60);
+			}
+			break;
+
 		case T_REMAIN_DISC:
 			if (s->shuffle || s->program) {
 				if (s->cur_idx == 0) {
-					(void) strcpy(str, " --:--");
+					(void) strcpy(str, "   :  ");
 					break;
 				}
 				else
@@ -827,11 +1232,19 @@ dpy_time(curstat_t *s, bool_t blank)
 	else
 		cd_pause_blink(s, FALSE);
 
-	if (strcmp(str, prev) == 0)
+	if (strcmp(str, prev) == 0 && !mode_chg) {
+		if (s->cur_idx == 0 && playing)
+			/* Update the keypad indicator */
+			dpy_keypad_ind(s);
+
 		/* No change: just return */
 		return;
+	}
 
-	xs = XmStringCreateSimple(str);
+	xs = XmStringCreateLtoR(
+		str,
+		(geom_main_getmode() == MAIN_NORMAL) ? CHSET1 : CHSET2
+	);
 
 	XtVaSetValues(
 		widgets.main.time_ind,
@@ -954,7 +1367,7 @@ dpy_rptcnt(curstat_t *s)
 	char			str[12];
 	static char		prevstr[12];
 
-	if (s->repeat && s->mode == MOD_PLAY)
+	if (s->repeat && (s->mode == MOD_PLAY || s->mode == MOD_PAUSE))
 		(void) sprintf(str, "%u", s->rptcnt);
 	else
 		(void) strcpy(str, "-");
@@ -1109,6 +1522,10 @@ dpy_timemode(curstat_t *s)
 		str = app_data.str_elapse;
 		break;
 
+	case T_ELAPSED_DISC:
+		str = app_data.str_elapsedisc;
+		break;
+
 	case T_REMAIN_TRACK:
 		str = app_data.str_remaintrk;
 		break;
@@ -1367,116 +1784,6 @@ set_bal_slider(int val)
 }
 
 
-/*
- * set_btn_lbltype
- *	Set the main window pushbuttons label type
- *
- * Args:
- *	type - BTN_PIXMAP or BTN_STRING.
- *
- * Return:
- *	Nothing.
- */
-void
-set_btn_lbltype(byte_t type)
-{
-	type = (type == BTN_STRING) ? XmSTRING : XmPIXMAP;
-
-	XtVaSetValues(widgets.main.btnlbl_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.lock_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.repeat_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.shuffle_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.eject_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.poweroff_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.dbprog_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.help_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.options_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.time_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.ab_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.sample_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.keypad_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.playpause_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.stop_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.prevdisc_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.nextdisc_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.prevtrk_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.nexttrk_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.previdx_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.nextidx_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.rew_btn,
-		XmNlabelType, type,
-		NULL
-	);
-	XtVaSetValues(widgets.main.ff_btn,
-		XmNlabelType, type,
-		NULL
-	);
-}
-
-
 /*
  * scale_warp
  *	Scale track warp value (0-255) to the number of CD logical audio
@@ -1625,6 +1932,31 @@ cd_info_popup(char *title, char *msg)
 }
 
 
+/*
+ * cd_info_popup_auto
+ *	Pop up the information message dialog box, which will auto-popdown
+ *	in 5 seconds.
+ *
+ * Args:
+ *	title - The title bar text string.
+ *	msg - The information message text string.
+ *
+ * Return:
+ *	Nothing.
+ */
+void
+cd_info_popup_auto(char *title, char *msg)
+{
+	cd_info_popup(title, msg);
+
+	(void) cd_timeout(
+		5000,	/* popup interval */
+		XtUnmanageChild,
+		(byte_t *) widgets.dialog.info
+	);
+}
+
+
 /*
  * cd_warning_popup
  *	Pop up the warning message dialog box.
@@ -1870,10 +2202,16 @@ cd_init(curstat_t *s)
 	struct stat	stbuf;
 
 
-
 	DBGPRN(errfp, "XMCD v%s%s PL%d DEBUG MODE\n\n",
 	       VERSION, VERSION_EXT, PATCHLEVEL);
 
+	/* app-defaults file check */
+	if (app_data.version == NULL || 
+	    strncmp(VERSION, app_data.version, strlen(VERSION)) != 0) {
+		CD_FATAL(app_data.str_appdef);
+		return;
+	}
+
 #ifndef NOMKTMPDIR
 	errmsg = (char *) MEM_ALLOC(
 		strlen(app_data.str_tmpdirerr) + strlen(TEMP_DIR)
@@ -1886,19 +2224,7 @@ cd_init(curstat_t *s)
 	/* Make temporary directory, if needed */
 	(void) sprintf(errmsg, app_data.str_tmpdirerr, TEMP_DIR);
 	if (LSTAT(TEMP_DIR, &stbuf) < 0) {
-		if (errno == ENOENT) {
-			int	omask;
-
-			/* The permissions should be writable by all */
-			omask = umask(0);
-			if (mkdir(TEMP_DIR, 0777) < 0) {
-				(void) umask((mode_t) omask);
-				CD_FATAL(errmsg);
-				return;
-			}
-			(void) umask((mode_t) omask);
-		}
-		else {
+		if (!util_mkdir(TEMP_DIR, 0777)) {
 			CD_FATAL(errmsg);
 			return;
 		}
@@ -1944,12 +2270,13 @@ cd_init(curstat_t *s)
 	}
 	if (app_data.device != NULL) {
 		if ((int) strlen(app_data.device) >= FILE_PATH_SZ ||
-		    (int) strlen(basename(app_data.device)) >= FILE_BASE_SZ) {
+		    (int) strlen(util_basename(app_data.device)) >=
+		    FILE_BASE_SZ) {
 			CD_FATAL(app_data.str_longpatherr);
 			return;
 		}
 	}
-	hd = homedir(get_ouid());
+	hd = util_homedir(util_get_ouid());
 	if ((int) strlen(hd) >= FILE_PATH_SZ) {
 		CD_FATAL(app_data.str_longpatherr);
 		return;
@@ -1965,7 +2292,7 @@ cd_init(curstat_t *s)
 
 	/* Initialize CD database services */
 	(void) strcpy(cddb_cldata.prog, PROGNAME);
-	(void) strcpy(cddb_cldata.user, loginname());
+	(void) strcpy(cddb_cldata.user, util_loginname());
 	cddb_cldata.isdemo = di_isdemo;
 	cddb_cldata.curstat_addr = curstat_addr;
 	cddb_cldata.fatal_msg = cd_fatal_popup;
@@ -1988,7 +2315,7 @@ cd_init(curstat_t *s)
 		XmStringFree(xs);
 	}
 
-	bdevname = basename(app_data.device);
+	bdevname = util_basename(app_data.device);
 
 	/* Get system-wide device-specific configuration parameters */
 	(void) sprintf(str, SYS_DSCFG_PATH, app_data.libdir, bdevname);
@@ -2032,25 +2359,26 @@ cd_init(curstat_t *s)
 	di_shuffle(s, app_data.shuffle_mode);
 	set_shuffle_btn(s->shuffle);
 	keypad_mode = KPMODE_TRACK;
+	s->time_dpy = (byte_t) app_data.timedpy_mode;
 
 	/* Set default options */
 	cd_options_reset(widgets.options.reset_btn, (XtPointer) s, NULL);
 
-	/* Set the main window title */
+	/* Set the main window and icon titles */
 	(void) strncpy(titlestr, app_data.main_title, STR_BUF_SZ - 12);
 	titlestr[STR_BUF_SZ - 12] = '\0';
 	(void) sprintf(titlestr, "%s %d", titlestr, app_data.devnum);
-	XStoreName(
-		XtDisplay(widgets.toplevel),
-		XtWindow(widgets.toplevel),
-		titlestr
+	XtVaSetValues(widgets.toplevel,
+		XmNtitle, titlestr,
+		XmNiconName, PROGNAME,
+		NULL
 	);
 }
 
 
 /*
  * cd_start
- *	Start up I/O to the CD player.
+ *	Secondary startup functions
  *
  * Args:
  *	s - Pointer to the curstat_t structure.
@@ -2064,8 +2392,14 @@ cd_start(curstat_t *s)
 	/* Allow popup dialogs from here on */
 	popup_ok = TRUE;
 
+	/* Start up libutil */
+	util_start();
+
 	/* Start up I/O interface */
 	di_start(s);
+
+	/* Start up help */
+	help_start();
 }
 
 
@@ -2132,7 +2466,7 @@ cd_quit(curstat_t *s)
 
 	/* Uninstall current keyboard grabs */
 	p.reason = XmCR_FOCUS;
-	cd_form_focus_chg(
+	cd_shell_focus_chg(
 		widgets.toplevel,
 		(XtPointer) widgets.toplevel,
 		(XtPointer) &p
@@ -2184,6 +2518,9 @@ cd_busycurs(bool_t busy, int winmap)
 		if ((winmap & CURS_DBPROG) &&
 		    (win = XtWindow(widgets.dbprog.form)) != (Window) 0)
 			XDefineCursor(dpy, win, wcur);
+		if ((winmap & CURS_WWWWARP) &&
+		    (win = XtWindow(widgets.wwwwarp.form)) != (Window) 0)
+			XDefineCursor(dpy, win, wcur);
 		if ((winmap & CURS_DBEXTD) &&
 		    (win = XtWindow(widgets.dbextd.form)) != (Window) 0)
 			XDefineCursor(dpy, win, wcur);
@@ -2213,6 +2550,9 @@ cd_busycurs(bool_t busy, int winmap)
 		if ((winmap & CURS_DBPROG) &&
 		    (win = XtWindow(widgets.dbprog.form)) != (Window) 0)
 			XUndefineCursor(dpy, win);
+		if ((winmap & CURS_WWWWARP) &&
+		    (win = XtWindow(widgets.wwwwarp.form)) != (Window) 0)
+			XUndefineCursor(dpy, win);
 		if ((winmap & CURS_DBEXTD) &&
 		    (win = XtWindow(widgets.dbextd.form)) != (Window) 0)
 			XUndefineCursor(dpy, win);
@@ -2261,31 +2601,17 @@ onsig(int sig)
 void
 cd_checkbox(Widget w, XtPointer client_data, XtPointer call_data)
 {
-	XmRowColumnCallbackStruct
-			*p = (XmRowColumnCallbackStruct *)(void *) call_data;
-	curstat_t	*s = (curstat_t *)(void *) client_data;
+	XmRowColumnCallbackStruct	*p =
+		(XmRowColumnCallbackStruct *)(void *) call_data;
+	curstat_t			*s =
+		(curstat_t *)(void *) client_data;
 
 	if (p->reason != XmCR_ACTIVATE)
 		return;
 
 	DBGPRN(errfp, "\n* CHKBOX: ");
 
-	if (p->widget == widgets.main.btnlbl_btn) {
-		DBGPRN(errfp, "disp\n");
-
-		btnlbl_state = !btnlbl_state;
-		set_btn_lbltype((byte_t)
-			(btnlbl_state ? BTN_STRING : BTN_PIXMAP)
-		);
-
-		/* Overload the function of this button to also
-		 * dump the contents of the curstat_t structure
-		 * in debug mode
-		 */
-		if (app_data.debug)
-			di_dump_curstat(s);
-	}
-	else if (p->widget == widgets.main.lock_btn) {
+	if (p->widget == widgets.main.lock_btn) {
 		DBGPRN(errfp, "lock\n");
 
 		di_lock(s, (bool_t) !s->caddy_lock);
@@ -2303,6 +2629,46 @@ cd_checkbox(Widget w, XtPointer client_data, XtPointer call_data)
 }
 
 
+/*
+ * cd_mode
+ *	Main window mode button callback function
+ */
+/*ARGSUSED*/
+void
+cd_mode(Widget w, XtPointer client_data, XtPointer call_data)
+{
+	curstat_t	*s = (curstat_t *)(void *) client_data;
+
+	DBGPRN(errfp, "\n* MODE\n");
+
+	/* Change the main window arrangement.  Unmap the form while
+	 * the change takes place so that the user does not see a
+	 * bunch of jumbled widgets while they adjust to the new size.
+	 */
+	XtUnmapWidget(widgets.main.form);
+	geom_main_chgmode(&widgets);
+	XtMapWidget(widgets.main.form);
+
+	/* Force indicator font change */
+	mode_chg = TRUE;
+	dpy_track(s);
+	dpy_time(s, FALSE);
+	mode_chg = FALSE;
+
+	/* This is a hack to prevent the tooltip from popping up due to
+	 * the main window reconfiguration.
+	 */
+	skip_next_tooltip = TRUE;
+
+	/* Overload the function of this button to also
+	 * dump the contents of the curstat_t structure
+	 * in debug mode
+	 */
+	if (app_data.debug)
+		di_dump_curstat(s);
+}
+
+
 /*
  * cd_load_eject
  *	Main window load/eject button callback function
@@ -2349,12 +2715,12 @@ cd_load_eject(Widget w, XtPointer client_data, XtPointer call_data)
 
 
 /*
- * cd_poweroff
+ * cd_quit_btn
  *	Main window quit button callback function
  */
 /*ARGSUSED*/
 void
-cd_poweroff(Widget w, XtPointer client_data, XtPointer call_data)
+cd_quit_btn(Widget w, XtPointer client_data, XtPointer call_data)
 {
 	DBGPRN(errfp, "\n* QUIT\n");
 
@@ -2404,6 +2770,10 @@ cd_time(Widget w, XtPointer client_data, XtPointer call_data)
 
 	switch (s->time_dpy) {
 	case T_ELAPSED:
+		s->time_dpy = T_ELAPSED_DISC;
+		break;
+
+	case T_ELAPSED_DISC:
 		s->time_dpy = T_REMAIN_TRACK;
 		break;
 
@@ -2488,6 +2858,7 @@ void
 cd_play_pause(Widget w, XtPointer client_data, XtPointer call_data)
 {
 	curstat_t	*s = (curstat_t *)(void *) client_data;
+	sword32_t	sav_trk;
 
 	DBGPRN(errfp, "\n* PLAY_PAUSE\n");
 
@@ -2503,8 +2874,15 @@ cd_play_pause(Widget w, XtPointer client_data, XtPointer call_data)
 		}
 	}
 
+	sav_trk = s->cur_trk;
+
 	di_play_pause(s);
 
+	if (sav_trk >= 0) {
+		/* Update curfile */
+		dbprog_curfileupd();
+	}
+
 	switch (s->mode) {
 	case MOD_PAUSE:
 		dpy_time(s, FALSE);
@@ -3083,18 +3461,20 @@ cd_keypad_enter(Widget w, XtPointer client_data, XtPointer call_data)
 		}
 
 		s->cur_trk_addr = warp_offset;
-		blktomsf(s->cur_trk_addr,
-			 &s->cur_trk_min,
-			 &s->cur_trk_sec,
-			 &s->cur_trk_frame,
-			 0
+		util_blktomsf(
+			s->cur_trk_addr,
+			&s->cur_trk_min,
+			&s->cur_trk_sec,
+			&s->cur_trk_frame,
+			0
 		);
 		s->cur_tot_addr = s->trkinfo[i].addr + warp_offset;
-		blktomsf(s->cur_tot_addr,
-			 &s->cur_tot_min,
-			 &s->cur_tot_sec,
-			 &s->cur_tot_frame,
-			 MSF_OFFSET(s)
+		util_blktomsf(
+			s->cur_tot_addr,
+			&s->cur_tot_min,
+			&s->cur_tot_sec,
+			&s->cur_tot_frame,
+			MSF_OFFSET
 		);
 
 		/* Start playback at new position */
@@ -3175,18 +3555,20 @@ cd_warp(Widget w, XtPointer client_data, XtPointer call_data)
 		DBGPRN(errfp, "\n* TRACK WARP\n");
 
 		s->cur_trk_addr = warp_offset;
-		blktomsf(s->cur_trk_addr,
-			 &s->cur_trk_min,
-			 &s->cur_trk_sec,
-			 &s->cur_trk_frame,
-			 0
+		util_blktomsf(
+			s->cur_trk_addr,
+			&s->cur_trk_min,
+			&s->cur_trk_sec,
+			&s->cur_trk_frame,
+			0
 		);
 		s->cur_tot_addr = s->trkinfo[i].addr + warp_offset;
-		blktomsf(s->cur_tot_addr,
-			 &s->cur_tot_min,
-			 &s->cur_tot_sec,
-			 &s->cur_tot_frame,
-			 MSF_OFFSET(s)
+		util_blktomsf(
+			s->cur_tot_addr,
+			&s->cur_tot_min,
+			&s->cur_tot_sec,
+			&s->cur_tot_frame,
+			MSF_OFFSET
 		);
 
 		if (s->mode == MOD_STOP) {
@@ -3230,7 +3612,7 @@ cd_options_popup(Widget w, XtPointer client_data, XtPointer call_data)
 		return;
 	}
 
-	/* Pop up keypad window */
+	/* Pop up options window */
 	XtManageChild(widgets.options.form);
 
 	XmProcessTraversal(
@@ -3269,7 +3651,7 @@ cd_options_reset(Widget w, XtPointer client_data, XtPointer call_data)
 	if (call_data != NULL) {
 		/* Re-read defaults */
 
-		bdevname = basename(app_data.device);
+		bdevname = util_basename(app_data.device);
 
 		/* Get system-wide device-specific configuration parameters */
 		(void) sprintf(str, SYS_DSCFG_PATH, app_data.libdir, bdevname);
@@ -3277,7 +3659,7 @@ cd_options_reset(Widget w, XtPointer client_data, XtPointer call_data)
 
 		/* Get user device-specific configuration parameters */
 		(void) sprintf(str, USR_DSCFG_PATH,
-			       homedir(get_ouid()), bdevname);
+			       util_homedir(util_get_ouid()), bdevname);
 		di_devspec_parmload(str, FALSE);
 
 		/* Set the channel routing */
@@ -3412,7 +3794,8 @@ cd_options_save(Widget w, XtPointer client_data, XtPointer call_data)
 
 	/* Save user device-specific configuration parameters */
 	(void) sprintf(str, USR_DSCFG_PATH,
-		       homedir(get_ouid()), basename(app_data.device));
+		       util_homedir(util_get_ouid()),
+		       util_basename(app_data.device));
 	di_devspec_parmsave(str);
 
 	/* Make the Reset and Save buttons insensitive */
@@ -3857,17 +4240,16 @@ cd_about(Widget w, XtPointer client_data, XtPointer call_data)
 
 
 /*
- * cd_help
+ * cd_help_popup
  *	Program help window popup callback function
  */
 /*ARGSUSED*/
 void
 cd_help_popup(Widget w, XtPointer client_data, XtPointer call_data)
 {
-	if (w == widgets.main.help_btn && XtIsManaged(widgets.help.form)) {
+	if (w == widgets.main.help_btn && help_isactive()) {
 		/* Pop down help window */
-		XtUnmanageChild(widgets.help.form);
-
+		help_popdown();
 		return;
 	}
 
@@ -3885,8 +4267,8 @@ void
 cd_help_popdown(Widget w, XtPointer client_data, XtPointer call_data)
 {
 	/* Pop down help window */
-	if (XtIsManaged(widgets.help.form))
-		XtUnmanageChild(widgets.help.form);
+	if (help_isactive())
+		help_popdown();
 }
 
 
@@ -3970,31 +4352,31 @@ cd_rmcallback(Widget w, XtPointer client_data, XtPointer call_data)
 
 
 /*
- * cd_form_focus_chg
+ * cd_shell_focus_chg
  *	Focus change callback.  Used to implement keyboard grabs for
  *	hotkey handling.
  */
 /*ARGSUSED*/
 void
-cd_form_focus_chg(Widget w, XtPointer client_data, XtPointer call_data)
+cd_shell_focus_chg(Widget w, XtPointer client_data, XtPointer call_data)
 {
 	XmAnyCallbackStruct	*p = (XmAnyCallbackStruct *)(void *) call_data;
-	Widget			form = (Widget) client_data;
-	static Widget		prev_form = (Widget) NULL;
+	Widget			shell = (Widget) client_data;
+	static Widget		prev_shell = (Widget) NULL;
 
-	if (p->reason != XmCR_FOCUS || form == (Widget) NULL)
+	if (p->reason != XmCR_FOCUS || shell == (Widget) NULL)
 		return;
 
-	if (prev_form != NULL) {
-		if (form == prev_form)
+	if (prev_shell != NULL) {
+		if (shell == prev_shell)
 			return;
 		else
-			hotkey_ungrabkeys(prev_form);
+			hotkey_ungrabkeys(prev_shell);
 	}
 
-	if (form != widgets.toplevel) {
-		hotkey_grabkeys(form);
-		prev_form = form;
+	if (shell != widgets.toplevel) {
+		hotkey_grabkeys(shell);
+		prev_shell = shell;
 	}
 }
 
@@ -4020,20 +4402,88 @@ cd_exit(Widget w, XtPointer client_data, XtPointer call_data)
 }
 
 
+/*
+ * cd_tooltip_cancel
+ *	Cancel the tooltip window
+ */
+/*ARGSUSED*/
+void
+cd_tooltip_cancel(Widget w, XtPointer client_data, XtPointer call_data)
+{
+	cd_tooltip_popdown(widgets.tooltip.shell);
+}
+
+
 /**************** ^^ Callback routines ^^ ****************/
 
 /***************** vv Event Handlers vv ******************/
 
 
+/* Mapping table for main window controls and their label color change
+ * function pointer, and associated label pixmaps.
+ */
+struct wpix_tab {
+	Widget	*wptr;
+	void	(*set_func)(Widget w, Pixmap px, Pixel color);
+	Pixmap	*fpx;
+	Pixmap	*hpx;
+} wpix_tab[] = {
+    { &widgets.main.mode_btn, set_btn_color,
+      &pixmaps.main.mode_pixmap, &pixmaps.main.mode_hlpixmap },
+    { &widgets.main.eject_btn, set_btn_color,
+      &pixmaps.main.eject_pixmap, &pixmaps.main.eject_hlpixmap },
+    { &widgets.main.quit_btn, set_btn_color,
+      &pixmaps.main.quit_pixmap, &pixmaps.main.quit_hlpixmap },
+    { &widgets.main.dbprog_btn, set_btn_color,
+      &pixmaps.main.dbprog_pixmap, &pixmaps.main.dbprog_hlpixmap },
+    { &widgets.main.help_btn, set_btn_color,
+      &pixmaps.main.help_pixmap, &pixmaps.main.help_hlpixmap },
+    { &widgets.main.options_btn, set_btn_color,
+      &pixmaps.main.options_pixmap, &pixmaps.main.options_hlpixmap },
+    { &widgets.main.time_btn, set_btn_color,
+      &pixmaps.main.time_pixmap, &pixmaps.main.time_hlpixmap },
+    { &widgets.main.ab_btn, set_btn_color,
+      &pixmaps.main.ab_pixmap, &pixmaps.main.ab_hlpixmap },
+    { &widgets.main.sample_btn, set_btn_color,
+      &pixmaps.main.sample_pixmap, &pixmaps.main.sample_hlpixmap },
+    { &widgets.main.keypad_btn, set_btn_color,
+      &pixmaps.main.keypad_pixmap, &pixmaps.main.keypad_hlpixmap },
+    { &widgets.main.level_scale, set_scale_color,
+      NULL, NULL },
+    { &widgets.main.playpause_btn, set_btn_color,
+      &pixmaps.main.playpause_pixmap, &pixmaps.main.playpause_hlpixmap },
+    { &widgets.main.stop_btn, set_btn_color,
+      &pixmaps.main.stop_pixmap, &pixmaps.main.stop_hlpixmap },
+    { &widgets.main.prevdisc_btn, set_btn_color,
+      &pixmaps.main.prevdisc_pixmap, &pixmaps.main.prevdisc_hlpixmap },
+    { &widgets.main.nextdisc_btn, set_btn_color,
+      &pixmaps.main.nextdisc_pixmap, &pixmaps.main.nextdisc_hlpixmap },
+    { &widgets.main.prevtrk_btn, set_btn_color,
+      &pixmaps.main.prevtrk_pixmap, &pixmaps.main.prevtrk_hlpixmap },
+    { &widgets.main.nexttrk_btn, set_btn_color,
+      &pixmaps.main.nexttrk_pixmap, &pixmaps.main.nexttrk_hlpixmap },
+    { &widgets.main.previdx_btn, set_btn_color,
+      &pixmaps.main.previdx_pixmap, &pixmaps.main.previdx_hlpixmap },
+    { &widgets.main.nextidx_btn, set_btn_color,
+      &pixmaps.main.nextidx_pixmap, &pixmaps.main.nextidx_hlpixmap },
+    { &widgets.main.rew_btn, set_btn_color,
+      &pixmaps.main.rew_pixmap, &pixmaps.main.rew_hlpixmap },
+    { &widgets.main.ff_btn, set_btn_color,
+      &pixmaps.main.ff_pixmap, &pixmaps.main.ff_hlpixmap },
+    { NULL, NULL, 0, 0 }
+};
+
+
 /*
- * cd_btn_focus_chg
- *	Pushbutton keyboard focus change event handler.  Used to change
- *	the main window pushbuttons' label color.
+ * cd_focus_chg
+ *	Widget keyboard focus change event handler.  Used to change
+ *	the main window controls' label color.
  */
 /*ARGSUSED*/
 void
-cd_btn_focus_chg(Widget w, XtPointer client_data, XEvent *ev, Boolean *cont)
+cd_focus_chg(Widget w, XtPointer client_data, XEvent *ev)
 {
+	int		i;
 	unsigned char	focuspolicy;		
 	static bool_t	first = TRUE;
 	static int	count = 0;
@@ -4069,99 +4519,71 @@ cd_btn_focus_chg(Widget w, XtPointer client_data, XEvent *ev, Boolean *cont)
 		if (count <= 0)
 			return;
 
-		/* Restore original forground pixmap */
-		if (w == widgets.main.eject_btn)
-			set_btn_color(w, pixmaps.main.eject_pixmap, fg);
-		else if (w == widgets.main.poweroff_btn)
-			set_btn_color(w, pixmaps.main.poweroff_pixmap, fg);
-		else if (w == widgets.main.dbprog_btn)
-			set_btn_color(w, pixmaps.main.dbprog_pixmap, fg);
-		else if (w == widgets.main.help_btn)
-			set_btn_color(w, pixmaps.main.help_pixmap, fg);
-		else if (w == widgets.main.options_btn)
-			set_btn_color(w, pixmaps.main.options_pixmap, fg);
-		else if (w == widgets.main.time_btn)
-			set_btn_color(w, pixmaps.main.time_pixmap, fg);
-		else if (w == widgets.main.ab_btn)
-			set_btn_color(w, pixmaps.main.ab_pixmap, fg);
-		else if (w == widgets.main.sample_btn)
-			set_btn_color(w, pixmaps.main.sample_pixmap, fg);
-		else if (w == widgets.main.keypad_btn)
-			set_btn_color(w, pixmaps.main.keypad_pixmap, fg);
-		else if (w == widgets.main.level_scale)
-			set_scale_color(w, fg);
-		else if (w == widgets.main.playpause_btn)
-			set_btn_color(w, pixmaps.main.playpause_pixmap, fg);
-		else if (w == widgets.main.stop_btn)
-			set_btn_color(w, pixmaps.main.stop_pixmap, fg);
-		else if (w == widgets.main.prevdisc_btn)
-			set_btn_color(w, pixmaps.main.prevdisc_pixmap, fg);
-		else if (w == widgets.main.nextdisc_btn)
-			set_btn_color(w, pixmaps.main.nextdisc_pixmap, fg);
-		else if (w == widgets.main.prevtrk_btn)
-			set_btn_color(w, pixmaps.main.prevtrk_pixmap, fg);
-		else if (w == widgets.main.nexttrk_btn)
-			set_btn_color(w, pixmaps.main.nexttrk_pixmap, fg);
-		else if (w == widgets.main.previdx_btn)
-			set_btn_color(w, pixmaps.main.previdx_pixmap, fg);
-		else if (w == widgets.main.nextidx_btn)
-			set_btn_color(w, pixmaps.main.nextidx_pixmap, fg);
-		else if (w == widgets.main.rew_btn)
-			set_btn_color(w, pixmaps.main.rew_pixmap, fg);
-		else if (w == widgets.main.ff_btn)
-			set_btn_color(w, pixmaps.main.ff_pixmap, fg);
-
+		/* Restore original foreground pixmap */
+		for (i = 0; wpix_tab[i].set_func != NULL; i++) {
+			if (w == *(wpix_tab[i].wptr)) {
+				wpix_tab[i].set_func(w,
+					(wpix_tab[i].fpx == NULL) ?
+					    (Pixmap) 0 : *(wpix_tab[i].fpx),
+					fg
+				);
+				break;
+			}
+		}
 		count--;
 	}
 	else if (ev->type == FocusIn) {
 		if (count >= 1)
 			return;
 
-		/* Set new highlighted forground pixmap */
-		if (w == widgets.main.eject_btn)
-			set_btn_color(w, pixmaps.main.eject_hlpixmap, hl);
-		else if (w == widgets.main.poweroff_btn)
-			set_btn_color(w, pixmaps.main.poweroff_hlpixmap, hl);
-		else if (w == widgets.main.dbprog_btn)
-			set_btn_color(w, pixmaps.main.dbprog_hlpixmap, hl);
-		else if (w == widgets.main.help_btn)
-			set_btn_color(w, pixmaps.main.help_hlpixmap, hl);
-		else if (w == widgets.main.options_btn)
-			set_btn_color(w, pixmaps.main.options_hlpixmap, hl);
-		else if (w == widgets.main.time_btn)
-			set_btn_color(w, pixmaps.main.time_hlpixmap, hl);
-		else if (w == widgets.main.ab_btn)
-			set_btn_color(w, pixmaps.main.ab_hlpixmap, hl);
-		else if (w == widgets.main.sample_btn)
-			set_btn_color(w, pixmaps.main.sample_hlpixmap, hl);
-		else if (w == widgets.main.keypad_btn)
-			set_btn_color(w, pixmaps.main.keypad_hlpixmap, hl);
-		else if (w == widgets.main.level_scale)
-			set_scale_color(w, hl);
-		else if (w == widgets.main.playpause_btn)
-			set_btn_color(w, pixmaps.main.playpause_hlpixmap, hl);
-		else if (w == widgets.main.stop_btn)
-			set_btn_color(w, pixmaps.main.stop_hlpixmap, hl);
-		else if (w == widgets.main.prevdisc_btn)
-			set_btn_color(w, pixmaps.main.prevdisc_hlpixmap, hl);
-		else if (w == widgets.main.nextdisc_btn)
-			set_btn_color(w, pixmaps.main.nextdisc_hlpixmap, hl);
-		else if (w == widgets.main.prevtrk_btn)
-			set_btn_color(w, pixmaps.main.prevtrk_hlpixmap, hl);
-		else if (w == widgets.main.nexttrk_btn)
-			set_btn_color(w, pixmaps.main.nexttrk_hlpixmap, hl);
-		else if (w == widgets.main.previdx_btn)
-			set_btn_color(w, pixmaps.main.previdx_hlpixmap, hl);
-		else if (w == widgets.main.nextidx_btn)
-			set_btn_color(w, pixmaps.main.nextidx_hlpixmap, hl);
-		else if (w == widgets.main.rew_btn)
-			set_btn_color(w, pixmaps.main.rew_hlpixmap, hl);
-		else if (w == widgets.main.ff_btn)
-			set_btn_color(w, pixmaps.main.ff_hlpixmap, hl);
-
+		/* Set new highlighted foreground pixmap */
+		for (i = 0; wpix_tab[i].set_func != NULL; i++) {
+			if (w == *(wpix_tab[i].wptr)) {
+				wpix_tab[i].set_func(w,
+					(wpix_tab[i].fpx == NULL) ?
+					    (Pixmap) 0 : *(wpix_tab[i].hpx),
+					hl
+				);
+				break;
+			}
+		}
 		count++;
 	}
 }
 
+/*
+ * cd_xing_chg
+ *	Widget enter/leave crossing event handler.  Used to manage
+ *	pop-up tool-tips.
+ */
+/*ARGSUSED*/
+void
+cd_xing_chg(Widget w, XtPointer client_data, XEvent *ev)
+{
+	if (!app_data.tooltip_enable ||
+	    ev->xcrossing.mode != NotifyNormal ||
+	    ev->xcrossing.detail == NotifyInferior)
+		return;
+
+	if (ev->type == EnterNotify) {
+		if (skip_next_tooltip) {
+			skip_next_tooltip = FALSE;
+			return;
+		}
+
+		if (tooltip1_id < 0) {
+			tooltip1_id = cd_timeout(
+				app_data.tooltip_delay,
+				cd_tooltip_popup,
+				(byte_t *) w
+			);
+		}
+	}
+	else if (ev->type == LeaveNotify) {
+		cd_tooltip_popdown(widgets.tooltip.shell);
+	}
+}
+
+
 /***************** ^^ Event Handlers ^^ ******************/
 
