@@ -1,7 +1,7 @@
 /*
  *   xmcd - Motif(tm) CD Audio Player
  *
- *   Copyright (C) 1993-1997  Ti Kan
+ *   Copyright (C) 1993-1998  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -20,18 +20,23 @@
  *
  */
 #ifndef LINT
-static char *_geom_c_ident_ = "@(#)geom.c	6.13 97/07/24";
+static char *_geom_c_ident_ = "@(#)geom.c	6.28 98/05/19";
 #endif
 
-#define _XMINC_
-
 #include "common_d/appenv.h"
+#include "xmcd_d/xmcd.h"
 #include "xmcd_d/widget.h"
 #include "xmcd_d/geom.h"
 
 
+#define WM_FUDGE	15		/* WM decoration fudge factor */
+#define WLIST_SIZE	30		/* Widget list size */
+
+
 extern appdata_t	app_data;
 
+STATIC int		main_mode;	/* Main window mode */
+
 
 /***********************
  *  internal routines  *
@@ -39,8 +44,8 @@ extern appdata_t	app_data;
 
 
 /*
- * force_main_geometry
- *	Set the geometry of the widgets in the main window.
+ * geom_normal_force
+ *	Set the geometry of the widgets in the main window in normal mode.
  *
  * Args:
  *	m - Pointer to the main widgets structure.
@@ -49,14 +54,26 @@ extern appdata_t	app_data;
  *	Nothing.
  */
 STATIC void
-force_main_geometry(widgets_t *m)
+geom_normal_force(widgets_t *m)
 {
-	XmString	dash = XmStringCreateSimple("-"),
-			blank = XmStringCreateSimple("");
-
-
-	/* Main window widgets */
+	/* Set normal mode main window size */
+	XtVaSetValues(m->toplevel,
+		XmNwidth, app_data.normal_width,
+		XmNheight, app_data.normal_height,
+		NULL
+	);
 
+	XtVaSetValues(m->main.mode_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNleftPosition, LEFT_MODE,
+		XmNrightPosition, RIGHT_MODE,
+		XmNtopPosition, TOP_MODE,
+		XmNbottomPosition, BOTTOM_MODE,
+		NULL
+	);
 	XtVaSetValues(m->main.chkbox_frm,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
@@ -69,17 +86,6 @@ force_main_geometry(widgets_t *m)
 		XmNshadowType, XmSHADOW_OUT,
 		NULL
 	);
-	XtVaSetValues(m->main.btnlbl_btn,
-		XmNheight, 16,
-		XmNrecomputeSize, False,
-		NULL
-	);
-	if (!app_data.main_showfocus) {
-		XtVaSetValues(m->main.btnlbl_btn,
-			XmNhighlightThickness, 0,
-			NULL
-		);
-	}
 	XtVaSetValues(m->main.lock_btn,
 		XmNheight, 16,
 		XmNrecomputeSize, False,
@@ -124,18 +130,17 @@ force_main_geometry(widgets_t *m)
 		XmNbottomPosition, BOTTOM_EJECT,
 		NULL
 	);
-	XtVaSetValues(m->main.poweroff_btn,
+	XtVaSetValues(m->main.quit_btn,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
 		XmNtopAttachment, XmATTACH_POSITION,
 		XmNbottomAttachment, XmATTACH_POSITION,
-		XmNleftPosition, LEFT_POWEROFF,
-		XmNrightPosition, RIGHT_POWEROFF,
-		XmNtopPosition, TOP_POWEROFF,
-		XmNbottomPosition, BOTTOM_POWEROFF,
+		XmNleftPosition, LEFT_QUIT,
+		XmNrightPosition, RIGHT_QUIT,
+		XmNtopPosition, TOP_QUIT,
+		XmNbottomPosition, BOTTOM_QUIT,
 		NULL
 	);
-
 	XtVaSetValues(m->main.disc_ind,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
@@ -145,7 +150,6 @@ force_main_geometry(widgets_t *m)
 		XmNrightPosition, RIGHT_DISCIND,
 		XmNtopPosition, TOP_DISCIND,
 		XmNbottomPosition, BOTTOM_DISCIND,
-		XmNlabelString, dash,
 		NULL
 	);
 	XtVaSetValues(m->main.track_ind,
@@ -157,7 +161,6 @@ force_main_geometry(widgets_t *m)
 		XmNrightPosition, RIGHT_TRACKIND,
 		XmNtopPosition, TOP_TRACKIND,
 		XmNbottomPosition, BOTTOM_TRACKIND,
-		XmNlabelString, dash,
 		NULL
 	);
 	XtVaSetValues(m->main.index_ind,
@@ -169,7 +172,6 @@ force_main_geometry(widgets_t *m)
 		XmNrightPosition, RIGHT_INDEXIND,
 		XmNtopPosition, TOP_INDEXIND,
 		XmNbottomPosition, BOTTOM_INDEXIND,
-		XmNlabelString, dash,
 		NULL
 	);
 	XtVaSetValues(m->main.time_ind,
@@ -181,7 +183,6 @@ force_main_geometry(widgets_t *m)
 		XmNrightPosition, RIGHT_TIMEIND,
 		XmNtopPosition, TOP_TIMEIND,
 		XmNbottomPosition, BOTTOM_TIMEIND,
-		XmNlabelString, dash,
 		NULL
 	);
 	XtVaSetValues(m->main.rptcnt_ind,
@@ -193,7 +194,6 @@ force_main_geometry(widgets_t *m)
 		XmNrightPosition, RIGHT_RPTCNTIND,
 		XmNtopPosition, TOP_RPTCNTIND,
 		XmNbottomPosition, BOTTOM_RPTCNTIND,
-		XmNlabelString, dash,
 		NULL
 	);
 	XtVaSetValues(m->main.dbmode_ind,
@@ -205,7 +205,6 @@ force_main_geometry(widgets_t *m)
 		XmNrightPosition, RIGHT_DBMODEIND,
 		XmNtopPosition, TOP_DBMODEIND,
 		XmNbottomPosition, BOTTOM_DBMODEIND,
-		XmNlabelString, blank,
 		NULL
 	);
 	XtVaSetValues(m->main.progmode_ind,
@@ -217,7 +216,6 @@ force_main_geometry(widgets_t *m)
 		XmNrightPosition, RIGHT_PROGMODEIND,
 		XmNtopPosition, TOP_PROGMODEIND,
 		XmNbottomPosition, BOTTOM_PROGMODEIND,
-		XmNlabelString, blank,
 		NULL
 	);
 	XtVaSetValues(m->main.timemode_ind,
@@ -229,7 +227,6 @@ force_main_geometry(widgets_t *m)
 		XmNrightPosition, RIGHT_TIMEMODEIND,
 		XmNtopPosition, TOP_TIMEMODEIND,
 		XmNbottomPosition, BOTTOM_TIMEMODEIND,
-		XmNlabelString, blank,
 		NULL
 	);
 	XtVaSetValues(m->main.playmode_ind,
@@ -241,7 +238,6 @@ force_main_geometry(widgets_t *m)
 		XmNrightPosition, RIGHT_PLAYMODEIND,
 		XmNtopPosition, TOP_PLAYMODEIND,
 		XmNbottomPosition, BOTTOM_PLAYMODEIND,
-		XmNlabelString, blank,
 		NULL
 	);
 	XtVaSetValues(m->main.dtitle_ind,
@@ -254,7 +250,6 @@ force_main_geometry(widgets_t *m)
 		XmNtopPosition, TOP_DTITLEIND,
 		XmNbottomPosition, BOTTOM_DTITLEIND,
 		XmNalignment, XmALIGNMENT_BEGINNING,
-		XmNlabelString, blank,
 		NULL
 	);
 	XtVaSetValues(m->main.ttitle_ind,
@@ -267,7 +262,6 @@ force_main_geometry(widgets_t *m)
 		XmNtopPosition, TOP_TTITLEIND,
 		XmNbottomPosition, BOTTOM_TTITLEIND,
 		XmNalignment, XmALIGNMENT_BEGINNING,
-		XmNlabelString, blank,
 		NULL
 	);
 	XtVaSetValues(m->main.dbprog_btn,
@@ -356,11 +350,6 @@ force_main_geometry(widgets_t *m)
 		XmNrightPosition, RIGHT_LEVEL,
 		XmNtopPosition, TOP_LEVEL,
 		XmNbottomPosition, BOTTOM_LEVEL,
-		XmNshowValue, True,
-		XmNminimum, 0,
-		XmNmaximum, 100,
-		XmNorientation, XmHORIZONTAL,
-		XmNprocessingDirection, XmMAX_ON_RIGHT,
 		NULL
 	);
 	XtVaSetValues(m->main.playpause_btn,
@@ -473,14 +462,249 @@ force_main_geometry(widgets_t *m)
 		XmNbottomPosition, BOTTOM_FF,
 		NULL
 	);
-
-	XmStringFree(dash);
-	XmStringFree(blank);
 }
 
 
 /*
- * force_keypad_geometry
+ * geom_basic_force
+ *	Set the geometry of the widgets in the main window in basic mode.
+ *
+ * Args:
+ *	m - Pointer to the main widgets structure.
+ *
+ * Return:
+ *	Nothing.
+ */
+STATIC void
+geom_basic_force(widgets_t *m)
+{
+	/* Set basic mode main window size */
+	XtVaSetValues(m->toplevel,
+		XmNwidth, app_data.basic_width,
+		XmNheight, app_data.basic_height,
+		NULL
+	);
+
+	XtVaSetValues(m->main.mode_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNleftPosition, LEFT_BMODE,
+		XmNrightPosition, RIGHT_BMODE,
+		XmNtopPosition, TOP_BMODE,
+		XmNbottomPosition, BOTTOM_BMODE,
+		NULL
+	);
+	XtVaSetValues(m->main.eject_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNleftPosition, LEFT_BEJECT,
+		XmNrightPosition, RIGHT_BEJECT,
+		XmNtopPosition, TOP_BEJECT,
+		XmNbottomPosition, BOTTOM_BEJECT,
+		NULL
+	);
+	XtVaSetValues(m->main.quit_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNleftPosition, LEFT_BQUIT,
+		XmNrightPosition, RIGHT_BQUIT,
+		XmNtopPosition, TOP_BQUIT,
+		XmNbottomPosition, BOTTOM_BQUIT,
+		NULL
+	);
+	XtVaSetValues(m->main.track_ind,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNleftPosition, LEFT_BTRACKIND,
+		XmNrightPosition, RIGHT_BTRACKIND,
+		XmNtopPosition, TOP_BTRACKIND,
+		XmNbottomPosition, BOTTOM_BTRACKIND,
+		NULL
+	);
+	XtVaSetValues(m->main.time_ind,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNleftPosition, LEFT_BTIMEIND,
+		XmNrightPosition, RIGHT_BTIMEIND,
+		XmNtopPosition, TOP_BTIMEIND,
+		XmNbottomPosition, BOTTOM_BTIMEIND,
+		NULL
+	);
+	XtVaSetValues(m->main.level_scale,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNleftPosition, LEFT_BLEVEL,
+		XmNrightPosition, RIGHT_BLEVEL,
+		XmNtopPosition, TOP_BLEVEL,
+		XmNbottomPosition, BOTTOM_BLEVEL,
+		NULL
+	);
+	XtVaSetValues(m->main.playpause_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNleftPosition, LEFT_BPLAYPAUSE,
+		XmNrightPosition, RIGHT_BPLAYPAUSE,
+		XmNtopPosition, TOP_BPLAYPAUSE,
+		XmNbottomPosition, BOTTOM_BPLAYPAUSE,
+		NULL
+	);
+	XtVaSetValues(m->main.stop_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNleftPosition, LEFT_BSTOP,
+		XmNrightPosition, RIGHT_BSTOP,
+		XmNtopPosition, TOP_BSTOP,
+		XmNbottomPosition, BOTTOM_BSTOP,
+		NULL
+	);
+	XtVaSetValues(m->main.prevtrk_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNleftPosition, LEFT_BPREVTRK,
+		XmNrightPosition, RIGHT_BPREVTRK,
+		XmNtopPosition, TOP_BPREVTRK,
+		XmNbottomPosition, BOTTOM_BPREVTRK,
+		NULL
+	);
+	XtVaSetValues(m->main.nexttrk_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNleftPosition, LEFT_BNEXTTRK,
+		XmNrightPosition, RIGHT_BNEXTTRK,
+		XmNtopPosition, TOP_BNEXTTRK,
+		XmNbottomPosition, BOTTOM_BNEXTTRK,
+		NULL
+	);
+}
+
+
+/*
+ * geom_main_force
+ *	Set the geometry of the widgets in the main window.
+ *
+ * Args:
+ *	m - Pointer to the main widgets structure.
+ *
+ * Return:
+ *	Nothing.
+ */
+STATIC void
+geom_main_force(widgets_t *m)
+{
+	Display		*display = XtDisplay(m->toplevel);
+	int		screen = DefaultScreen(display),
+			i;
+	Dimension	width,
+			height,
+			swidth,
+			sheight;
+	Position	x,
+			y,
+			new_x,
+			new_y;
+	static Widget	wlist[WLIST_SIZE];
+	static int	wlist_size = 0;
+	static bool_t	first = TRUE;
+
+	if (first) {
+		first = FALSE;
+
+		/* Set up the list of main window widgets that gets
+		 * managed and unmanaged when switching modes.
+		 */
+		i = 0;
+		wlist[i++] = m->main.chkbox_frm;
+		wlist[i++] = m->main.disc_ind;
+		wlist[i++] = m->main.index_ind;
+		wlist[i++] = m->main.rptcnt_ind;
+		wlist[i++] = m->main.dbmode_ind;
+		wlist[i++] = m->main.timemode_ind;
+		wlist[i++] = m->main.playmode_ind;
+		wlist[i++] = m->main.dtitle_ind;
+		wlist[i++] = m->main.ttitle_ind;
+		wlist[i++] = m->main.dbprog_btn;
+		wlist[i++] = m->main.dbprog_btn;
+		wlist[i++] = m->main.options_btn;
+		wlist[i++] = m->main.help_btn;
+		wlist[i++] = m->main.time_btn;
+		wlist[i++] = m->main.ab_btn;
+		wlist[i++] = m->main.sample_btn;
+		wlist[i++] = m->main.keypad_btn;
+		wlist[i++] = m->main.prevdisc_btn;
+		wlist[i++] = m->main.nextdisc_btn;
+		wlist[i++] = m->main.previdx_btn;
+		wlist[i++] = m->main.nextidx_btn;
+		wlist[i++] = m->main.rew_btn;
+		wlist[i++] = m->main.ff_btn;
+		wlist_size = i;
+	}
+
+	if (main_mode == MAIN_NORMAL) {
+		XtManageChildren((WidgetList) wlist, wlist_size);
+		geom_normal_force(m);
+	}
+	else if (main_mode == MAIN_BASIC) {
+		XtUnmanageChildren((WidgetList) wlist, wlist_size);
+		geom_basic_force(m);
+	}
+
+	/* Make sure the full window will appear entirely within
+	 * the screen boundaries after resizing.  Note that this
+	 * tries to accommodate window manager decorations around
+	 * the window (with the WM_FUDGE factor), but it's not always
+	 * sufficient.
+	 */
+	swidth = (Dimension) XDisplayWidth(display, screen);
+	sheight = (Dimension) XDisplayHeight(display, screen);
+
+	XtVaGetValues(m->toplevel,
+		XmNx, &x,
+		XmNy, &y,
+		XmNwidth, &width,
+		XmNheight, &height,
+		NULL
+	);
+
+	new_x = x;
+	new_y = y;
+
+	if (x < 0)
+		new_x = WM_FUDGE;
+	else if ((Dimension) (x + width) > swidth)
+		new_x = swidth - width - WM_FUDGE;
+
+	if (y < 0)
+		new_y = WM_FUDGE;
+	else if ((Dimension) (y + height) > sheight)
+		new_y = sheight - height - WM_FUDGE;
+
+	if (new_x != x || new_y != y)
+		XtVaSetValues(m->toplevel, XmNx, new_x, XmNy, new_y, NULL);
+}
+
+
+/*
+ * geom_keypad_force
  *	Set the geometry of the widgets in the keypad window.
  *
  * Args:
@@ -490,10 +714,8 @@ force_main_geometry(widgets_t *m)
  *	Nothing.
  */
 STATIC void
-force_keypad_geometry(widgets_t *m)
+geom_keypad_force(widgets_t *m)
 {
-	/* Keypad popup window widgets */
-
 	XtVaSetValues(m->keypad.keypad_lbl,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
@@ -707,12 +929,6 @@ force_keypad_geometry(widgets_t *m)
 		XmNrightPosition, RIGHT_WARPSCALE,
 		XmNtopPosition, TOP_WARPSCALE,
 		XmNbottomPosition, BOTTOM_WARPSCALE,
-		XmNshowValue, False,
-		XmNminimum, 0,
-		XmNmaximum, 255,
-		XmNorientation, XmHORIZONTAL,
-		XmNprocessingDirection, XmMAX_ON_RIGHT,
-		XmNhighlightOnEnter, True,
 		NULL
 	);
 
@@ -741,7 +957,7 @@ force_keypad_geometry(widgets_t *m)
 
 
 /*
- * force_options_geometry
+ * geom_options_force
  *	Set the geometry of the widgets in the options window.
  *
  * Args:
@@ -751,10 +967,8 @@ force_keypad_geometry(widgets_t *m)
  *	Nothing.
  */
 STATIC void
-force_options_geometry(widgets_t *m)
+geom_options_force(widgets_t *m)
 {
-	/* Options popup window widgets */
-
 	XtVaSetValues(m->options.load_lbl,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
@@ -1083,13 +1297,6 @@ force_options_geometry(widgets_t *m)
 		XmNrightPosition, RIGHT_BAL_SCALE,
 		XmNtopPosition, TOP_BAL_SCALE,
 		XmNbottomPosition, BOTTOM_BAL_SCALE,
-		XmNshowValue, False,
-		XmNminimum, -50,
-		XmNmaximum, 50,
-		XmNvalue, 0,
-		XmNorientation, XmHORIZONTAL,
-		XmNprocessingDirection, XmMAX_ON_RIGHT,
-		XmNhighlightOnEnter, True,
 		NULL
 	);
 
@@ -1152,7 +1359,7 @@ force_options_geometry(widgets_t *m)
 
 
 /*
- * force_dbprog_geometry
+ * geom_dbprog_force
  *	Set the geometry of the widgets in the database/program window.
  *
  * Args:
@@ -1162,10 +1369,8 @@ force_options_geometry(widgets_t *m)
  *	Nothing.
  */
 STATIC void
-force_dbprog_geometry(widgets_t *m)
+geom_dbprog_force(widgets_t *m)
 {
-	/* CD program/database popup window widgets */
-
 	XtVaSetValues(m->dbprog.tottime_lbl,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
@@ -1202,15 +1407,15 @@ force_dbprog_geometry(widgets_t *m)
 		NULL
 	);
 
-	XtVaSetValues(m->dbprog.about_btn,
+	XtVaSetValues(m->dbprog.wwwwarp_btn,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
 		XmNtopAttachment, XmATTACH_POSITION,
 		XmNbottomAttachment, XmATTACH_POSITION,
-		XmNleftPosition, LEFT_ABOUT,
-		XmNrightPosition, RIGHT_ABOUT,
-		XmNtopPosition, TOP_ABOUT,
-		XmNbottomPosition, BOTTOM_ABOUT,
+		XmNleftPosition, LEFT_WEB,
+		XmNrightPosition, RIGHT_WEB,
+		XmNtopPosition, TOP_WEB,
+		XmNbottomPosition, BOTTOM_WEB,
 		NULL
 	);
 
@@ -1536,7 +1741,98 @@ force_dbprog_geometry(widgets_t *m)
 
 
 /*
- * force_extd_geometry
+ * geom_wwwwarp_force
+ *	Set the geometry of the widgets in the wwwWarp window.
+ *
+ * Args:
+ *	m - Pointer to the main widgets structure.
+ *
+ * Return:
+ *	Nothing.
+ */
+STATIC void
+geom_wwwwarp_force(widgets_t *m)
+{
+	XtVaSetValues(m->wwwwarp.wwwwarp_lbl,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_WLPIXMAP,
+		XmNrightPosition, RIGHT_WLPIXMAP,
+		XmNtopPosition, TOP_WLPIXMAP,
+		NULL
+	);
+
+	XtVaSetValues(m->wwwwarp.sel_lbl,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_WLSELLBL,
+		XmNrightPosition, RIGHT_WLSELLBL,
+		XmNtopPosition, TOP_WLSELLBL,
+		NULL
+	);
+
+	XtVaSetValues(m->wwwwarp.sel_radbox,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_WLRADBOX,
+		XmNrightPosition, RIGHT_WLRADBOX,
+		XmNtopPosition, TOP_WLRADBOX,
+		NULL
+	);
+
+	XtVaSetValues(m->wwwwarp.srchsite_opt,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_NONE,
+		XmNtopAttachment, XmATTACH_WIDGET,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_WLSRCHSITES,
+		XmNtopWidget, m->wwwwarp.sel_radbox,
+		NULL
+	);
+
+	XtVaSetValues(m->wwwwarp.wwwwarp_sep,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_WLSEP,
+		XmNrightPosition, RIGHT_WLSEP,
+		XmNtopPosition, TOP_WLSEP,
+		NULL
+	);
+
+	XtVaSetValues(m->wwwwarp.go_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_WLGO,
+		XmNrightPosition, RIGHT_WLGO,
+		XmNtopPosition, TOP_WLGO,
+		NULL
+	);
+
+	XtVaSetValues(m->wwwwarp.cancel_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_WLCANCEL,
+		XmNrightPosition, RIGHT_WLCANCEL,
+		XmNtopPosition, TOP_WLCANCEL,
+		NULL
+	);
+}
+
+
+/*
+ * geom_extd_force
  *	Set the geometry of the widgets in the extended disc info
  *	window.
  *
@@ -1547,10 +1843,20 @@ force_dbprog_geometry(widgets_t *m)
  *	Nothing.
  */
 STATIC void
-force_extd_geometry(widgets_t *m)
+geom_extd_force(widgets_t *m)
 {
-	/* Extended Disc Info popup window widgets */
-
+	XtVaSetValues(m->dbextd.discno_lbl,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNleftPosition, LEFT_DISCNUMLBL,
+		XmNrightPosition, RIGHT_DISCNUMLBL,
+		XmNtopPosition, TOP_DISCNUMLBL,
+		XmNbottomPosition, BOTTOM_DISCNUMLBL,
+		NULL
+	);
+	
 	XtVaSetValues(m->dbextd.disc_lbl,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
@@ -1622,7 +1928,7 @@ force_extd_geometry(widgets_t *m)
 
 
 /*
- * force_extt_geometry
+ * geom_extt_force
  *	Set the geometry of the widgets in the extended track info
  *	window.
  *
@@ -1633,9 +1939,43 @@ force_extd_geometry(widgets_t *m)
  *	Nothing.
  */
 STATIC void
-force_extt_geometry(widgets_t *m)
+geom_extt_force(widgets_t *m)
 {
-	/* Extended Track Info popup window widgets */
+	XtVaSetValues(m->dbextt.prev_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNleftPosition, LEFT_EXTTPREV,
+		XmNrightPosition, RIGHT_EXTTPREV,
+		XmNtopPosition, TOP_EXTTPREV,
+		XmNbottomPosition, BOTTOM_EXTTPREV,
+		NULL
+	);
+
+	XtVaSetValues(m->dbextt.next_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNleftPosition, LEFT_EXTTNEXT,
+		XmNrightPosition, RIGHT_EXTTNEXT,
+		XmNtopPosition, TOP_EXTTNEXT,
+		XmNbottomPosition, BOTTOM_EXTTNEXT,
+		NULL
+	);
+	
+	XtVaSetValues(m->dbextt.trkno_lbl,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNleftPosition, LEFT_EXTTTRKNUMLBL,
+		XmNrightPosition, RIGHT_EXTTTRKNUMLBL,
+		XmNtopPosition, TOP_EXTTTRKNUMLBL,
+		XmNbottomPosition, BOTTOM_EXTTTRKNUMLBL,
+		NULL
+	);
 
 	XtVaSetValues(m->dbextt.trk_lbl,
 		XmNleftAttachment, XmATTACH_POSITION,
@@ -1648,7 +1988,7 @@ force_extt_geometry(widgets_t *m)
 		XmNbottomPosition, BOTTOM_TRKLBL,
 		NULL
 	);
-	
+
 	XtVaSetValues(XtParent(m->dbextt.trk_txt),
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
@@ -1708,7 +2048,7 @@ force_extt_geometry(widgets_t *m)
 
 
 /*
- * force_help_geometry
+ * geom_help_force
  *	Set the geometry of the widgets in the help display window.
  *
  * Args:
@@ -1718,8 +2058,18 @@ force_extt_geometry(widgets_t *m)
  *	Nothing.
  */
 STATIC void
-force_help_geometry(widgets_t *m)
+geom_help_force(widgets_t *m)
 {
+	XtVaSetValues(m->help.topic_opt,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_NONE,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_TOPIC,
+		XmNtopPosition, TOP_TOPIC,
+		NULL
+	);
+
 	XtVaSetValues(XtParent(m->help.help_txt),
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
@@ -1743,21 +2093,32 @@ force_help_geometry(widgets_t *m)
 		NULL
 	);
 	
-	XtVaSetValues(m->help.ok_btn,
+	XtVaSetValues(m->help.about_btn,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
 		XmNtopAttachment, XmATTACH_POSITION,
 		XmNbottomAttachment, XmATTACH_NONE,
-		XmNleftPosition, LEFT_HELPOK,
-		XmNrightPosition, RIGHT_HELPOK,
-		XmNtopPosition, TOP_HELPOK,
+		XmNleftPosition, LEFT_ABOUT,
+		XmNrightPosition, RIGHT_ABOUT,
+		XmNtopPosition, TOP_ABOUT,
+		NULL
+	);
+
+	XtVaSetValues(m->help.cancel_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_HELPCANCEL,
+		XmNrightPosition, RIGHT_HELPCANCEL,
+		XmNtopPosition, TOP_HELPCANCEL,
 		NULL
 	);
 }
 
 
 /*
- * force_dirsel_geometry
+ * geom_dirsel_force
  *	Set the geometry of the widgets in the CD database directory
  *	selector window.
  *
@@ -1768,10 +2129,8 @@ force_help_geometry(widgets_t *m)
  *	Nothing.
  */
 STATIC void
-force_dirsel_geometry(widgets_t *m)
+geom_dirsel_force(widgets_t *m)
 {
-	/* Directory selector popup widgets */
-
 	XtVaSetValues(m->dirsel.dir_lbl,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
@@ -1832,7 +2191,7 @@ force_dirsel_geometry(widgets_t *m)
 
 
 /*
- * force_linksel_geometry
+ * geom_linksel_force
  *	Set the geometry of the widgets in the search-link selector
  *	list window.
  *
@@ -1843,10 +2202,8 @@ force_dirsel_geometry(widgets_t *m)
  *	Nothing.
  */
 STATIC void
-force_linksel_geometry(widgets_t *m)
+geom_linksel_force(widgets_t *m)
 {
-	/* Link selector popup widgets */
-
 	XtVaSetValues(m->linksel.link_lbl,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
@@ -1907,7 +2264,7 @@ force_linksel_geometry(widgets_t *m)
 
 
 /*
- * force_auth_geometry
+ * geom_auth_force
  *	Set the geometry of the widgets in the authorization window.
  *
  * Args:
@@ -1917,7 +2274,7 @@ force_linksel_geometry(widgets_t *m)
  *	Nothing.
  */
 STATIC void
-force_auth_geometry(widgets_t *m)
+geom_auth_force(widgets_t *m)
 {
 	XtVaSetValues(m->auth.auth_lbl,
 		XmNleftAttachment, XmATTACH_POSITION,
@@ -2009,23 +2366,6 @@ force_auth_geometry(widgets_t *m)
 }
 
 
-/*
- * force_dialog_geometry
- *	Set the geometry of the widgets in the dialog box windows.
- *
- * Args:
- *	m - Pointer to the main widgets structure.
- *
- * Return:
- *	Nothing.
- */
-/*ARGSUSED*/
-STATIC void
-force_dialog_geometry(widgets_t *m)
-{
-	/* For future expansion */
-}
-
 
 /***********************
  *   public routines   *
@@ -2033,7 +2373,7 @@ force_dialog_geometry(widgets_t *m)
 
 
 /*
- * force_geometry
+ * geom_force
  *	Top level function to set the geometry of the widgets in each
  *	main and sub-window.
  *
@@ -2044,19 +2384,59 @@ force_dialog_geometry(widgets_t *m)
  *	Nothing.
  */
 void
-force_geometry(widgets_t *m)
+geom_force(widgets_t *m)
 {
-	force_main_geometry(m);
-	force_keypad_geometry(m);
-	force_options_geometry(m);
-	force_dbprog_geometry(m);
-	force_extd_geometry(m);
-	force_extt_geometry(m);
-	force_help_geometry(m);
-	force_dirsel_geometry(m);
-	force_linksel_geometry(m);
-	force_auth_geometry(m);
-	force_dialog_geometry(m);
+	main_mode = app_data.main_mode;
+
+	geom_main_force(m);
+	geom_keypad_force(m);
+	geom_options_force(m);
+	geom_dbprog_force(m);
+	geom_wwwwarp_force(m);
+	geom_extd_force(m);
+	geom_extt_force(m);
+	geom_help_force(m);
+	geom_dirsel_force(m);
+	geom_linksel_force(m);
+	geom_auth_force(m);
 }
 
 
+/*
+ * geom_main_chgmode
+ *	Change the main window mode
+ *
+ * Args:
+ *	m - Pointer to the main widgets structure.
+ *
+ * Return:
+ *	Nothing.
+ */
+void
+geom_main_chgmode(widgets_t *m)
+{
+	if (main_mode == MAIN_NORMAL)
+		main_mode = MAIN_BASIC;
+	else
+		main_mode = MAIN_NORMAL;
+
+	geom_main_force(m);
+}
+
+
+/*
+ * geom_main_getmode
+ *	Return the current main window mode
+ *
+ * Args:
+ *	m - Pointer to the main widgets structure.
+ *
+ * Return:
+ *	Nothing.
+ */
+int
+geom_main_getmode(void)
+{
+	return (main_mode);
+}
+
