@@ -1,7 +1,7 @@
 /*
  *   xmcd - Motif(tm) CD Audio Player
  *
- *   Copyright (C) 1993-1997  Ti Kan
+ *   Copyright (C) 1993-1998  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -20,12 +20,12 @@
  *
  */
 #ifndef LINT
-static char *_help_c_ident_ = "@(#)help.c	6.21 97/07/24";
+static char *_help_c_ident_ = "@(#)help.c	6.41 98/05/22";
 #endif
 
-#define _XMINC_
-
 #include "common_d/appenv.h"
+#include "common_d/util.h"
+#include "common_d/patchlevel.h"
 #include "xmcd_d/xmcd.h"
 #include "xmcd_d/widget.h"
 #include "xmcd_d/cdfunc.h"
@@ -46,8 +46,11 @@ typedef struct {
 
 extern appdata_t	app_data;
 extern widgets_t	widgets;
+extern FILE		*errfp;
 
 STATIC wname_t		wname[MAX_HELP_WIDGETS];
+STATIC doc_topic_t	*dochead,
+			*doctail;
 
 
 /***********************
@@ -69,23 +72,18 @@ help_mapinit(void)
 {
 	int	i = 0;
 
-#if (XmVersion >= 1002)
-	wname[i].widget = widgets.main.btnlbl_btn;
-	wname[i++].hlpname = "Main.cbx";
+	wname[i].widget = widgets.main.mode_btn;
+	wname[i++].hlpname = "Mode.btn";
 	wname[i].widget = widgets.main.lock_btn;
 	wname[i++].hlpname = "Main.cbx";
 	wname[i].widget = widgets.main.repeat_btn;
 	wname[i++].hlpname = "Main.cbx";
 	wname[i].widget = widgets.main.shuffle_btn;
 	wname[i++].hlpname = "Main.cbx";
-#else
-	wname[i].widget = widgets.main.check_box;
-	wname[i++].hlpname = "Main.cbx";
-#endif
 	wname[i].widget = widgets.main.eject_btn;
 	wname[i++].hlpname = "Eject.btn";
-	wname[i].widget = widgets.main.poweroff_btn;
-	wname[i++].hlpname = "Power.btn";
+	wname[i].widget = widgets.main.quit_btn;
+	wname[i++].hlpname = "Quit.btn";
 	wname[i].widget = widgets.main.dbprog_btn;
 	wname[i++].hlpname = "DbProg.btn";
 	wname[i].widget = widgets.main.options_btn;
@@ -211,11 +209,11 @@ help_mapinit(void)
 	wname[i].widget = widgets.options.ok_btn;
 	wname[i++].hlpname = "OpOk.btn";
 	wname[i].widget = widgets.dbprog.tottime_lbl;
-	wname[i++].hlpname = "DpTotTim.ind";
+	wname[i++].hlpname = "DpTotTim.lbl";
 	wname[i].widget = widgets.dbprog.tottime_ind;
-	wname[i++].hlpname = "DpTotTim.ind";
-	wname[i].widget = widgets.dbprog.about_btn;
-	wname[i++].hlpname = "DpAbout.btn";
+	wname[i++].hlpname = "DpTotTim.lbl";
+	wname[i].widget = widgets.dbprog.wwwwarp_btn;
+	wname[i++].hlpname = "DpWeb.btn";
 	wname[i].widget = widgets.dbprog.dtitle_txt;
 	wname[i++].hlpname = "DpDTitle.txw";
 	wname[i].widget = widgets.dbprog.extd_btn;
@@ -246,6 +244,14 @@ help_mapinit(void)
 	wname[i++].hlpname = "DpLoad.btn";
 	wname[i].widget = widgets.dbprog.cancel_btn;
 	wname[i++].hlpname = "DpCancel.btn";
+	wname[i].widget = widgets.wwwwarp.sel_radbox;
+	wname[i++].hlpname = "WwSelect.rbx";
+	wname[i].widget = widgets.wwwwarp.srchsite_opt;
+	wname[i++].hlpname = "WwSearch.opt";
+	wname[i].widget = widgets.wwwwarp.go_btn;
+	wname[i++].hlpname = "WwGo.btn";
+	wname[i].widget = widgets.wwwwarp.cancel_btn;
+	wname[i++].hlpname = "WwCancel.btn";
 	wname[i].widget = widgets.dbextd.disc_txt;
 	wname[i++].hlpname = "DdDisc.txw";
 	wname[i].widget = widgets.dbextd.ok_btn;
@@ -254,6 +260,10 @@ help_mapinit(void)
 	wname[i++].hlpname = "DdClr.btn";
 	wname[i].widget = widgets.dbextd.cancel_btn;
 	wname[i++].hlpname = "DdCancel.btn";
+	wname[i].widget = widgets.dbextt.prev_btn;
+	wname[i++].hlpname = "DtDir.btn";
+	wname[i].widget = widgets.dbextt.next_btn;
+	wname[i++].hlpname = "DtDir.btn";
 	wname[i].widget = widgets.dbextt.trk_txt;
 	wname[i++].hlpname = "DtTrack.txw";
 	wname[i].widget = widgets.dbextt.ok_btn;
@@ -274,11 +284,113 @@ help_mapinit(void)
 	wname[i++].hlpname = "LsOk.btn";
 	wname[i].widget = widgets.linksel.cancel_btn;
 	wname[i++].hlpname = "LsCancel.btn";
+	wname[i].widget = widgets.help.topic_opt;
+	wname[i++].hlpname = "HpTopic.opt";
+	wname[i].widget = widgets.help.help_txt;
+	wname[i++].hlpname = "HpText.txw";
+	wname[i].widget = widgets.help.about_btn;
+	wname[i++].hlpname = "HpAbout.btn";
+	wname[i].widget = widgets.help.cancel_btn;
+	wname[i++].hlpname = "HpCancel.btn";
 	wname[i].widget = (Widget) NULL;
 	wname[i].hlpname = NULL;
 }
 
 
+/*
+ * help_docinit
+ *	Initialize the documentation topics list.
+ *
+ * Args:
+ *	Nothing.
+ *
+ * Return:
+ *	Nothing.
+ */
+STATIC void
+help_docinit(void)
+{
+	DIR		*dp;
+	struct dirent	*de;
+	doc_topic_t	*p;
+	struct stat	stbuf;
+	char		docdir[FILE_PATH_SZ],
+			fname[FILE_PATH_SZ];
+
+	if (app_data.libdir == NULL)
+		/* shrug */
+		return;
+
+	(void) sprintf(docdir, DOCFILE_PATH, app_data.libdir);
+	if ((dp = OPENDIR(docdir)) == NULL) {
+		DBGPRN(errfp, "Cannot open %s, no topics added.\n", docdir);
+		return;
+	}
+
+	while ((de = READDIR(dp)) != NULL) {
+		/* Skip ".", ".." and dot files */
+		if (de->d_name[0] == '.')
+			continue;
+
+		(void) sprintf(fname, "%s%s", docdir, de->d_name);
+		if (stat(fname, &stbuf) < 0)
+			continue;
+
+		/* Skip non-regular files */
+		if (!S_ISREG(stbuf.st_mode))
+			continue;
+
+		p = (doc_topic_t *) MEM_ALLOC(sizeof(doc_topic_t));
+		if (p == NULL) {
+			(void) CLOSEDIR(dp);
+			CD_FATAL(app_data.str_nomemory);
+			return;
+		}
+		p->next = NULL;
+		p->name = (char *) MEM_ALLOC(strlen(de->d_name) + 1);
+		if (p->name == NULL) {
+			(void) CLOSEDIR(dp);
+			CD_FATAL(app_data.str_nomemory);
+			return;
+		}
+		p->path = (char *) MEM_ALLOC(strlen(fname) + 1);
+		if (p->path == NULL) {
+			(void) CLOSEDIR(dp);
+			CD_FATAL(app_data.str_nomemory);
+			return;
+		}
+		(void) strcpy(p->name, de->d_name);
+		(void) strcpy(p->path, fname);
+
+		if (dochead == NULL)
+			dochead = doctail = p;
+		else if (strcmp(p->name, "README") == 0) {
+			/* Put README file on top */
+			p->next = dochead;
+			dochead = p;
+		}
+		else {
+			doctail->next = p;
+			doctail = p;
+		}
+	}
+
+	for (p = dochead; p != NULL; p = p->next) {
+		p->actbtn = XmCreatePushButton(
+			widgets.help.topic_menu,
+			p->name,
+			NULL,
+			0
+		);
+		XtManageChild(p->actbtn);
+
+		register_activate_cb(p->actbtn, help_topic_sel, p);
+	}
+
+	(void) CLOSEDIR(dp);
+}
+
+
 /*
  * help_getname
  *	Given a widget, return the associated help file name.
@@ -302,13 +414,202 @@ help_getname(Widget w)
 }
 
 
+/*
+ * help_loadfile
+ *	Pop up the help window and display text from the specified file.
+ *
+ * Args:
+ *	path - The file path
+ *
+ * Return:
+ *	Nothing.
+ */
+STATIC void
+help_loadfile(char *path)
+{
+	int		bufsz = STR_BUF_SZ * 2;
+	char		*tmpbuf,
+			*helptext;
+	FILE		*fp;
+#ifndef __VMS
+	pid_t		cpid;
+	waitret_t	stat_val;
+	int		ret,
+			pfd[2];
+	FILE		*wfp;
+
+	if (PIPE(pfd) < 0) {
+		DBGPRN(errfp, "help_loadfile: pipe failed (errno=%d)\n",
+			errno);
+		cd_beep();
+		return;
+	}
+
+	switch (cpid = FORK()) {
+	case 0:
+		/* Child */
+
+		/* Close un-needed pipe descriptor */
+		(void) close(pfd[0]);
+
+		DBGPRN(errfp, "Setting uid to %d, gid to %d\n",
+			(int) util_get_ouid(), (int) util_get_ogid());
+
+		/* Force uid and gid to original setting */
+		if (setuid(util_get_ouid()) < 0 ||
+		    setgid(util_get_ogid()) < 0) {
+			(void) close(pfd[1]);
+			exit(1);
+		}
+
+		DBGPRN(errfp, "Help: loading %s\n", path);
+
+		if ((fp = fopen(path, "r")) == NULL)
+			DBGPRN(errfp, "Cannot open %s\n", path);
+
+		if ((wfp = fdopen(pfd[1], "w")) == NULL) {
+			DBGPRN(errfp,
+				"help_loadfile: write pipe fdopen failed\n");
+			(void) close(pfd[1]);
+			exit(4);
+		}
+
+		if (fp == NULL) {
+			(void) fprintf(wfp, "%s", app_data.str_nohelp);
+			(void) fclose(wfp);
+			exit(0);
+		}
+
+		/* Allocate temporary buffer */
+		if ((tmpbuf = (char *) MEM_ALLOC(bufsz)) == NULL) {
+			(void) close(pfd[1]);
+			exit(2);
+		}
+		
+		while (fgets(tmpbuf, bufsz, fp) != NULL)
+			(void) fprintf(wfp, "%s", tmpbuf);
+
+		(void) fclose(fp);
+		(void) fclose(wfp);
+
+		exit(0);
+		/*NOTREACHED*/
+
+	case -1:
+		DBGPRN(errfp, "help_loadfile: fork failed (errno=%d)\n",
+			errno);
+		(void) close(pfd[0]);
+		(void) close(pfd[1]);
+
+		cd_beep();
+		return;
+
+	default:
+		/* Parent */
+
+		/* Close un-needed pipe descriptor */
+		(void) close(pfd[1]);
+
+		if ((fp = fdopen(pfd[0], "r")) == NULL) {
+			DBGPRN(errfp,
+			    "help_loadfile: read pipe fdopen failed\n");
+			cd_beep();
+			return;
+		}
+		break;
+	}
+#else
+	DBGPRN(errfp, "Help: loading %s\n", path);
+
+	if ((fp = fopen(path, "r")) == NULL)
+		DBGPRN(errfp, "Cannot open %s\n", path);
+#endif	/* __VMS */
+
+	if (fp == NULL) {
+		helptext = (char *) MEM_ALLOC(strlen(app_data.str_nohelp) + 1);
+		if (helptext == NULL) {
+			CD_FATAL(app_data.str_nomemory);
+			return;
+		}
+		strcpy(helptext, app_data.str_nohelp);
+	}
+	else {
+		/* Allocate temporary buffer */
+		if ((tmpbuf = (char *) MEM_ALLOC(bufsz)) == NULL) {
+			CD_FATAL(app_data.str_nomemory);
+			return;
+		}
+
+		helptext = NULL;
+
+		while (fgets(tmpbuf, bufsz, fp) != NULL) {
+			if (tmpbuf[0] == '#')
+				/* Comment */
+				continue;
+
+			if (helptext == NULL) {
+				helptext = (char *) MEM_ALLOC(
+					strlen(tmpbuf) + 1
+				);
+
+				if (helptext != NULL)
+					*helptext = '\0';
+			}
+			else {
+				helptext = (char *) MEM_REALLOC(
+					helptext,
+					strlen(helptext) + strlen(tmpbuf) + 1
+				);
+			}
+
+			if (helptext == NULL) {
+				CD_FATAL(app_data.str_nomemory);
+				(void) fclose(fp);
+				MEM_FREE(tmpbuf);
+				return;
+			}
+
+			(void) strcat(helptext, tmpbuf);
+		}
+
+		(void) fclose(fp);
+		MEM_FREE(tmpbuf);
+	}
+
+#ifndef __VMS
+	/* Wait for child to finish */
+	while ((ret = WAITPID(cpid, &stat_val, 0)) != cpid) {
+		if (ret < 0)
+			break;
+
+		/* Service some events */
+		event_loop(0);
+	}
+#endif
+
+	/* Set new help text */
+	XmTextSetString(widgets.help.help_txt, helptext);
+	MEM_FREE(helptext);
+
+	/* Pop it up */
+	if (!XtIsManaged(widgets.help.form))
+		XtManageChild(widgets.help.form);
+
+	/* Set keyboard focus on the Cancel button */
+	XmProcessTraversal(
+		widgets.help.cancel_btn,
+		XmTRAVERSE_CURRENT
+	);
+}
+
+
 /***********************
  *   public routines   *
  ***********************/
 
 
 /*
- * help_setup
+ * help_init
  *	Top level function to set up the help subsystem.
  *
  * Args:
@@ -318,11 +619,14 @@ help_getname(Widget w)
  *	Nothing.
  */
 void
-help_setup(widgets_t *m)
+help_init(void)
 {
 	int		i;
 	XtTranslations	xtab1,
 			xtab2;
+	widgets_t	*m;
+
+	m = &widgets;
 
 	xtab1 = XtParseTranslationTable(
 		"<Btn3Down>,<Btn3Up>: Help()\n"
@@ -332,16 +636,12 @@ help_setup(widgets_t *m)
 	);
 
 	/* Translations for the main window widgets */
-#if (XmVersion >= 1002)
-	XtOverrideTranslations(m->main.btnlbl_btn, xtab1);
+	XtOverrideTranslations(m->main.mode_btn, xtab1);
 	XtOverrideTranslations(m->main.lock_btn, xtab1);
 	XtOverrideTranslations(m->main.repeat_btn, xtab1);
 	XtOverrideTranslations(m->main.shuffle_btn, xtab1);
-#else
-	XtOverrideTranslations(m->main.check_box, xtab1);
-#endif
 	XtOverrideTranslations(m->main.eject_btn, xtab1);
-	XtOverrideTranslations(m->main.poweroff_btn, xtab1);
+	XtOverrideTranslations(m->main.quit_btn, xtab1);
 	XtOverrideTranslations(m->main.disc_ind, xtab1);
 	XtOverrideTranslations(m->main.track_ind, xtab1);
 	XtOverrideTranslations(m->main.index_ind, xtab1);
@@ -404,7 +704,7 @@ help_setup(widgets_t *m)
 	/* Translations for the dbprog window widgets */
 	XtOverrideTranslations(m->dbprog.tottime_lbl, xtab1);
 	XtOverrideTranslations(m->dbprog.tottime_ind, xtab1);
-	XtOverrideTranslations(m->dbprog.about_btn, xtab1);
+	XtOverrideTranslations(m->dbprog.wwwwarp_btn, xtab1);
 	XtOverrideTranslations(m->dbprog.dtitle_txt, xtab1);
 	XtOverrideTranslations(m->dbprog.extd_btn, xtab1);
 	XtOverrideTranslations(m->dbprog.trk_list, xtab2);
@@ -421,6 +721,12 @@ help_setup(widgets_t *m)
 	XtOverrideTranslations(m->dbprog.loaddb_btn, xtab1);
 	XtOverrideTranslations(m->dbprog.cancel_btn, xtab1);
 
+	/* Translations for the wwwWarp window widgets */
+	XtOverrideTranslations(m->wwwwarp.sel_radbox, xtab1);
+	XtOverrideTranslations(m->wwwwarp.srchsite_opt, xtab1);
+	XtOverrideTranslations(m->wwwwarp.go_btn, xtab1);
+	XtOverrideTranslations(m->wwwwarp.cancel_btn, xtab1);
+
 	/* Translations for the extd window widgets */
 	XtOverrideTranslations(m->dbextd.disc_txt, xtab1);
 	XtOverrideTranslations(m->dbextd.ok_btn, xtab1);
@@ -428,6 +734,8 @@ help_setup(widgets_t *m)
 	XtOverrideTranslations(m->dbextd.cancel_btn, xtab1);
 
 	/* Translations for the extt window widgets */
+	XtOverrideTranslations(m->dbextt.prev_btn, xtab1);
+	XtOverrideTranslations(m->dbextt.next_btn, xtab1);
 	XtOverrideTranslations(m->dbextt.trk_txt, xtab1);
 	XtOverrideTranslations(m->dbextt.ok_btn, xtab1);
 	XtOverrideTranslations(m->dbextt.clear_btn, xtab1);
@@ -443,14 +751,101 @@ help_setup(widgets_t *m)
 	XtOverrideTranslations(m->linksel.ok_btn, xtab1);
 	XtOverrideTranslations(m->linksel.cancel_btn, xtab1);
 
+	/* Translations for the help window widgets */
+	XtOverrideTranslations(m->help.topic_opt, xtab1);
+	XtOverrideTranslations(m->help.help_txt, xtab1);
+	XtOverrideTranslations(m->help.about_btn, xtab1);
+	XtOverrideTranslations(m->help.cancel_btn, xtab1);
+
 	/* Initialize helpfile mappings */
 	help_mapinit();
+
+	/* Initialize documentation topics list */
+	help_docinit();
+}
+
+
+/*
+ * help_start
+ *	Start up xmcd help system.
+ *
+ * Args:
+ *	None.
+ *
+ * Return:
+ *	Nothing.
+ */
+void
+help_start(void)
+{
+	int		ret;
+	pid_t		cpid;
+	waitret_t	stat_val;
+	char		*dirpath,
+			vfile[FILE_PATH_SZ],
+			path[FILE_PATH_SZ + 12];
+	struct stat	stbuf;
+
+	/* Popup the help window if this version of xmcd is being run
+	 * by the user for the first time.
+	 */
+	(void) sprintf(vfile, "%s-%s", PROGNAME, VERSION);
+	(void) sprintf(path, USR_VINIT_PATH,
+			util_homedir(util_get_ouid()), vfile);
+
+	if (stat(path, &stbuf) < 0) {
+		switch (cpid = FORK()) {
+		case 0:
+			/* Child process */
+			DBGPRN(errfp, "\nSetting uid to %d, gid to %d\n",
+				(int) util_get_ouid(), (int) util_get_ogid());
+
+			/* Force uid and gid to original setting */
+			if (setuid(util_get_ouid()) < 0 ||
+			    setgid(util_get_ogid()) < 0)
+				exit(1);
+
+			dirpath = util_dirname(path);
+			if (!util_mkdir(dirpath, 0755)) {
+				DBGPRN(errfp, "help_setup: cannot mkdir %s.\n",
+					dirpath);
+				exit(2);
+			}
+
+			/* Create version file */
+			if (creat(path, 0666) < 0) {
+				DBGPRN(errfp, "help_setup: cannot creat %s.\n",
+					path);
+				exit(3);
+			}
+			exit(0);
+			/*NOTREACHED*/
+
+		case -1:
+			/* fork failed */
+			break;
+
+		default:
+			/* Parent: wait for child to finish */
+			while ((ret = WAITPID(cpid, &stat_val, 0)) != cpid) {
+				if (ret < 0)
+					break;
+				/* Handle some events */
+				event_loop(0);
+			}
+			break;
+		}
+
+		/* Pop up help */
+		help_popup(widgets.main.help_btn);
+	}
 }
 
 
 /*
  * help_popup
- *	Pop up the help window and display appropriate help text.
+ *	Pop up the help window and display help text based on the
+ *	specified widget.
  *
  * Args:
  *	w - The widget which the help info is being displayed about.
@@ -461,69 +856,85 @@ help_setup(widgets_t *m)
 void
 help_popup(Widget w)
 {
-	int		bufsz = STR_BUF_SZ * 2;
-	char		hlpfile[FILE_PATH_SZ * 2],
-			*tmpbuf,
-			*hlpname;
-	FILE		*fp;
-	static char	*helptext = NULL;
-
-	if ((tmpbuf = (char *) MEM_ALLOC(bufsz)) == NULL) {
-		CD_FATAL(app_data.str_nomemory);
-		return;
-	}
+	char	hlpfile[FILE_PATH_SZ * 2],
+		*hlpname;
 
 	if ((hlpname = help_getname(w)) == NULL)
 		return;
 
 	(void) sprintf(hlpfile, HELPFILE_PATH, app_data.libdir, hlpname);
 
-	if ((fp = fopen(hlpfile, "r")) == NULL) {
-		/* Can't read help file on this widget */
-		XmTextSetString(widgets.help.help_txt, app_data.str_nohelp);
-		XtManageChild(widgets.help.form);
-		MEM_FREE(tmpbuf);
-		return;
-	}
+	XtVaSetValues(widgets.help.topic_opt,
+		XmNmenuHistory, widgets.help.online_btn,
+		NULL
+	);
 
-	if (helptext != NULL) {
-		MEM_FREE(helptext);
-		helptext = NULL;
-	}
+	/* Change to watch cursor */
+	cd_busycurs(TRUE, CURS_HELP);
 
-	while (fgets(tmpbuf, bufsz, fp) != NULL) {
-		if (tmpbuf[0] == '#')
-			/* Comment */
-			continue;
+	/* Load the help file */
+	help_loadfile(hlpfile);
 
-		if (helptext == NULL) {
-			helptext = (char *) MEM_ALLOC(strlen(tmpbuf) + 1);
-
-			if (helptext != NULL)
-				*helptext = '\0';
-		}
-		else {
-			helptext = (char *) MEM_REALLOC(
-				helptext,
-				strlen(helptext) + strlen(tmpbuf) + 1
-			);
-		}
-
-		if (helptext == NULL) {
-			CD_FATAL(app_data.str_nomemory);
-			(void) fclose(fp);
-			MEM_FREE(tmpbuf);
-			return;
-		}
-
-		(void) strcat(helptext, tmpbuf);
-	}
+	/* Change to normal cursor */
+	cd_busycurs(FALSE, CURS_HELP);
+}
+
+
+/*
+ * help_popdown
+ *	Pop down the help window.
+ *
+ * Args:
+ *	None
+ *
+ * Return:
+ *	Nothing.
+ */
+void
+help_popdown(void)
+{
+	if (XtIsManaged(widgets.help.form))
+		XtUnmanageChild(widgets.help.form);
+}
+
+
+/* help_isactive
+ *	Check if the help window is currently popped up.
+ *
+ * Args:
+ *	None
+ *
+ * Return:
+ *	TRUE: help currently popped up
+ *	FALSE: help not currently popped up
+ */
+bool_t
+help_isactive(void)
+{
+	return ((bool_t) XtIsManaged(widgets.help.form));
+}
+
+
+/**************** vv Callback routines vv ****************/
 
-	(void) fclose(fp);
-	MEM_FREE(tmpbuf);
+/*
+ * help_topic_sel
+ *	Help topic selector callback
+ */
+/*ARGSUSED*/
+void
+help_topic_sel(Widget w, XtPointer client_data, XtPointer call_data)
+{
+	doc_topic_t	*p = (doc_topic_t *)(void *) client_data;
 
-	XmTextSetString(widgets.help.help_txt, helptext);
-	XtManageChild(widgets.help.form);
+	if (p == NULL) {
+		/* Display generic help text */
+		help_popup(widgets.main.help_btn);
+	}
+	else if (p->actbtn == w) {
+		/* Display appropriate doc file */
+		help_loadfile(p->path);
+	}
 }
 
 
