@@ -1,7 +1,7 @@
 /*
  *   xmcd - Motif(tm) CD Audio Player
  *
- *   Copyright (C) 1993-1997  Ti Kan
+ *   Copyright (C) 1993-1998  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -23,14 +23,31 @@
 #define __HELP_H__
 
 #ifndef LINT
-static char *_help_h_ident_ = "@(#)help.h	6.6 97/07/24";
+static char *_help_h_ident_ = "@(#)help.h	6.12 98/05/19";
 #endif
 
 /* Max number of widgets supported by the help system */
-#define MAX_HELP_WIDGETS	103
+#define MAX_HELP_WIDGETS	112
+
+
+/* Documentation topics list structure */
+typedef struct doc_topic {
+	char		*name;
+	char		*path;
+	Widget		actbtn;
+	struct doc_topic *next;
+} doc_topic_t;
+
 
 /* Public functions */
-extern void	help_setup(widgets_t *);
+extern void	help_init(void);
+extern void	help_start(void);
 extern void	help_popup(Widget);
+extern void	help_popdown(void);
+extern bool_t	help_isactive(void);
+
+
+/* Callback functions */
+extern void	help_topic_sel(Widget, XtPointer, XtPointer);
 
 #endif	/* __HELP_H__ */
