@@ -1,7 +1,7 @@
 /*
  *   xmcd - Motif(tm) CD Audio Player
  *
- *   Copyright (C) 1993-1997  Ti Kan
+ *   Copyright (C) 1993-1998  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -23,15 +23,16 @@
 #define __HOTKEY_H__
 
 #ifndef LINT
-static char *_hotkey_h_ident_ = "@(#)hotkey.h	6.3 96/12/22";
+static char *_hotkey_h_ident_ = "@(#)hotkey.h	6.6 98/05/14";
 #endif
 
 #define MAX_TRANSLATIONS_SZ	4096
 
 /* Public functions */
-extern void	hotkey_setup(widgets_t *);
+extern void	hotkey_init(void);
 extern void	hotkey_grabkeys(Widget);
 extern void	hotkey_ungrabkeys(Widget);
+extern void	hotkey_tooltip_mnemonic(Widget);
 extern void	hotkey(Widget, XEvent *, String *, Cardinal *);
 
 #endif	/* __HOTKEY_H__ */
