@@ -1,7 +1,7 @@
 /*
  *   xmcd - Motif(tm) CD Audio Player
  *
- *   Copyright (C) 1993-1997  Ti Kan
+ *   Copyright (C) 1993-1998  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
 #define __RESOURCE_H__
 
 #ifndef LINT
-static char *_resource_h_ident_ = "@(#)resource.h	6.40 97/07/24";
+static char *_resource_h_ident_ = "@(#)resource.h	6.63 98/05/03";
 #endif
 
 
@@ -31,6 +31,26 @@ static char *_resource_h_ident_ = "@(#)resource.h	6.40 97/07/24";
 #define XmcdNlibdir			"libdir"
 #define XmcdCLibdir			"Libdir"
 
+/* X resources */
+#define XmcdNversion			"version"
+#define XmcdCVersion			"Version"
+#define XmcdNmainWindowMode		"mainWindowMode"
+#define XmcdCMainWindowMode		"MainWindowMode"
+#define XmcdNnormalMainWidth		"normalMainWidth"
+#define XmcdCNormalMainWidth		"NormalMainWidth"
+#define XmcdNnormalMainHeight		"normalMainHeight"
+#define XmcdCNormalMainHeight		"NormalMainHeight"
+#define XmcdNbasicMainWidth		"basicMainWidth"
+#define XmcdCBasicMainWidth		"BasicMainWidth"
+#define XmcdNbasicMainHeight		"basicMainHeight"
+#define XmcdCBasicMainHeight		"BasicMainHeight"
+#define XmcdNdisplayBlinkOnInterval	"displayBlinkOnInterval"
+#define XmcdCDisplayBlinkOnInterval	"DisplayBlinkOnInterval"
+#define XmcdNdisplayBlinkOffInterval	"displayBlinkOffInterval"
+#define XmcdCDisplayBlinkOffInterval	"DisplayBlinkOffInterval"
+#define XmcdNmainShowFocus		"mainShowFocus"
+#define XmcdCMainShowFocus		"MainShowFocus"
+
 /* Common config parameters */
 #define XmcdNdevice			"device"
 #define XmcdCDevice			"Device"
@@ -56,24 +76,26 @@ static char *_resource_h_ident_ = "@(#)resource.h	6.40 97/07/24";
 #define XmcdCPreviousThreshold		"PreviousThreshold"
 #define XmcdNsampleBlocks		"sampleBlocks"
 #define XmcdCSampleBlocks		"SampleBlocks"
-#define XmcdNdisplayBlinkOnInterval	"displayBlinkOnInterval"
-#define XmcdCDisplayBlinkOnInterval	"DisplayBlinkOnInterval"
-#define XmcdNdisplayBlinkOffInterval	"displayBlinkOffInterval"
-#define XmcdCDisplayBlinkOffInterval	"DisplayBlinkOffInterval"
+#define XmcdNtimeDisplayMode		"timeDisplayMode"
+#define XmcdCTimeDisplayMode		"TimeDisplayMode"
+#define XmcdNtooltipDelayInterval	"tooltipDelayInterval"
+#define XmcdCTooltipDelayInterval	"TooltipDelayInterval"
+#define XmcdNtooltipActiveInterval	"tooltipActiveInterval"
+#define XmcdCTooltipActiveInterval	"TooltipActiveInterval"
 #define XmcdNinsertPollDisable		"insertPollDisable"
 #define XmcdCInsertPollDisable		"InsertPollDisable"
 #define XmcdNcddbRemoteAutoSave		"cddbRemoteAutoSave"
 #define XmcdCCddbRemoteAutoSave		"CddbRemoteAutoSave"
 #define XmcdNcddbUseHttpProxy		"cddbUseHttpProxy"
 #define XmcdCCddbUseHttpProxy		"CddbUseHttpProxy"
-#define XmcdNmainShowFocus		"mainShowFocus"
-#define XmcdCMainShowFocus		"MainShowFocus"
 #define XmcdNshowScsiErrMsg		"showScsiErrMsg"
 #define XmcdCShowScsiErrMsg		"ShowScsiErrMsg"
 #define XmcdNsolaris2VolumeManager	"solaris2VolumeManager"
 #define XmcdCSolaris2VolumeManager	"Solaris2VolumeManager"
 #define XmcdNcurfileEnable		"curfileEnable"
 #define XmcdCCurfileEnable		"CurfileEnable"
+#define XmcdNtooltipEnable		"tooltipEnable"
+#define XmcdCTooltipEnable		"TooltipEnable"
 #define XmcdNdebugMode			"debugMode"
 #define XmcdCDebugMode			"DebugMode"
 
@@ -178,6 +200,8 @@ static char *_resource_h_ident_ = "@(#)resource.h	6.40 97/07/24";
 #define XmcdCProgModeMsg		"ProgModeMsg"
 #define XmcdNelapseMsg			"elapseMsg"
 #define XmcdCElapseMsg			"ElapseMsg"
+#define XmcdNelapseDiscMsg		"elapseDiscMsg"
+#define XmcdCElapseDiscMsg		"ElapseDiscMsg"
 #define XmcdNremainTrackMsg		"remainTrackMsg"
 #define XmcdCRemainTrackMsg		"RemainTrackMsg"
 #define XmcdNremainDiscMsg		"remainDiscMsg"
@@ -296,10 +320,16 @@ static char *_resource_h_ident_ = "@(#)resource.h	6.40 97/07/24";
 #define XmcdCProxyConfigErrMsg		"ProxyConfigErrMsg"
 #define XmcdNproxyAuthFailMsg		"proxyAuthFailMsg"
 #define XmcdCProxyAuthFailMsg		"ProxyAuthFailMsg"
+#define XmcdNbrowserStartMsg		"browserStartMsg"
+#define XmcdCBrowserStartMsg		"BrowserStartMsg"
+#define XmcdNbrowserFailMsg		"browserFailMsg"
+#define XmcdCBrowserFailMsg		"BrowserFailMsg"
+#define XmcdNappDefFileMsg		"appDefFileMsg"
+#define XmcdCAppDefFileMsg		"AppDefFileMsg"
 
 /* Short-cut key definitions */
-#define XmcdNbuttonLabelKey		"buttonLabelKey"
-#define XmcdCButtonLabelKey		"ButtonLabelKey"
+#define XmcdNmodeKey			"modeKey"
+#define XmcdCModeKey			"ModeKey"
 #define XmcdNlockKey			"lockKey"
 #define XmcdCLockKey			"LockKey"
 #define XmcdNrepeatKey			"repeatKey"
@@ -308,8 +338,8 @@ static char *_resource_h_ident_ = "@(#)resource.h	6.40 97/07/24";
 #define XmcdCShuffleKey			"ShuffleKey"
 #define XmcdNejectKey			"ejectKey"
 #define XmcdCEjectKey			"EjectKey"
-#define XmcdNpowerOffKey		"powerOffKey"
-#define XmcdCPowerOffKey		"PowerOffKey"
+#define XmcdNquitKey			"quitKey"
+#define XmcdCQuitKey			"QuitKey"
 #define XmcdNdbprogKey			"dbprogKey"
 #define XmcdCDbprogKey			"DbprogKey"
 #define XmcdNhelpKey			"helpKey"
@@ -382,6 +412,62 @@ STATIC XtResource	resources[] = {
 		(XtPointer) NULL,
 	},
 
+	/* X resources */
+	{
+		XmcdNversion, XmcdCVersion,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, version), XmRImmediate,
+		(XtPointer) NULL,
+	},
+	{
+		XmcdNmainWindowMode, XmcdCMainWindowMode,
+		XmRInt, sizeof(int),
+		XtOffsetOf(appdata_t, main_mode), XmRImmediate,
+		(XtPointer) 0,
+	},
+	{
+		XmcdNnormalMainWidth, XmcdCNormalMainWidth,
+		XmRInt, sizeof(int),
+		XtOffsetOf(appdata_t, normal_width), XmRImmediate,
+		(XtPointer) 360,
+	},
+	{
+		XmcdNnormalMainHeight, XmcdCNormalMainHeight,
+		XmRInt, sizeof(int),
+		XtOffsetOf(appdata_t, normal_height), XmRImmediate,
+		(XtPointer) 135,
+	},
+	{
+		XmcdNbasicMainWidth, XmcdCBasicMainWidth,
+		XmRInt, sizeof(int),
+		XtOffsetOf(appdata_t, basic_width), XmRImmediate,
+		(XtPointer) 195,
+	},
+	{
+		XmcdNbasicMainHeight, XmcdCBasicMainHeight,
+		XmRInt, sizeof(int),
+		XtOffsetOf(appdata_t, basic_height), XmRImmediate,
+		(XtPointer) 60,
+	},
+	{
+		XmcdNmainShowFocus, XmcdCMainShowFocus,
+		XmRBoolean, sizeof(Boolean),
+		XtOffsetOf(appdata_t, main_showfocus), XmRImmediate,
+		(XtPointer) True,
+	},
+	{
+		XmcdNdisplayBlinkOnInterval, XmcdCDisplayBlinkOnInterval,
+		XmRInt, sizeof(int),
+		XtOffsetOf(appdata_t, blinkon_interval), XmRImmediate,
+		(XtPointer) 850,
+	},
+	{
+		XmcdNdisplayBlinkOffInterval, XmcdCDisplayBlinkOffInterval,
+		XmRInt, sizeof(int),
+		XtOffsetOf(appdata_t, blinkoff_interval), XmRImmediate,
+		(XtPointer) 150,
+	},
+
 	/* Common config parameters */
 	{
 		XmcdNdevice, XmcdCDevice,
@@ -456,16 +542,22 @@ STATIC XtResource	resources[] = {
 		(XtPointer) 750,
 	},
 	{
-		XmcdNdisplayBlinkOnInterval, XmcdCDisplayBlinkOnInterval,
+		XmcdNtimeDisplayMode, XmcdCTimeDisplayMode,
 		XmRInt, sizeof(int),
-		XtOffsetOf(appdata_t, blinkon_interval), XmRImmediate,
-		(XtPointer) 850,
+		XtOffsetOf(appdata_t, timedpy_mode), XmRImmediate,
+		(XtPointer) 0,
 	},
 	{
-		XmcdNdisplayBlinkOffInterval, XmcdCDisplayBlinkOffInterval,
+		XmcdNtooltipDelayInterval, XmcdCTooltipDelayInterval,
 		XmRInt, sizeof(int),
-		XtOffsetOf(appdata_t, blinkoff_interval), XmRImmediate,
-		(XtPointer) 150,
+		XtOffsetOf(appdata_t, tooltip_delay), XmRImmediate,
+		(XtPointer) 1000,
+	},
+	{
+		XmcdNtooltipActiveInterval, XmcdCTooltipActiveInterval,
+		XmRInt, sizeof(int),
+		XtOffsetOf(appdata_t, tooltip_time), XmRImmediate,
+		(XtPointer) 3000,
 	},
 	{
 		XmcdNinsertPollDisable, XmcdCInsertPollDisable,
@@ -485,12 +577,6 @@ STATIC XtResource	resources[] = {
 		XtOffsetOf(appdata_t, http_proxy), XmRImmediate,
 		(XtPointer) False,
 	},
-	{
-		XmcdNmainShowFocus, XmcdCMainShowFocus,
-		XmRBoolean, sizeof(Boolean),
-		XtOffsetOf(appdata_t, main_showfocus), XmRImmediate,
-		(XtPointer) True,
-	},
 	{
 		XmcdNshowScsiErrMsg, XmcdCShowScsiErrMsg,
 		XmRBoolean, sizeof(Boolean),
@@ -509,6 +595,12 @@ STATIC XtResource	resources[] = {
 		XtOffsetOf(appdata_t, write_curfile), XmRImmediate,
 		(XtPointer) True,
 	},
+	{
+		XmcdNtooltipEnable, XmcdCTooltipEnable,
+		XmRBoolean, sizeof(Boolean),
+		XtOffsetOf(appdata_t, tooltip_enable), XmRImmediate,
+		(XtPointer) True,
+	},
 	{
 		XmcdNdebugMode, XmcdCDebugMode,
 		XmRBoolean, sizeof(Boolean),
@@ -677,7 +769,7 @@ STATIC XtResource	resources[] = {
 		XmcdNsearchPauseInterval, XmcdCSearchPauseInterval,
 		XmRInt, sizeof(int),
 		XtOffsetOf(appdata_t, skip_pause), XmRImmediate,
-		(XtPointer) 45,
+		(XtPointer) 80,
 	},
 	{
 		XmcdNsearchSpeedUpCount, XmcdCSearchSpeedUpCount,
@@ -813,6 +905,12 @@ STATIC XtResource	resources[] = {
 		XtOffsetOf(appdata_t, str_elapse), XmRImmediate,
 		(XtPointer) STR_ELAPSE,
 	},
+	{
+		XmcdNelapseDiscMsg, XmcdCElapseDiscMsg,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, str_elapsedisc), XmRImmediate,
+		(XtPointer) STR_ELAPSE_DISC,
+	},
 	{
 		XmcdNremainTrackMsg, XmcdCRemainTrackMsg,
 		XmRString, sizeof(String),
@@ -1167,12 +1265,30 @@ STATIC XtResource	resources[] = {
 		XtOffsetOf(appdata_t, str_authfail), XmRImmediate,
 		(XtPointer) STR_AUTHFAIL,
 	},
+	{
+		XmcdNbrowserStartMsg, XmcdCBrowserStartMsg,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, str_browser_start), XmRImmediate,
+		(XtPointer) STR_BROWSER_START,
+	},
+	{
+		XmcdNbrowserFailMsg, XmcdCBrowserFailMsg,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, str_browser_fail), XmRImmediate,
+		(XtPointer) STR_BROWSER_FAIL,
+	},
+	{
+		XmcdNappDefFileMsg, XmcdCAppDefFileMsg,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, str_appdef), XmRImmediate,
+		(XtPointer) STR_APPDEF,
+	},
 
 	/* Short-cut key definitions */
 	{
-		XmcdNbuttonLabelKey, XmcdCButtonLabelKey,
+		XmcdNmodeKey, XmcdCModeKey,
 		XmRString, sizeof(String),
-		XtOffsetOf(appdata_t, btnlbl_key), XmRImmediate,
+		XtOffsetOf(appdata_t, mode_key), XmRImmediate,
 		(XtPointer) "",
 	},
 	{
@@ -1200,9 +1316,9 @@ STATIC XtResource	resources[] = {
 		(XtPointer) "",
 	},
 	{
-		XmcdNpowerOffKey, XmcdCPowerOffKey,
+		XmcdNquitKey, XmcdCQuitKey,
 		XmRString, sizeof(String),
-		XtOffsetOf(appdata_t, poweroff_key), XmRImmediate,
+		XtOffsetOf(appdata_t, quit_key), XmRImmediate,
 		(XtPointer) "",
 	},
 	{
@@ -1399,236 +1515,5 @@ STATIC XrmOptionDescRec	options[] = {
 #endif
 };
 
-
-STATIC String		fallbacks[] = {
-	"*mainForm.width: 360",
-	"*mainForm.height: 135",
-	"*keypadForm.width: 150",
-	"*keypadForm.height: 275",
-	"*optionsForm.width: 220",
-	"*optionsForm.height: 420",
-	"*dbprogForm.width: 420",
-	"*dbprogForm.height: 440",
-	"*extDiscInfoForm.width: 390",
-	"*extDiscInfoForm.height: 340",
-	"*extTrackInfoForm.width: 390",
-	"*extTrackInfoForm.height: 340",
-	"*helpForm.width: 390",
-	"*helpForm.height: 340",
-	"*dirSelectForm.width: 270",
-	"*dirSelectForm.height: 230",
-	"*linkSelectForm.width: 350",
-	"*linkSelectForm.height: 430",
-	"*authForm.width: 300",
-	"*authForm.height: 180",
-	"*mainForm.checkBox*labelType: PIXMAP",
-	"*mainForm.XmPushButton*labelType: PIXMAP",
-	"*mainForm*checkBoxFrame*fontList: -*-helvetica-medium-r-*--10-100-*",
-	"*mainForm*XmPushButton*fontList: -*-helvetica-medium-r-*--10-100-*",
-	"*discIndicator.fontList: -*-helvetica-bold-o-*--14-140-*",
-	"*trackIndicator.fontList: -*-helvetica-medium-o-*--24-240-*",
-	"*indexIndicator.fontList: -*-helvetica-bold-o-*--14-140-*",
-	"*timeIndicator.fontList: -*-helvetica-medium-o-*--24-240-*",
-	"*repeatCountIndicator.fontList: -*-helvetica-bold-o-*--12-120-*",
-	"*dbModeIndicator.fontList: -*-helvetica-bold-r-*--12-120-*",
-	"*progModeIndicator.fontList: -*-helvetica-bold-r-*--12-120-*",
-	"*timeModeIndicator.fontList: -*-helvetica-bold-r-*--12-120-*",
-	"*playModeIndicator.fontList: -*-helvetica-bold-r-*--12-120-*",
-	"*discTitleIndicator.fontList: -*-helvetica-medium-r-*--10-100-*",
-	"*trackTitleIndicator.fontList: -*-helvetica-medium-r-*--10-100-*",
-	"*aboutPopup*fontList: -*-times-bold-i-*--24-240-*=chset1, -*-times-bold-i-*--12-120-*=chset2, -*-fixed-medium-r-*--10-100-*=chset3, fixed",
-	"*totalTimeLabel.fontList: -*-helvetica-medium-r-*--10-100-*",
-	"*totalTimeIndicator.fontList: -*-helvetica-medium-r-*--10-100-*",
-	"*discIdIndicator.fontList: -*-helvetica-medium-r-*--10-100-*",
-	"*dirSelectList.fontList: -*-helvetica-medium-r-*--12-120-*",
-	"*linkSelectList*fontList: -*-helvetica-medium-r-*--12-120-*=chset1, -*-helvetica-bold-r-*--12-120-*=chset2, fixed",
-	"*extDiscInfoLabel.fontList: -*-helvetica-bold-r-*--12-120-*",
-	"*extTrackInfoLabel.fontList: -*-helvetica-bold-r-*--12-120-*",
-	"*keypadForm*keypadLabel.fontList: -*-helvetica-medium-r-*--10-100-*",
-	"*keypadForm*keypadIndicator.fontList: -*-helvetica-bold-o-*--14-140-*",
-	"*keypadForm*keypadSelectBox*fontList: -*-helvetica-medium-r-*--12-120-*",
-	"*keypadForm*trackWarpLabel.fontList: -*-helvetica-medium-r-*--10-100-*",
-	"*keypadForm*keypadCancelButton.fontList: -*-helvetica-medium-r-*--12-120-*",
-	"*keypadForm*fontList: -*-helvetica-medium-r-*--10-100-*",
-	"*optionsForm*XmFrame*fontList: -*-helvetica-medium-r-*--10-100-*",
-	"*optionsForm*XmLabel*fontList: -*-helvetica-medium-r-*--10-100-*",
-	"*optionsForm*balanceCenterButton.fontList: -*-helvetica-medium-r-*--10-100-*",
-	"*optionsForm*fontList: -*-helvetica-medium-r-*--12-120-*",
-	"*helpForm*XmPushButton*fontList: -*-helvetica-medium-r-*--12-120-*",
-	"*authForm*fontList: -*-helvetica-medium-r-*--12-120-*",
-	"*XmList.fontList: -*-helvetica-medium-r-*--12-120-*=chset1, -*-helvetica-bold-r-*--12-120-*=chset2, fixed",
-	"*XmText.fontList: -*-helvetica-medium-r-*--12-120-*",
-	"*XmScale.fontList: 6x10",
-	"*fontList: -*-helvetica-medium-r-*--12-120-*",
-	"*checkBox*button_0.labelString: btn",
-	"*checkBox*button_1.labelString: lock",
-	"*checkBox*button_2.labelString: rept",
-	"*checkBox*button_3.labelString: shuf",
-	"*ejectButton.labelString: eject",
-	"*powerOffButton.labelString: quit",
-	"*dbprogButton.labelString: cddb\nprog",
-	"*helpButton.labelString: help",
-	"*optionsButton.labelString: opt",
-	"*timeButton.labelString: time",
-	"*abButton.labelString: a->b",
-	"*sampleButton.labelString: samp",
-	"*keypadButton.labelString: kpad",
-	"*playPauseButton.labelString: play / pause",
-	"*stopButton.labelString: stop",
-	"*prevDiscButton.labelString: < disc",
-	"*nextDiscButton.labelString: disc >",
-	"*prevTrackButton.labelString: < track",
-	"*nextTrackButton.labelString: track >",
-	"*prevIndexButton.labelString: < index",
-	"*nextIndexButton.labelString: index >",
-	"*rewButton.labelString: << rew",
-	"*ffButton.labelString: ff >>",
-	"*keypadForm.dialogTitle: Keypad",
-	"*keypadLabel.labelString: Direct access",
-	"*keypadSelectBox*button_0.labelString: disc",
-	"*keypadSelectBox*button_1.labelString: track",
-	"*keypadNumButton0.labelString: 0",
-	"*keypadNumButton1.labelString: 1",
-	"*keypadNumButton2.labelString: 2",
-	"*keypadNumButton3.labelString: 3",
-	"*keypadNumButton4.labelString: 4",
-	"*keypadNumButton5.labelString: 5",
-	"*keypadNumButton6.labelString: 6",
-	"*keypadNumButton7.labelString: 7",
-	"*keypadNumButton8.labelString: 8",
-	"*keypadNumButton9.labelString: 9",
-	"*keypadEnterButton.labelString: Enter",
-	"*keypadClearButton.labelString: Clear",
-	"*trackWarpLabel.labelString: Track warp",
-	"*keypadCancelButton.labelString: Cancel",
-	"*optionsForm.dialogTitle: Options",
-	"*onLoadLabel.labelString: On Load",
-	"*onLoadCheckBox*button_0.labelString: auto lock",
-	"*onLoadRadioBox*button_0.labelString: none",
-	"*onLoadRadioBox*button_1.labelString: spin down",
-	"*onLoadRadioBox*button_2.labelString: auto play",
-	"*onExitLabel.labelString: On Exit",
-	"*onExitRadioBox*button_0.labelString: none",
-	"*onExitRadioBox*button_1.labelString: auto stop",
-	"*onExitRadioBox*button_2.labelString: auto eject",
-	"*onDoneLabel.labelString: On Done",
-	"*onDoneCheckBox*button_0.labelString: auto eject",
-	"*onEjectLabel.labelString: On Eject",
-	"*onEjectCheckBox*button_0.labelString: auto exit",
-	"*changerLabel.labelString: CD changer",
-	"*changerCheckBox*button_0.labelString: multi play",
-	"*changerCheckBox*button_1.labelString: reverse",
-	"*channelRouteLabel.labelString: Channel routing",
-	"*channelRouteRadioBox*button_0.labelString: normal",
-	"*channelRouteRadioBox*button_1.labelString: reverse",
-	"*channelRouteRadioBox*button_2.labelString: mono L",
-	"*channelRouteRadioBox*button_3.labelString: mono R",
-	"*channelRouteRadioBox*button_4.labelString: mono L+R",
-	"*volTaperLabel.labelString: Volume ctrl taper",
-	"*volTaperRadioBox*button_0.labelString: linear",
-	"*volTaperRadioBox*button_1.labelString: square",
-	"*volTaperRadioBox*button_2.labelString: inverse sqr",
-	"*balanceLabel.labelString: Balance",
-	"*balanceLeftLabel.labelString: Left",
-	"*balanceRightLabel.labelString: Right",
-	"*balanceCenterButton.labelString: Center",
-	"*resetButton.labelString: Reset",
-	"*saveButton.labelString: Save",
-	"*okButton.labelString: OK",
-	"*dbprogForm.dialogTitle: CD Database / Track Program Editor",
-	"*totalTimeLabel.labelString: Disc total:",
-	"*aboutButton.labelString: About...",
-	"*discTitleLabel.labelString: Disc artist / Title",
-	"*discLabel.labelString: Disc",
-	"*extDiscInfoButton.labelString: Ext Info...",
-	"*trackListLabel.labelString: Track / Time / Title",
-	"*timeSelectLabel.labelString: Time",
-	"*timeSelectBox*button_0.labelString: total",
-	"*timeSelectBox*button_1.labelString: track",
-	"*discIdLabel.labelString: Disc ID",
-	"*discIdIndicator.labelString: --",
-	"*trackTitleLabel.labelString: Track title edit",
-	"*trackLabel.labelString: Track",
-	"*extTrackInfoButton.labelString: Ext Info...",
-	"*programLabel.labelString: Program",
-	"*addProgramButton.labelString: Add",
-	"*clearProgramButton.labelString: Clear",
-	"*playProgramButton.labelString: Play",
-	"*programSequenceLabel.labelString: Program sequence",
-	"*sendButton.labelString: Send",
-	"*saveDatabaseButton.labelString: Save",
-	"*linkDatabaseButton.labelString: Link",
-	"*loadDatabaseButton.labelString: Load",
-	"*dbprogCancelButton.labelString: Cancel",
-	"*dirSelectForm.dialogTitle: CD Database Directory Selection",
-	"*dirSelectLabel.labelString: Choose one",
-	"*dirSelectOkButton.labelString: OK",
-	"*dirSelectCancelButton.labelString: Cancel",
-	"*linkSelectForm.dialogTitle: CDDB Link Entry Selection",
-	"*linkSelectLabel.labelString: Choose one\nAvg diff time, Disc artist / title",
-	"*linkSelectOkButton.labelString: OK",
-	"*linkSelectCancelButton.labelString: Cancel",
-	"*authForm.dialogTitle: Proxy Authorization",
-	"*authLabel.labelString: Enter your proxy user name and password:",
-	"*nameLabel.labelString: Name",
-	"*passwordLabel.labelString: Password",
-	"*authOkButton.labelString: OK",
-	"*authCancelButton.labelString: Cancel",
-	"*extDiscInfoForm.dialogTitle: Disc Information",
-	"*extDiscInfoOkButton.labelString: OK",
-	"*extDiscInfoClearButton.labelString: Clear",
-	"*extDiscInfoCancelButton.labelString: Cancel",
-	"*extTrackInfoForm.dialogTitle: Track Information",
-	"*extTrackInfoOkButton.labelString: OK",
-	"*extTrackInfoClearButton.labelString: Clear",
-	"*extTrackInfoCancelButton.labelString: Cancel",
-	"*cddbMatchPopup*dialogTitle: CD Database Server",
-	"*cddbMatchPopup*okLabelString: Yes",
-	"*cddbMatchPopup*cancelLabelString: No",
-	"*cddbMatchPopup*helpLabelString: Stop",
-	"*helpForm.dialogTitle: Xmcd Help",
-	"*helpOkButton.labelString: OK",
-	"*questionPopup*okLabelString: Yes",
-	"*questionPopup*cancelLabelString: No",
-	"*discIndicator.foreground: white",
-	"*discIndicator.background: black",
-	"*trackIndicator.foreground: white",
-	"*trackIndicator.background: black",
-	"*indexIndicator.foreground: white",
-	"*indexIndicator.background: black",
-	"*timeIndicator.foreground: white",
-	"*timeIndicator.background: black",
-	"*discTitleIndicator.foreground: black",
-	"*discTitleIndicator.background: white",
-	"*trackTitleIndicator.foreground: black",
-	"*trackTitleIndicator.background: white",
-	"*repeatCountIndicator.foreground: white",
-	"*repeatCountIndicator.background: black",
-	"*dbModeIndicator.foreground: white",
-	"*dbModeIndicator.background: black",
-	"*progModeIndicator.foreground: white",
-	"*progModeIndicator.background: black",
-	"*timeModeIndicator.foreground: white",
-	"*timeModeIndicator.background: black",
-	"*playModeIndicator.foreground: white",
-	"*playModeIndicator.background: black",
-	"*keypadIndicator.foreground: white",
-	"*keypadIndicator.background: black",
-	"*foreground: black",
-	"*background: white",
-	"*mainForm*borderWidth: 0",
-	"*keypadForm*borderWidth: 0",
-	"*optionsForm*borderWidth: 0",
-	"*dbprogForm*borderWidth: 0",
-	"*dirselForm*borderWidth: 0",
-	"*linkselForm*borderWidth: 0",
-	"*helpForm*borderWidth: 0",
-	"*borderWidth: 0",
-	"*mainForm*highlightThickness: 0",
-	"*highlightThickness: 1",
-	"*labelType: STRING",
-	NULL,
-};
-
 #endif	/* __RESOURCE_H__ */
 
